@@ -1,16 +1,16 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
- 
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
- 
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -38,16 +38,16 @@ extern bool d_gcc_is_target_win32();
 
 /* used in toobj.c */
 struct VarDeclaration;
-struct Symbol;
-struct ClassDeclaration;
 extern void d_gcc_emit_local_variable(VarDeclaration *);
 extern bool d_gcc_supports_weak();
 
 #if V2
-typedef enum {
+struct Symbol;
+enum RTLSYM
+{
     RTLSYM_DHIDDENFUNC,
     N_RTLSYM
-} RTLSYM;
+};
 extern Symbol* rtlsym[N_RTLSYM];
 #endif
 
@@ -55,6 +55,13 @@ extern Symbol* rtlsym[N_RTLSYM];
 extern bool d_gcc_force_templates();
 extern Module * d_gcc_get_output_module();
 
+#if V2
+/* used in interpret.c */
+struct FuncDeclaration;
+typedef ArrayBase<struct Expression> Expressions;
+extern Expression * d_gcc_eval_builtin(FuncDeclaration *, Expressions *);
+#endif
+
 #endif
 
 #endif
