@@ -24,6 +24,8 @@
 #include "module.h"
 #include "init.h"
 
+extern int binary(const char *p , const char **tab, int high);
+
 /**************************************
  * Hash table of array op functions already generated or known about.
  */
@@ -51,8 +53,18 @@ bool isArrayOpValid(Expression *e)
             case TOKxor:
             case TOKand:
             case TOKor:
+            case TOKassign:
+            case TOKaddass:
+            case TOKminass:
+            case TOKmulass:
+            case TOKdivass:
+            case TOKmodass:
+            case TOKxorass:
+            case TOKandass:
+            case TOKorass:
 #if DMDV2
             case TOKpow:
+            case TOKpowass:
 #endif
                  return isArrayOpValid(((BinExp *)e)->e1) && isArrayOpValid(((BinExp *)e)->e2);
 
@@ -286,18 +298,7 @@ Expression *BinExp::arrayOp(Scope *sc)
             "_arraySliceSliceMulass_w",
         };
 
-        // Search for the library function using a linear search
-        int i = -1;
-        size_t namelen = strlen(name);
-        for( int ii = 0; ii < sizeof(libArrayopFuncs) / sizeof(char *); ii++)
-        {
-            if( strlen(libArrayopFuncs[ii]) == namelen &&
-                strcmp(libArrayopFuncs[ii], name) == 0 )
-            {
-                i = 0;
-            }
-        }
-
+        int i = binary(name, libArrayopFuncs, sizeof(libArrayopFuncs) / sizeof(char *));
         if (i == -1)
         {
 #ifdef DEBUG    // Make sure our array is alphabetized
@@ -322,7 +323,7 @@ Expression *BinExp::arrayOp(Scope *sc)
             Initializer *init = new ExpInitializer(0, new IntegerExp(0, 0, Type::tsize_t));
             Dsymbol *d = new VarDeclaration(0, Type::tsize_t, Id::p, init);
             Statement *s1 = new ForStatement(0,
-                new DeclarationStatement(0, d),
+                new ExpStatement(0, d),
                 new CmpExp(TOKlt, 0, new IdentifierExp(0, Id::p), new ArrayLengthExp(0, new IdentifierExp(0, p->ident))),
                 new PostExp(TOKplusplus, 0, new IdentifierExp(0, Id::p)),
                 new ExpStatement(0, loopbody));
@@ -342,10 +343,11 @@ Expression *BinExp::arrayOp(Scope *sc)
              */
             TypeFunction *ftype = new TypeFunction(fparams, type, 0, LINKc);
             //printf("ftype: %s\n", ftype->toChars());
-            fd = new FuncDeclaration(0, 0, Lexer::idPool(name), STCundefined, ftype);
+            fd = new FuncDeclaration(loc, 0, Lexer::idPool(name), STCundefined, ftype);
             fd->fbody = fbody;
             fd->protection = PROTpublic;
             fd->linkage = LINKc;
+            fd->isArrayOp = 1;
 
             sc->module->importedFrom->members->push(fd);
 
@@ -361,7 +363,20 @@ Expression *BinExp::arrayOp(Scope *sc)
         else
         {   /* In library, refer to it.
              */
-            fd = FuncDeclaration::genCfunc(type, name, type, type, type);
+            fd = FuncDeclaration::genCfunc(type, name);
+#ifdef IN_GCC
+            /* Setup function parameters for GCC backend
+             */
+            TypeFunction * tf = (TypeFunction *) fd->type;
+            Parameters * targs = new Parameters;
+            targs->setDim(arguments->dim);
+            for (unsigned i = 0; i < arguments->dim; i++)
+            {
+                targs->data[i] = new Parameter(STCin,
+                        ((Expression *) arguments->data[i])->type, NULL, NULL);
+            }
+            tf->parameters = targs;
+#endif
         }
         sv->ptrvalue = fd;      // cache symbol in hash table
     }
@@ -606,6 +621,19 @@ int Expression::isArrayOperand()
             case TOKxor:
             case TOKand:
             case TOKor:
+            case TOKassign:
+            case TOKaddass:
+            case TOKminass:
+            case TOKmulass:
+            case TOKdivass:
+            case TOKmodass:
+            case TOKxorass:
+            case TOKandass:
+            case TOKorass:
+#if DMDV2
+            case TOKpow:
+            case TOKpowass:
+#endif
             case TOKneg:
             case TOKtilde:
                 return 1;
