@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -17,15 +17,12 @@ struct OutBuffer;
 struct Module;
 struct Scope;
 struct ScopeDsymbol;
-#ifdef _DH
+struct DebugCondition;
 #include "lexer.h" // dmdhg
-#endif
 enum TOK;
-#ifdef _DH
 struct HdrGenState;
-#endif
 
-int findCondition(Array *ids, Identifier *ident);
+int findCondition(Strings *ids, Identifier *ident);
 
 struct Condition
 {
