@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -32,6 +32,7 @@
 #include "expression.h"
 #include "aggregate.h"
 #include "declaration.h"
+#include "utf.h"
 
 #ifdef IN_GCC
 #include "d-gcc-real.h"
@@ -39,10 +40,6 @@
 
 static real_t zero;     // work around DMC bug for now
 
-#if __FreeBSD__
-#define fmodl fmod      // hack for now, fix later
-#endif
-
 #define LOG 0
 
 Expression *expType(Type *type, Expression *e)
@@ -498,22 +495,22 @@ Expression *Mod(Type *type, Expression *e1, Expression *e2)
         {   real_t r2 = e2->toReal();
 
 #ifdef __DMC__
-            c = fmodl(e1->toReal(), r2) + fmodl(e1->toImaginary(), r2) * I;
+            c = Port::fmodl(e1->toReal(), r2) + Port::fmodl(e1->toImaginary(), r2) * I;
 #elif defined(IN_GCC)
             c = complex_t(e1->toReal() % r2, e1->toImaginary() % r2);
 #else
-            c = complex_t(fmodl(e1->toReal(), r2), fmodl(e1->toImaginary(), r2));
+            c = complex_t(Port::fmodl(e1->toReal(), r2), Port::fmodl(e1->toImaginary(), r2));
 #endif
         }
         else if (e2->type->isimaginary())
         {   real_t i2 = e2->toImaginary();
 
 #ifdef __DMC__
-            c = fmodl(e1->toReal(), i2) + fmodl(e1->toImaginary(), i2) * I;
+            c = Port::fmodl(e1->toReal(), i2) + Port::fmodl(e1->toImaginary(), i2) * I;
 #elif defined(IN_GCC)
             c = complex_t(e1->toReal() % i2, e1->toImaginary() % i2);
 #else
-            c = complex_t(fmodl(e1->toReal(), i2), fmodl(e1->toImaginary(), i2));
+            c = complex_t(Port::fmodl(e1->toReal(), i2), Port::fmodl(e1->toImaginary(), i2));
 #endif
         }
         else
@@ -540,6 +537,21 @@ Expression *Mod(Type *type, Expression *e1, Expression *e2)
             e2 = new IntegerExp(loc, 1, e2->type);
             n2 = 1;
         }
+        if (n2 == -1 && !type->isunsigned())
+        {    // Check for int.min % -1
+            if (n1 == 0xFFFFFFFF80000000ULL && type->toBasetype()->ty != Tint64)
+            {
+                e2->error("integer overflow: int.min % -1");
+                e2 = new IntegerExp(loc, 1, e2->type);
+                n2 = 1;
+            }
+            else if (n1 == 0x8000000000000000LL) // long.min % -1
+            {
+                e2->error("integer overflow: long.min % -1");
+                e2 = new IntegerExp(loc, 1, e2->type);
+                n2 = 1;
+            }
+        }
         if (e1->type->isunsigned() || e2->type->isunsigned())
             n = ((d_uns64) n1) % ((d_uns64) n2);
         else
@@ -558,13 +570,13 @@ Expression *Shl(Type *type, Expression *e1, Expression *e2)
 }
 
 Expression *Shr(Type *type, Expression *e1, Expression *e2)
-{   Expression *e;
+{
     Loc loc = e1->loc;
-    unsigned count;
-    dinteger_t value;
 
-    value = e1->toInteger();
-    count = e2->toInteger();
+    dinteger_t value = e1->toInteger();
+    dinteger_t dcount = e2->toInteger();
+    assert(dcount <= 0xFFFFFFFF);
+    unsigned count = (unsigned)dcount;
     switch (e1->type->toBasetype()->ty)
     {
         case Tint8:
@@ -599,32 +611,35 @@ Expression *Shr(Type *type, Expression *e1, Expression *e2)
                 value = (d_uns64)(value) >> count;
                 break;
 
+        case Terror:
+                return e1;
+
         default:
                 assert(0);
     }
-    e = new IntegerExp(loc, value, type);
+    Expression *e = new IntegerExp(loc, value, type);
     return e;
 }
 
 Expression *Ushr(Type *type, Expression *e1, Expression *e2)
-{   Expression *e;
+{
     Loc loc = e1->loc;
-    unsigned count;
-    dinteger_t value;
 
-    value = e1->toInteger();
-    count = e2->toInteger();
+    dinteger_t value = e1->toInteger();
+    dinteger_t dcount = e2->toInteger();
+    assert(dcount <= 0xFFFFFFFF);
+    unsigned count = (unsigned)dcount;
     switch (e1->type->toBasetype()->ty)
     {
         case Tint8:
         case Tuns8:
-                assert(0);              // no way to trigger this
+                // Possible only with >>>=. >>> always gets promoted to int.
                 value = (value & 0xFF) >> count;
                 break;
 
         case Tint16:
         case Tuns16:
-                assert(0);              // no way to trigger this
+                // Possible only with >>>=. >>> always gets promoted to int.
                 value = (value & 0xFFFF) >> count;
                 break;
 
@@ -638,10 +653,13 @@ Expression *Ushr(Type *type, Expression *e1, Expression *e2)
                 value = (d_uns64)(value) >> count;
                 break;
 
+        case Terror:
+                return e1;
+
         default:
                 assert(0);
     }
-    e = new IntegerExp(loc, value, type);
+    Expression *e = new IntegerExp(loc, value, type);
     return e;
 }
 
@@ -734,8 +752,8 @@ Expression *Equal(enum TOK op, Type *type, Expression *e1, Expression *e2)
         else
         {
             for (size_t i = 0; i < es1->elements->dim; i++)
-            {   Expression *ee1 = (Expression *)es1->elements->data[i];
-                Expression *ee2 = (Expression *)es2->elements->data[i];
+            {   Expression *ee1 = (*es1->elements)[i];
+                Expression *ee2 = (*es2->elements)[i];
 
                 Expression *v = Equal(TOKequal, Type::tint32, ee1, ee2);
                 if (v == EXP_CANT_INTERPRET)
@@ -748,9 +766,9 @@ Expression *Equal(enum TOK op, Type *type, Expression *e1, Expression *e2)
     }
     else if (e1->op == TOKarrayliteral && e2->op == TOKstring)
     {   // Swap operands and use common code
-        Expression *e = e1;
+        Expression *etmp = e1;
         e1 = e2;
-        e2 = e;
+        e2 = etmp;
         goto Lsa;
     }
     else if (e1->op == TOKstring && e2->op == TOKarrayliteral)
@@ -764,10 +782,11 @@ Expression *Equal(enum TOK op, Type *type, Expression *e1, Expression *e2)
             cmp = 0;
         else
         {
+            cmp = 1;            // if dim1 winds up being 0
             for (size_t i = 0; i < dim1; i++)
             {
                 uinteger_t c = es1->charAt(i);
-                Expression *ee2 = (Expression *)es2->elements->data[i];
+                Expression *ee2 = (*es2->elements)[i];
                 if (ee2->isConst() != 1)
                     return EXP_CANT_INTERPRET;
                 cmp = (c == ee2->toInteger());
@@ -793,8 +812,8 @@ Expression *Equal(enum TOK op, Type *type, Expression *e1, Expression *e2)
         {
             cmp = 1;
             for (size_t i = 0; i < es1->elements->dim; i++)
-            {   Expression *ee1 = (Expression *)es1->elements->data[i];
-                Expression *ee2 = (Expression *)es2->elements->data[i];
+            {   Expression *ee1 = (*es1->elements)[i];
+                Expression *ee2 = (*es2->elements)[i];
 
                 if (ee1 == ee2)
                     continue;
@@ -882,11 +901,11 @@ Expression *Identity(enum TOK op, Type *type, Expression *e1, Expression *e2)
 
         cmp = (es1->var == es2->var && es1->offset == es2->offset);
     }
-    else if (e1->isConst() == 1 && e2->isConst() == 1)
+    else
+    {
         return Equal((op == TOKidentity) ? TOKequal : TOKnotequal,
                 type, e1, e2);
-    else
-        assert(0);
+    }
     if (op == TOKnotidentity)
         cmp ^= 1;
     return new IntegerExp(loc, cmp, type);
@@ -938,9 +957,8 @@ Expression *Cmp(enum TOK op, Type *type, Expression *e1, Expression *e2)
 #if IN_GCC
         if (r1.isNan() || r2.isNan())   // if unordered
 #else
-        if (isnan(r1) || isnan(r2))     // if unordered
+        if (Port::isNan(r1) || Port::isNan(r2)) // if unordered
 #endif
-        //if (Port::isNan(r1) || Port::isNan(r2))       // if unordered
         {
             switch (op)
             {
@@ -1055,7 +1073,7 @@ Expression *Cast(Type *type, Type *to, Expression *e1)
     Loc loc = e1->loc;
 
     //printf("Cast(type = %s, to = %s, e1 = %s)\n", type->toChars(), to->toChars(), e1->toChars());
-    //printf("e1->type = %s\n", e1->type->toChars());
+    //printf("\te1->type = %s\n", e1->type->toChars());
     if (type->equals(e1->type) && to->equals(type))
         return e1;
 
@@ -1074,14 +1092,10 @@ Expression *Cast(Type *type, Type *to, Expression *e1)
     }
 
     if (e1->op == TOKarrayliteral && typeb == tb)
-    {
         return e1;
-    }
 
     if (e1->isConst() != 1)
-    {
         return EXP_CANT_INTERPRET;
-        }
 
     if (tb->ty == Tbool)
         e = new IntegerExp(loc, e1->toInteger() != 0, type);
@@ -1090,7 +1104,6 @@ Expression *Cast(Type *type, Type *to, Expression *e1)
         if (e1->type->isfloating())
         {   dinteger_t result;
 #ifdef IN_GCC
-            d_int64 r;
             Type * rt = e1->type;
             if (rt->iscomplex())
             {
@@ -1101,17 +1114,13 @@ Expression *Cast(Type *type, Type *to, Expression *e1)
                     case Tcomplex80: rt = Type::tfloat80; break;
                     default: assert(0);
                 }
-                r = e1->toReal().toInt(rt, type);
-            }
-            else
-            {
-                r = e1->toInteger();
             }
+            d_int64 r = e1->toReal().toInt(rt, type);
 #else
             real_t r = e1->toReal();
 #endif
 
-            switch (type->toBasetype()->ty)
+            switch (typeb->ty)
             {
                 case Tint8:     result = (d_int8)r;     break;
                 case Tchar:
@@ -1160,7 +1169,7 @@ Expression *Cast(Type *type, Type *to, Expression *e1)
         assert(sd);
         Expressions *elements = new Expressions;
         for (size_t i = 0; i < sd->fields.dim; i++)
-        {   Dsymbol *s = (Dsymbol *)sd->fields.data[i];
+        {   Dsymbol *s = sd->fields.tdata()[i];
             VarDeclaration *v = s->isVarDeclaration();
             assert(v);
 
@@ -1225,8 +1234,8 @@ Expression *Index(Type *type, Expression *e1, Expression *e2)
         if (i >= es1->len)
             e1->error("string index %"PRIuMAX" is out of bounds [0 .. %"PRIuSIZE"]", i, es1->len);
         else
-        {   unsigned value = es1->charAt(i);
-            e = new IntegerExp(loc, value, type);
+        {
+            e = new IntegerExp(loc, es1->charAt(i), type);
         }
     }
     else if (e1->type->toBasetype()->ty == Tsarray && e2->op == TOKint64)
@@ -1235,30 +1244,34 @@ Expression *Index(Type *type, Expression *e1, Expression *e2)
         uinteger_t i = e2->toInteger();
 
         if (i >= length)
-        {   e2->error("array index %"PRIuMAX" is out of bounds %s[0 .. %"PRIuMAX"]", i, e1->toChars(), length);
+        {   e1->error("array index %"PRIuMAX" is out of bounds %s[0 .. %"PRIuMAX"]", i, e1->toChars(), length);
         }
-        else if (e1->op == TOKarrayliteral && !e1->checkSideEffect(2))
+        else if (e1->op == TOKarrayliteral)
         {   ArrayLiteralExp *ale = (ArrayLiteralExp *)e1;
-            e = (Expression *)ale->elements->data[i];
+            e = ale->elements->tdata()[i];
             e->type = type;
+            if (e->checkSideEffect(2))
+                e = EXP_CANT_INTERPRET;
         }
     }
     else if (e1->type->toBasetype()->ty == Tarray && e2->op == TOKint64)
     {
         uinteger_t i = e2->toInteger();
 
-        if (e1->op == TOKarrayliteral && !e1->checkSideEffect(2))
+        if (e1->op == TOKarrayliteral)
         {   ArrayLiteralExp *ale = (ArrayLiteralExp *)e1;
             if (i >= ale->elements->dim)
-            {   e2->error("array index %ju is out of bounds %s[0 .. %u]", i, e1->toChars(), ale->elements->dim);
+            {   e1->error("array index %"PRIuMAX" is out of bounds %s[0 .. %u]", i, e1->toChars(), ale->elements->dim);
             }
             else
-            {   e = (Expression *)ale->elements->data[i];
+            {   e = ale->elements->tdata()[i];
                 e->type = type;
+                if (e->checkSideEffect(2))
+                    e = EXP_CANT_INTERPRET;
             }
         }
     }
-    else if (e1->op == TOKassocarrayliteral && !e1->checkSideEffect(2))
+    else if (e1->op == TOKassocarrayliteral)
     {
         AssocArrayLiteralExp *ae = (AssocArrayLiteralExp *)e1;
         /* Search the keys backwards, in case there are duplicate keys
@@ -1266,13 +1279,15 @@ Expression *Index(Type *type, Expression *e1, Expression *e2)
         for (size_t i = ae->keys->dim; i;)
         {
             i--;
-            Expression *ekey = (Expression *)ae->keys->data[i];
+            Expression *ekey = ae->keys->tdata()[i];
             Expression *ex = Equal(TOKequal, Type::tbool, ekey, e2);
             if (ex == EXP_CANT_INTERPRET)
                 return ex;
             if (ex->isBool(TRUE))
-            {   e = (Expression *)ae->values->data[i];
+            {   e = ae->values->tdata()[i];
                 e->type = type;
+                if (e->checkSideEffect(2))
+                    e = EXP_CANT_INTERPRET;
                 break;
             }
         }
@@ -1302,7 +1317,7 @@ Expression *Slice(Type *type, Expression *e1, Expression *lwr, Expression *upr)
         if (iupr > es1->len || ilwr > iupr)
             e1->error("string slice [%"PRIuMAX" .. %"PRIuMAX"] is out of bounds", ilwr, iupr);
         else
-        {   dinteger_t value;
+        {
             void *s;
             size_t len = iupr - ilwr;
             int sz = es1->sz;
@@ -1332,9 +1347,9 @@ Expression *Slice(Type *type, Expression *e1, Expression *lwr, Expression *upr)
         {
             Expressions *elements = new Expressions();
             elements->setDim(iupr - ilwr);
-            memcpy(elements->data,
-                   es1->elements->data + ilwr,
-                   (iupr - ilwr) * sizeof(es1->elements->data[0]));
+            memcpy(elements->tdata(),
+                   es1->elements->tdata() + ilwr,
+                   (iupr - ilwr) * sizeof(es1->elements->tdata()[0]));
             e = new ArrayLiteralExp(e1->loc, elements);
             e->type = type;
         }
@@ -1352,14 +1367,16 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
     Type *t2 = e2->type->toBasetype();
 
     //printf("Cat(e1 = %s, e2 = %s)\n", e1->toChars(), e2->toChars());
-    //printf("\tt1 = %s, t2 = %s\n", t1->toChars(), t2->toChars())
+    //printf("\tt1 = %s, t2 = %s, type = %s\n", t1->toChars(), t2->toChars(), type->toChars());
 
     if (e1->op == TOKnull && (e2->op == TOKint64 || e2->op == TOKstructliteral))
     {   e = e2;
+        t = t1;
         goto L2;
     }
     else if ((e1->op == TOKint64 || e1->op == TOKstructliteral) && e2->op == TOKnull)
     {   e = e1;
+        t = t2;
      L2:
         Type *tn = e->type->toBasetype();
         if (tn->ty == Tchar || tn->ty == Twchar || tn->ty == Tdchar)
@@ -1367,18 +1384,18 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
             // Create a StringExp
             void *s;
             StringExp *es;
-            size_t len = 1;
-            int sz = tn->size();
+            if (t->nextOf())
+                t = t->nextOf()->toBasetype();
+            int sz = t->size();
+
             dinteger_t v = e->toInteger();
 
+            size_t len = (t->ty == tn->ty) ? 1 : utf_codeLength(sz, v);
             s = mem.malloc((len + 1) * sz);
-            switch (sz)
-            {
-                case 1: *(d_uns8*)s = v; break;
-                case 2: *(d_uns16*)s = v; break;
-                case 4: *(d_uns32*)s = v; break;
-                default: assert(0);
-            }
+            if (t->ty == tn->ty)
+                memcpy((unsigned char *)s, &v, sz);
+            else
+                utf_encode(sz, s, v);
 
             // Add terminating 0
             memset((unsigned char *)s + len * sz, 0, sz);
@@ -1397,6 +1414,14 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
         e->type = type;
         return e;
     }
+    else if (e1->op == TOKnull && e2->op == TOKnull)
+    {
+        if (type == e1->type)
+            return e1;
+        if (type == e2->type)
+            return e2;
+        return new NullExp(e1->loc, type);
+    }
     else if (e1->op == TOKstring && e2->op == TOKstring)
     {
         // Concatenate the strings
@@ -1404,7 +1429,6 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
         StringExp *es1 = (StringExp *)e1;
         StringExp *es2 = (StringExp *)e2;
         StringExp *es;
-        Type *t;
         size_t len = es1->len + es2->len;
         int sz = es1->sz;
 
@@ -1426,10 +1450,62 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
         es = new StringExp(loc, s, len);
         es->sz = sz;
         es->committed = es1->committed | es2->committed;
-        if (es1->committed)
-            t = es1->type;
-        else
-            t = es2->type;
+        es->type = type;
+        e = es;
+    }
+    else if (e2->op == TOKstring && e1->op == TOKarrayliteral &&
+        t1->nextOf()->isintegral())
+    {
+        // Concatenate the strings
+        StringExp *es1 = (StringExp *)e2;
+        ArrayLiteralExp *es2 = (ArrayLiteralExp *)e1;
+        size_t len = es1->len + es2->elements->dim;
+        int sz = es1->sz;
+
+        void *s = mem.malloc((len + 1) * sz);
+        memcpy((char *)s + sz * es2->elements->dim, es1->string, es1->len * sz);
+        for (size_t i = 0; i < es2->elements->dim; i++)
+        {   Expression *es2e = es2->elements->tdata()[i];
+            if (es2e->op != TOKint64)
+                return EXP_CANT_INTERPRET;
+            dinteger_t v = es2e->toInteger();
+            memcpy((unsigned char *)s + i * sz, &v, sz);
+        }
+
+        // Add terminating 0
+        memset((unsigned char *)s + len * sz, 0, sz);
+
+        StringExp *es = new StringExp(loc, s, len);
+        es->sz = sz;
+        es->committed = 0;
+        es->type = type;
+        e = es;
+    }
+    else if (e1->op == TOKstring && e2->op == TOKarrayliteral &&
+        t2->nextOf()->isintegral())
+    {
+        // Concatenate the strings
+        StringExp *es1 = (StringExp *)e1;
+        ArrayLiteralExp *es2 = (ArrayLiteralExp *)e2;
+        size_t len = es1->len + es2->elements->dim;
+        int sz = es1->sz;
+
+        void *s = mem.malloc((len + 1) * sz);
+        memcpy(s, es1->string, es1->len * sz);
+        for (size_t i = 0; i < es2->elements->dim; i++)
+        {   Expression *es2e = es2->elements->tdata()[i];
+            if (es2e->op != TOKint64)
+                return EXP_CANT_INTERPRET;
+            dinteger_t v = es2e->toInteger();
+            memcpy((unsigned char *)s + (es1->len + i) * sz, &v, sz);
+        }
+
+        // Add terminating 0
+        memset((unsigned char *)s + len * sz, 0, sz);
+
+        StringExp *es = new StringExp(loc, s, len);
+        es->sz = sz;
+        es->committed = 0; //es1->committed;
         es->type = type;
         e = es;
     }
@@ -1437,24 +1513,23 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
     {
         // Concatenate the strings
         void *s;
-        void *sch;
         StringExp *es1 = (StringExp *)e1;
         StringExp *es;
-        Type *t;
-        size_t len = es1->len + 1;
         int sz = es1->sz;
         dinteger_t v = e2->toInteger();
 
+        // Is it a concatentation of homogenous types?
+        // (char[] ~ char, wchar[]~wchar, or dchar[]~dchar)
+        bool homoConcat = (sz == t2->size());
+        size_t len = es1->len;
+        len += homoConcat ? 1 : utf_codeLength(sz, v);
+
         s = mem.malloc((len + 1) * sz);
         memcpy(s, es1->string, es1->len * sz);
-        sch = (unsigned char *)s + es1->len * sz;
-        switch (sz)
-        {
-            case 1: *(d_uns8*)sch = v; break;
-            case 2: *(d_uns16*)sch = v; break;
-            case 4: *(d_uns32*)sch = v; break;
-            default: assert(0);
-        }
+        if (homoConcat)
+             memcpy((unsigned char *)s + (sz * es1->len), &v, sz);
+        else
+             utf_encode(sz, (unsigned char *)s + (sz * es1->len), v);
 
         // Add terminating 0
         memset((unsigned char *)s + len * sz, 0, sz);
@@ -1462,7 +1537,6 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
         es = new StringExp(loc, s, len);
         es->sz = sz;
         es->committed = es1->committed;
-        t = es1->type;
         es->type = type;
         e = es;
     }
@@ -1472,19 +1546,12 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
         void *s;
         StringExp *es2 = (StringExp *)e2;
         StringExp *es;
-        Type *t;
         size_t len = 1 + es2->len;
         int sz = es2->sz;
         dinteger_t v = e1->toInteger();
 
         s = mem.malloc((len + 1) * sz);
-        switch (sz)
-        {
-            case 1: *(d_uns8*)s = v; break;
-            case 2: *(d_uns16*)s = v; break;
-            case 4: *(d_uns32*)s = v; break;
-            default: assert(0);
-        }
+        memcpy((unsigned char *)s, &v, sz);
         memcpy((unsigned char *)s + sz, es2->string, es2->len * sz);
 
         // Add terminating 0
@@ -1493,7 +1560,6 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
         es = new StringExp(loc, s, len);
         es->sz = sz;
         es->committed = es2->committed;
-        t = es2->type;
         es->type = type;
         e = es;
     }
@@ -1510,7 +1576,7 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
 
         if (type->toBasetype()->ty == Tsarray)
         {
-            e->type = new TypeSArray(e1->type->toBasetype()->next, new IntegerExp(0, es1->elements->dim, Type::tindex));
+            e->type = new TypeSArray(t1->nextOf(), new IntegerExp(loc, es1->elements->dim, Type::tindex));
             e->type = e->type->semantic(loc, NULL);
         }
         else
@@ -1518,15 +1584,15 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
     }
     else if (e1->op == TOKarrayliteral && e2->op == TOKnull &&
         t1->nextOf()->equals(t2->nextOf()))
-     {
+    {
         e = e1;
         goto L3;
-     }
-     else if (e1->op == TOKnull && e2->op == TOKarrayliteral &&
+    }
+    else if (e1->op == TOKnull && e2->op == TOKarrayliteral &&
         t1->nextOf()->equals(t2->nextOf()))
-     {
+    {
         e = e2;
-      L3:
+     L3:
         // Concatenate the array with null
         ArrayLiteralExp *es = (ArrayLiteralExp *)e;
 
@@ -1540,16 +1606,15 @@ Expression *Cat(Type *type, Expression *e1, Expression *e2)
         }
         else
             e->type = type;
-     }
+    }
     else if ((e1->op == TOKarrayliteral || e1->op == TOKnull) &&
         e1->type->toBasetype()->nextOf()->equals(e2->type))
     {
         ArrayLiteralExp *es1;
         if (e1->op == TOKarrayliteral)
         {   es1 = (ArrayLiteralExp *)e1;
-
-        es1 = new ArrayLiteralExp(es1->loc, (Expressions *)es1->elements->copy());
-        es1->elements->push(e2);
+            es1 = new ArrayLiteralExp(es1->loc, (Expressions *)es1->elements->copy());
+            es1->elements->push(e2);
         }
         else
         {
