@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -40,7 +40,9 @@ Declaration::Declaration(Identifier *id)
     storage_class = STCundefined;
     protection = PROTundefined;
     linkage = LINKdefault;
+#if IN_GCC
     attributes = NULL;
+#endif
     inuse = 0;
 }
 
@@ -53,7 +55,7 @@ const char *Declaration::kind()
     return "declaration";
 }
 
-target_size_t Declaration::size(Loc loc)
+unsigned Declaration::size(Loc loc)
 {
     assert(type);
     return type->size();
@@ -119,7 +121,7 @@ void Declaration::checkModify(Loc loc, Scope *sc, Type *t)
             if (fd &&
                 ((fd->isCtorDeclaration() && storage_class & STCfield) ||
                  (fd->isStaticCtorDeclaration() && !(storage_class & STCfield))) &&
-                fd->toParent() == toParent()
+                fd->toParent2() == toParent()
                )
             {
                 VarDeclaration *v = isVarDeclaration();
@@ -166,12 +168,12 @@ void Declaration::checkModify(Loc loc, Scope *sc, Type *t)
 #endif
 
 
-
 /********************************* TupleDeclaration ****************************/
 
 TupleDeclaration::TupleDeclaration(Loc loc, Identifier *id, Objects *objects)
     : Declaration(id)
 {
+    this->loc = loc;
     this->type = NULL;
     this->objects = objects;
     this->isexp = 0;
@@ -323,13 +325,15 @@ void TypedefDeclaration::semantic(Scope *sc)
         type = type->addStorageClass(storage_class);
 #endif
         type = type->semantic(loc, sc);
+#if IN_GCC
         if (attributes)
             attributes->append(sc->attributes);
         else
             attributes = sc->attributes;
+#endif
         if (sc->parent->isFuncDeclaration() && init)
             semantic2(sc);
-            storage_class |= sc->stc & STCdeprecated;
+        storage_class |= sc->stc & STCdeprecated;
     }
     else if (sem == 1)
     {
@@ -344,7 +348,7 @@ void TypedefDeclaration::semantic2(Scope *sc)
     {   sem = 3;
         if (init)
         {
-            init = init->semantic(sc, basetype);
+            init = init->semantic(sc, basetype, WANTinterpret);
 
             ExpInitializer *ie = init->isExpInitializer();
             if (ie)
@@ -417,7 +421,7 @@ AliasDeclaration::AliasDeclaration(Loc loc, Identifier *id, Dsymbol *s)
 
 Dsymbol *AliasDeclaration::syntaxCopy(Dsymbol *s)
 {
-        //printf("AliasDeclaration::syntaxCopy()\n");
+    //printf("AliasDeclaration::syntaxCopy()\n");
     assert(!s);
     AliasDeclaration *sa;
     if (type)
@@ -502,7 +506,7 @@ void AliasDeclaration::semantic(Scope *sc)
     }
     else
 #endif
-    type->resolve(loc, sc, &e, &t, &s);
+        type->resolve(loc, sc, &e, &t, &s);
     if (s)
     {
         goto L2;
@@ -526,7 +530,7 @@ void AliasDeclaration::semantic(Scope *sc)
     else if (t)
     {
         type = t;
-         }
+    }
     if (overnext)
         ScopeDsymbol::multiplyDefined(0, this, overnext);
     this->inSemantic = 0;
@@ -557,15 +561,17 @@ void AliasDeclaration::semantic(Scope *sc)
             }
         }
         if (overnext)
-            ScopeDsymbol::multiplyDefined(0, s, overnext);
+            ScopeDsymbol::multiplyDefined(0, this, overnext);
         if (s == this)
         {
             assert(global.errors);
             s = NULL;
         }
     }
-    //printf("setting aliassym %s to %s %s\n", toChars(), s->kind(), s->toChars());
-    aliassym = s;
+    if (!type || type->ty != Terror)
+    {   //printf("setting aliassym %s to %s %s\n", toChars(), s->kind(), s->toChars());
+        aliassym = s;
+    }
     this->inSemantic = 0;
 }
 
@@ -610,7 +616,7 @@ const char *AliasDeclaration::kind()
 
 Type *AliasDeclaration::getType()
 {
-        //printf("AliasDeclaration::getType() %s\n", type->toChars());
+    //printf("AliasDeclaration::getType() %s\n", type->toChars());
 #if 0
     if (!type->deco && scope)
         semantic(scope);
@@ -628,6 +634,7 @@ Dsymbol *AliasDeclaration::toAlias()
     if (inSemantic)
     {   error("recursive alias declaration");
         aliassym = new TypedefDeclaration(loc, ident, Type::terror, NULL);
+        type = Type::terror;
     }
     else if (!aliassym && scope)
         semantic(scope);
@@ -687,7 +694,7 @@ VarDeclaration::VarDeclaration(Loc loc, Type *type, Identifier *id, Initializer
 #endif
     this->loc = loc;
     offset = 0;
-    noauto = 0;
+    noscope = 0;
 #if DMDV2
     isargptr = FALSE;
 #endif
@@ -698,7 +705,7 @@ VarDeclaration::VarDeclaration(Loc loc, Type *type, Identifier *id, Initializer
     aliassym = NULL;
     onstack = 0;
     canassign = 0;
-    value = NULL;
+    setValueNull();
 }
 
 Dsymbol *VarDeclaration::syntaxCopy(Dsymbol *s)
@@ -754,16 +761,26 @@ void VarDeclaration::semantic(Scope *sc)
     //if (strcmp(toChars(), "mul") == 0) halt();
 #endif
 
+    if (scope)
+    {   sc = scope;
+        scope = NULL;
+    }
+
     storage_class |= sc->stc;
     if (storage_class & STCextern && init)
         error("extern symbols cannot have initializers");
 
+    AggregateDeclaration *ad = isThis();
+    if (ad)
+        storage_class |= ad->storage_class & STC_TYPECTOR;
+
     /* If auto type inference, do the inference
      */
     int inferred = 0;
     if (!type)
     {   inuse++;
         type = init->inferType(sc);
+        type = type->semantic(loc, sc);
         inuse--;
         inferred = 1;
 
@@ -785,10 +802,12 @@ void VarDeclaration::semantic(Scope *sc)
     this->parent = sc->parent;
     //printf("this = %p, parent = %p, '%s'\n", this, parent, parent->toChars());
     protection = sc->protection;
+#if IN_GCC
     if (attributes)
         attributes->append(sc->attributes);
     else
         attributes = sc->attributes;
+#endif
     //printf("sc->stc = %x\n", sc->stc);
     //printf("storage_class = x%x\n", storage_class);
 
@@ -821,6 +840,8 @@ void VarDeclaration::semantic(Scope *sc)
             error("no definition of struct %s", ts->toChars());
         }
     }
+    if ((storage_class & STCauto) && !inferred)
+       error("storage class 'auto' has no effect if type is not inferred, did you mean 'scope'?");
 
     if (tb->ty == Ttuple)
     {   /* Instead, declare variables for each of the tuple elements
@@ -914,7 +935,7 @@ void VarDeclaration::semantic(Scope *sc)
             }
             else
 #endif
-            aad->addField(sc, this);
+                aad->addField(sc, this);
         }
 
         InterfaceDeclaration *id = parent->isInterfaceDeclaration();
@@ -922,7 +943,8 @@ void VarDeclaration::semantic(Scope *sc)
         {
             error("field not allowed in interface");
         }
-/* Templates cannot add fields to aggregates
+
+        /* Templates cannot add fields to aggregates
          */
         TemplateInstance *ti = parent->isTemplateInstance();
         if (ti)
@@ -937,10 +959,10 @@ void VarDeclaration::semantic(Scope *sc)
             }
 
             // If it's a member template
-            AggregateDeclaration *ad = ti->tempdecl->isMember();
-            if (ad && storage_class != STCundefined)
+            AggregateDeclaration *ad2 = ti->tempdecl->isMember();
+            if (ad2 && storage_class != STCundefined)
             {
-                error("cannot use template to add field to aggregate '%s'", ad->toChars());
+                error("cannot use template to add field to aggregate '%s'", ad2->toChars());
             }
         }
     }
@@ -953,14 +975,14 @@ void VarDeclaration::semantic(Scope *sc)
     }
 #endif
 
-    if (type->isauto() && !noauto)
+    if (type->isscope() && !noscope)
     {
         if (storage_class & (STCfield | STCout | STCref | STCstatic) || !fd)
         {
             error("globals, statics, fields, ref and out parameters cannot be auto");
         }
 
-        if (!(storage_class & (STCauto | STCscope)))
+        if (!(storage_class & STCscope))
         {
             if (!(storage_class & STCparameter) && ident != Id::withSym)
                 error("reference to scope class must be scope");
@@ -1014,7 +1036,8 @@ void VarDeclaration::semantic(Scope *sc)
     if (init)
     {
         sc = sc->push();
-    sc->stc &= ~(STC_TYPECTOR | STCpure | STCnothrow | STCref);
+        sc->stc &= ~(STC_TYPECTOR | STCpure | STCnothrow | STCref);
+
         ArrayInitializer *ai = init->isArrayInitializer();
         if (ai && tb->ty == Taarray)
         {
@@ -1042,14 +1065,13 @@ void VarDeclaration::semantic(Scope *sc)
             // possibilities.
             if (fd && !isStatic() && !isConst() && !init->isVoidInitializer())
             {
-
                 //printf("fd = '%s', var = '%s'\n", fd->toChars(), toChars());
                 if (!ei)
                 {
                     Expression *e = init->toExpression();
                     if (!e)
                     {
-                        init = init->semantic(sc, type);
+                        init = init->semantic(sc, type, 0); // Don't need to interpret
                         e = init->toExpression();
                         if (!e)
                         {   error("is not a static and cannot have static initializer");
@@ -1068,7 +1090,7 @@ void VarDeclaration::semantic(Scope *sc)
                     ei->exp = ei->exp->semantic(sc);
                     if (!ei->exp->implicitConvTo(type))
                     {
-                        sinteger_t dim = ((TypeSArray *)t)->dim->toInteger();
+                        dinteger_t dim = ((TypeSArray *)t)->dim->toInteger();
                         // If multidimensional static array, treat as one large array
                         while (1)
                         {
@@ -1150,7 +1172,7 @@ void VarDeclaration::semantic(Scope *sc)
             }
             else
             {
-                init = init->semantic(sc, type);
+                init = init->semantic(sc, type, WANTinterpret);
                 if (fd && isConst() && !isStatic())
                 {   // Make it static
                     storage_class |= STCstatic;
@@ -1182,7 +1204,7 @@ void VarDeclaration::semantic(Scope *sc)
                 }
                 else if (si || ai)
                 {   i2 = init->syntaxCopy();
-                    i2 = i2->semantic(sc, type);
+                    i2 = i2->semantic(sc, type, WANTinterpret);
                 }
                 inuse--;
                 global.gag--;
@@ -1200,20 +1222,31 @@ void VarDeclaration::semantic(Scope *sc)
                 }
                 else if (ei)
                 {
-                    e = e->optimize(WANTvalue | WANTinterpret);
-                    if (e->op == TOKint64 || e->op == TOKstring || e->op == TOKfloat64)
-                    {
-                        ei->exp = e;            // no errors, keep result
-                    }
-#if DMDV2
+                    if (isDataseg() || (storage_class & STCmanifest))
+                        e = e->optimize(WANTvalue | WANTinterpret);
                     else
+                        e = e->optimize(WANTvalue);
+                    switch (e->op)
                     {
-                        /* Save scope for later use, to try again
-                         */
-                        scope = new Scope(*sc);
-                        scope->setNoFree();
-                    }
+                        case TOKint64:
+                        case TOKfloat64:
+                        case TOKstring:
+                        case TOKarrayliteral:
+                        case TOKassocarrayliteral:
+                        case TOKstructliteral:
+                        case TOKnull:
+                            ei->exp = e;            // no errors, keep result
+                            break;
+
+                        default:
+#if DMDV2
+                            /* Save scope for later use, to try again
+                             */
+                            scope = new Scope(*sc);
+                            scope->setNoFree();
 #endif
+                            break;
+                    }
                 }
                 else
                     init = i2;          // no errors, keep result
@@ -1253,7 +1286,7 @@ void VarDeclaration::semantic2(Scope *sc)
             printf("type = %p\n", ei->exp->type);
         }
 #endif
-        init = init->semantic(sc, type);
+        init = init->semantic(sc, type, WANTinterpret);
         inuse--;
     }
 }
@@ -1290,12 +1323,33 @@ void VarDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
             ((AssignExp *)ie->exp)->e2->toCBuffer(buf, hgs);
         else
 #endif
-        init->toCBuffer(buf, hgs);
+            init->toCBuffer(buf, hgs);
     }
     buf->writeByte(';');
     buf->writenl();
 }
 
+AggregateDeclaration *VarDeclaration::isThis()
+{
+    AggregateDeclaration *ad = NULL;
+
+    if (!(storage_class & (STCstatic | STCextern | STCmanifest | STCtemplateparameter |
+                           STCtls | STCgshared | STCctfe)))
+    {
+        if ((storage_class & (STCconst | STCimmutable)) && init)
+            return NULL;
+
+        for (Dsymbol *s = this; s; s = s->parent)
+        {
+            ad = s->isMember();
+            if (ad)
+                break;
+            if (!s->parent || !s->parent->isTemplateMixin()) break;
+        }
+    }
+    return ad;
+}
+
 int VarDeclaration::needThis()
 {
     //printf("VarDeclaration::needThis(%s, x%x)\n", toChars(), storage_class);
@@ -1322,7 +1376,7 @@ void VarDeclaration::checkCtorConstInit()
 
 void VarDeclaration::checkNestedReference(Scope *sc, Loc loc)
 {
-        //printf("VarDeclaration::checkNestedReference() %s\n", toChars());
+    //printf("VarDeclaration::checkNestedReference() %s\n", toChars());
     if (parent && !isDataseg() && parent != sc->parent)
     {
         // The function that this variable is in
@@ -1334,6 +1388,25 @@ void VarDeclaration::checkNestedReference(Scope *sc, Loc loc)
         {
             if (loc.filename)
                 fdthis->getLevel(loc, fdv);
+#if IN_GCC
+            for (int i = 0; i < nestedrefs.dim; i++)
+            {   FuncDeclaration *f = (FuncDeclaration *)nestedrefs.data[i];
+                if (f == fdthis)
+                    goto L1;
+            }
+            nestedrefs.push(fdthis);
+          L1: ;
+
+
+            for (int i = 0; i < fdv->frameVars.dim; i++)
+            {   Dsymbol *s = (Dsymbol *)fdv->frameVars.data[i];
+                if (s == this)
+                    goto L2;
+            }
+
+            fdv->frameVars.push(this);
+          L2: ;
+#endif
             nestedref = 1;
             fdv->nestedFrameRef = 1;
             //printf("var %s in function %s is nested ref\n", toChars(), fdv->toChars());
@@ -1389,20 +1462,20 @@ int VarDeclaration::isCTFE()
 
 int VarDeclaration::hasPointers()
 {
-        //printf("VarDeclaration::hasPointers() %s, ty = %d\n", toChars(), type->ty);
+    //printf("VarDeclaration::hasPointers() %s, ty = %d\n", toChars(), type->ty);
     return (!isDataseg() && type->hasPointers());
 }
 
 /******************************************
- * If a variable has an auto destructor call, return call for it.
+ * If a variable has a scope destructor call, return call for it.
  * Otherwise, return NULL.
  */
 
-Expression *VarDeclaration::callAutoDtor(Scope *sc)
+Expression *VarDeclaration::callScopeDtor(Scope *sc)
 {   Expression *e = NULL;
 
-    //printf("VarDeclaration::callAutoDtor() %s\n", toChars());
-    if (storage_class & (STCauto | STCscope) && !noauto)
+    //printf("VarDeclaration::callScopeDtor() %s\n", toChars());
+    if (storage_class & (STCauto | STCscope) && !noscope)
     {
         for (ClassDeclaration *cd = type->isClassHandle();
              cd;
@@ -1429,6 +1502,15 @@ Expression *VarDeclaration::callAutoDtor(Scope *sc)
     return e;
 }
 
+/******************************************
+ */
+
+void ObjectNotFound(Identifier *id)
+{
+    Type::error(0, "%s not found. object.d may be incorrectly installed or corrupt.", id->toChars());
+    fatal();
+}
+
 
 /********************************* ClassInfoDeclaration ****************************/
 
@@ -1606,9 +1688,9 @@ TypeInfoTupleDeclaration::TypeInfoTupleDeclaration(Type *tinfo)
 // For the "this" parameter to member functions
 
 ThisDeclaration::ThisDeclaration(Loc loc, Type *t)
-    : VarDeclaration(loc, t, Id::This, NULL)
+   : VarDeclaration(loc, t, Id::This, NULL)
 {
-    noauto = 1;
+    noscope = 1;
 }
 
 Dsymbol *ThisDeclaration::syntaxCopy(Dsymbol *s)
@@ -1616,3 +1698,4 @@ Dsymbol *ThisDeclaration::syntaxCopy(Dsymbol *s)
     assert(0);          // should never be produced by syntax
     return NULL;
 }
+
