@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -89,6 +89,8 @@ enum STC
 #define STCtrusted      0x400000000LL
 #define STCsystem       0x800000000LL
 #define STCctfe         0x1000000000LL  // can be used in CTFE, even if it is static
+#define STCdisable      0x2000000000LL  // for functions that are not callable
+#define STCresult       0x4000000000LL  // for result variables passed to out contracts
 
 struct Match
 {
@@ -123,12 +125,14 @@ struct Declaration : Dsymbol
     enum PROT protection;
     enum LINK linkage;
     int inuse;                  // used to detect cycles
-    Expressions * attributes; // GCC decl/type attributes
+#if IN_GCC
+    Expressions * attributes;   // GCC decl/type attributes
+#endif
 
     Declaration(Identifier *id);
     void semantic(Scope *sc);
     const char *kind();
-    target_size_t size(Loc loc);
+    unsigned size(Loc loc);
     void checkModify(Loc loc, Scope *sc, Type *t);
 
     void emitComment(Scope *sc);
@@ -149,7 +153,7 @@ struct Declaration : Dsymbol
     int isConst()        { return storage_class & STCconst; }
     int isImmutable()    { return storage_class & STCimmutable; }
     int isAuto()         { return storage_class & STCauto; }
-    int isScope()        { return storage_class & (STCscope | STCauto); }
+    int isScope()        { return storage_class & STCscope; }
     int isSynchronized() { return storage_class & STCsynchronized; }
     int isParameter()    { return storage_class & STCparameter; }
     int isDeprecated()   { return storage_class & STCdeprecated; }
@@ -201,10 +205,8 @@ struct TypedefDeclaration : Declaration
     const char *kind();
     Type *getType();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-#ifdef _DH
     Type *htype;
     Type *hbasetype;
-#endif
 
     void toDocBuffer(OutBuffer *buf);
 
@@ -235,10 +237,8 @@ struct AliasDeclaration : Declaration
     Type *getType();
     Dsymbol *toAlias();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-#ifdef _DH
     Type *htype;
     Dsymbol *haliassym;
-#endif
 
     void toDocBuffer(OutBuffer *buf);
 
@@ -250,21 +250,34 @@ struct AliasDeclaration : Declaration
 struct VarDeclaration : Declaration
 {
     Initializer *init;
-    target_size_t offset;
-    int noauto;                 // no auto semantics
+    unsigned offset;
+    int noscope;                 // no auto semantics
 #if DMDV2
     FuncDeclarations nestedrefs; // referenced by these lexically nested functions
     bool isargptr;              // if parameter that _argptr points to
 #else
     int nestedref;              // referenced by a lexically nested function
+#if IN_GCC
+    FuncDeclarations nestedrefs; // referenced by these lexically nested functions
+#endif
 #endif
     int ctorinit;               // it has been initialized in a ctor
     int onstack;                // 1: it has been allocated on the stack
                                 // 2: on stack, run destructor anyway
     int canassign;              // it can be assigned to
     Dsymbol *aliassym;          // if redone as alias to another symbol
-    Expression *value;          // when interpreting, this is the value
-                                // (NULL if value not determinable)
+
+    // When interpreting, these hold the value (NULL if value not determinable)
+    // The various functions are used only to detect compiler CTFE bugs
+    Expression *literalvalue;
+    Expression *getValue() { return literalvalue; }
+    void setValueNull();
+    void setValueWithoutChecking(Expression *newval);
+    void createRefValue(Expression *newval); // struct or array literal
+    void setRefValue(Expression *newval);
+    void setStackValue(Expression *newval);
+    void createStackValue(Expression *newval);
+
 #if DMDV2
     VarDeclaration *rundtor;    // if !NULL, rundtor is tested at runtime to see
                                 // if the destructor should be run. Used to prevent
@@ -277,10 +290,9 @@ struct VarDeclaration : Declaration
     void semantic2(Scope *sc);
     const char *kind();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-#ifdef _DH
     Type *htype;
     Initializer *hinit;
-#endif
+    AggregateDeclaration *isThis();
     int needThis();
     int isImportedSymbol();
     int isDataseg();
@@ -291,7 +303,7 @@ struct VarDeclaration : Declaration
     int canTakeAddressOf();
     int needsAutoDtor();
 #endif
-    Expression *callAutoDtor(Scope *sc);
+    Expression *callScopeDtor(Scope *sc);
     ExpInitializer *getExpInitializer();
     Expression *getConstInitializer();
     void checkCtorConstInit();
@@ -519,7 +531,7 @@ enum BUILTIN { };
 
 struct FuncDeclaration : Declaration
 {
-    Array *fthrows;                     // Array of Type's of exceptions (not used)
+    Types *fthrows;                     // Array of Type's of exceptions (not used)
     Statement *frequire;
     Statement *fensure;
     Statement *fbody;
@@ -540,7 +552,8 @@ struct FuncDeclaration : Declaration
     VarDeclaration *v_arguments_var;    // '_arguments' variable
     VarDeclaration *v_argptr;           // '_argptr' variable
 #endif
-    Dsymbols *parameters;               // Array of VarDeclaration's for parameters
+    VarDeclaration *v_argsave;          // save area for args passed in registers for variadic functions
+    VarDeclarations *parameters;        // Array of VarDeclaration's for parameters
     DsymbolTable *labtab;               // statement label symbol table
     Declaration *overnext;              // next in overload list
     Loc endloc;                         // location of closing curly bracket
@@ -550,6 +563,7 @@ struct FuncDeclaration : Declaration
     ILS inlineStatus;
     int inlineNest;                     // !=0 if nested inline
     int cantInterpret;                  // !=0 if cannot interpret function
+    int isArrayOp;                                              // !=0 if array operation
     int semanticRun;                    // 1 semantic() run
                                         // 2 semantic2() run
                                         // 3 semantic3() started
@@ -586,6 +600,11 @@ struct FuncDeclaration : Declaration
                                         // functions
 #else
     int nestedFrameRef;                 // !=0 if nested variables referenced
+#if IN_GCC
+    VarDeclarations frameVars;          // local variables in this function
+                                        // which are referenced by nested
+                                        // functions
+#endif
 #endif
 
     FuncDeclaration(Loc loc, Loc endloc, Identifier *id, StorageClass storage_class, Type *type);
@@ -642,13 +661,11 @@ struct FuncDeclaration : Declaration
     Statement *mergeFensure(Statement *);
     Parameters *getParameters(int *pvarargs);
 
-    static FuncDeclaration *genCfunc(Type *treturn, const char *name,
-        Type *t1 = 0, Type *t2 = 0, Type *t3 = 0);
-    static FuncDeclaration *genCfunc(Type *treturn, Identifier *id,
-        Type *t1 = 0, Type *t2 = 0, Type *t3 = 0);
+    static FuncDeclaration *genCfunc(Type *treturn, const char *name, Type *t1 = NULL, Type *t2 = NULL, Type *t3 = NULL);
+    static FuncDeclaration *genCfunc(Type *treturn, Identifier *id, Type *t1 = NULL, Type *t2 = NULL, Type *t3 = NULL);
 
     Symbol *toSymbol();
-    Symbol *toThunkSymbol(target_ptrdiff_t offset);     // thunk version
+    Symbol *toThunkSymbol(int offset);  // thunk version
     void toObjFile(int multiobj);                       // compile to .obj file
     int cvMember(unsigned char *p);
     void buildClosure(IRState *irs);
@@ -712,7 +729,7 @@ struct CtorDeclaration : FuncDeclaration
 struct PostBlitDeclaration : FuncDeclaration
 {
     PostBlitDeclaration(Loc loc, Loc endloc);
-   PostBlitDeclaration(Loc loc, Loc endloc, Identifier *id);
+    PostBlitDeclaration(Loc loc, Loc endloc, Identifier *id);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -776,6 +793,7 @@ struct SharedStaticCtorDeclaration : StaticCtorDeclaration
 
 struct StaticDtorDeclaration : FuncDeclaration
 {   VarDeclaration *vgate;      // 'gate' variable
+
     StaticDtorDeclaration(Loc loc, Loc endloc);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
@@ -861,9 +879,7 @@ struct DeleteDeclaration : FuncDeclaration
     int isVirtual();
     int addPreInvariant();
     int addPostInvariant();
-#ifdef _DH
     DeleteDeclaration *isDeleteDeclaration() { return this; }
-#endif
 };
 
 #endif /* DMD_DECLARATION_H */
