@@ -25,7 +25,7 @@
 
 /********************************* Import ****************************/
 
-Import::Import(Loc loc, Array *packages, Identifier *id, Identifier *aliasId,
+Import::Import(Loc loc, Identifiers *packages, Identifier *id, Identifier *aliasId,
         int isstatic)
     : Dsymbol(id)
 {
@@ -175,7 +175,7 @@ void Import::semantic(Scope *sc)
         }
 #endif
 
-// Modules need a list of each imported module
+        // Modules need a list of each imported module
         //printf("%s imports %s\n", sc->module->toChars(), mod->toChars());
         sc->module->aimports.push(mod);
 
