@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2009 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -43,15 +43,17 @@ const char Pline[] = "line";
 const char Ptype[] = "type";
 const char Pcomment[] = "comment";
 const char Pmembers[] = "members";
+const char Pprotection[] = "protection";
+const char* Pprotectionnames[] = {NULL, "none", "private", "package", "protected", "public", "export"};
 
 void JsonRemoveComma(OutBuffer *buf);
 
-void json_generate(Array *modules)
+void json_generate(Modules *modules)
 {   OutBuffer buf;
 
     buf.writestring("[\n");
-    for (int i = 0; i < modules->dim; i++)
-    {   Module *m = (Module *)modules->data[i];
+    for (size_t i = 0; i < modules->dim; i++)
+    {   Module *m = modules->tdata()[i];
         if (global.params.verbose)
             printf("json gen %s\n", m->toChars());
         m->toJsonBuffer(&buf);
@@ -64,7 +66,7 @@ void json_generate(Array *modules)
     char *arg = global.params.xfilename;
     if (!arg || !*arg)
     {   // Generate lib file name from first obj name
-        char *n = (char *)global.params.objfiles->data[0];
+        char *n = global.params.objfiles->tdata()[0];
 
         n = FileName::name(n);
         FileName *fn = FileName::forceExt(n, global.json_ext);
@@ -192,8 +194,8 @@ void Module::toJsonBuffer(OutBuffer *buf)
     buf->writestring(" : [\n");
 
     size_t offset = buf->offset;
-    for (int i = 0; i < members->dim; i++)
-    {   Dsymbol *s = (Dsymbol *)members->data[i];
+    for (size_t i = 0; i < members->dim; i++)
+    {   Dsymbol *s = members->tdata()[i];
         if (offset != buf->offset)
         {   buf->writestring(",\n");
             offset = buf->offset;
@@ -211,13 +213,13 @@ void AttribDeclaration::toJsonBuffer(OutBuffer *buf)
 {
     //printf("AttribDeclaration::toJsonBuffer()\n");
 
-    Array *d = include(NULL, NULL);
+    Dsymbols *d = include(NULL, NULL);
 
     if (d)
     {
         size_t offset = buf->offset;
         for (unsigned i = 0; i < d->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)d->data[i];
+        {   Dsymbol *s = d->tdata()[i];
             //printf("AttribDeclaration::toJsonBuffer %s\n", s->toChars());
             if (offset != buf->offset)
             {   buf->writestring(",\n");
@@ -259,6 +261,10 @@ void Declaration::toJsonBuffer(OutBuffer *buf)
 
     JsonProperty(buf, Pname, toChars());
     JsonProperty(buf, Pkind, kind());
+
+    if (prot())
+        JsonProperty(buf, Pprotection, Pprotectionnames[prot()]);
+
     if (type)
         JsonProperty(buf, Ptype, type->toChars());
 
@@ -285,8 +291,13 @@ void AggregateDeclaration::toJsonBuffer(OutBuffer *buf)
 
     JsonProperty(buf, Pname, toChars());
     JsonProperty(buf, Pkind, kind());
+
+    if (prot())
+        JsonProperty(buf, Pprotection, Pprotectionnames[prot()]);
+
     if (comment)
         JsonProperty(buf, Pcomment, (const char *)comment);
+
     if (loc.linnum)
         JsonProperty(buf, Pline, loc.linnum);
 
@@ -302,7 +313,7 @@ void AggregateDeclaration::toJsonBuffer(OutBuffer *buf)
             JsonString(buf, "interfaces");
             buf->writestring(" : [\n");
             size_t offset = buf->offset;
-            for (int i = 0; i < cd->interfaces_dim; i++)
+            for (size_t i = 0; i < cd->interfaces_dim; i++)
             {   BaseClass *b = cd->interfaces[i];
                 if (offset != buf->offset)
                 {   buf->writestring(",\n");
@@ -320,8 +331,8 @@ void AggregateDeclaration::toJsonBuffer(OutBuffer *buf)
         JsonString(buf, Pmembers);
         buf->writestring(" : [\n");
         size_t offset = buf->offset;
-        for (int i = 0; i < members->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)members->data[i];
+        for (size_t i = 0; i < members->dim; i++)
+        {   Dsymbol *s = members->tdata()[i];
             if (offset != buf->offset)
             {   buf->writestring(",\n");
                 offset = buf->offset;
@@ -344,6 +355,10 @@ void TemplateDeclaration::toJsonBuffer(OutBuffer *buf)
 
     JsonProperty(buf, Pname, toChars());
     JsonProperty(buf, Pkind, kind());
+
+    if (prot())
+        JsonProperty(buf, Pprotection, Pprotectionnames[prot()]);
+
     if (comment)
         JsonProperty(buf, Pcomment, (const char *)comment);
 
@@ -353,8 +368,8 @@ void TemplateDeclaration::toJsonBuffer(OutBuffer *buf)
     JsonString(buf, Pmembers);
     buf->writestring(" : [\n");
     size_t offset = buf->offset;
-    for (int i = 0; i < members->dim; i++)
-    {   Dsymbol *s = (Dsymbol *)members->data[i];
+    for (size_t i = 0; i < members->dim; i++)
+    {   Dsymbol *s = members->tdata()[i];
         if (offset != buf->offset)
         {   buf->writestring(",\n");
             offset = buf->offset;
@@ -374,9 +389,9 @@ void EnumDeclaration::toJsonBuffer(OutBuffer *buf)
     {
         if (members)
         {
-            for (int i = 0; i < members->dim; i++)
+            for (size_t i = 0; i < members->dim; i++)
             {
-                Dsymbol *s = (Dsymbol *)members->data[i];
+                Dsymbol *s = members->tdata()[i];
                 s->toJsonBuffer(buf);
                 buf->writestring(",\n");
             }
@@ -389,6 +404,10 @@ void EnumDeclaration::toJsonBuffer(OutBuffer *buf)
 
     JsonProperty(buf, Pname, toChars());
     JsonProperty(buf, Pkind, kind());
+
+    if (prot())
+        JsonProperty(buf, Pprotection, Pprotectionnames[prot()]);
+
     if (comment)
         JsonProperty(buf, Pcomment, (const char *)comment);
 
@@ -403,8 +422,8 @@ void EnumDeclaration::toJsonBuffer(OutBuffer *buf)
         JsonString(buf, Pmembers);
         buf->writestring(" : [\n");
         size_t offset = buf->offset;
-        for (int i = 0; i < members->dim; i++)
-        {   Dsymbol *s = (Dsymbol *)members->data[i];
+        for (size_t i = 0; i < members->dim; i++)
+        {   Dsymbol *s = members->tdata()[i];
             if (offset != buf->offset)
             {   buf->writestring(",\n");
                 offset = buf->offset;
@@ -427,6 +446,9 @@ void EnumMember::toJsonBuffer(OutBuffer *buf)
     JsonProperty(buf, Pname, toChars());
     JsonProperty(buf, Pkind, kind());
 
+    if (prot())
+        JsonProperty(buf, Pprotection, Pprotectionnames[prot()]);
+
     if (comment)
         JsonProperty(buf, Pcomment, (const char *)comment);
 
