@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -177,7 +177,6 @@ const char *Token::toChars()
             break;
 #endif
 
-
         case TOKstring:
 #if CSTRINGS
             p = string;
@@ -683,7 +682,6 @@ void Lexer::scan(Token *t)
 
                 StringValue *sv = stringtable.update((char *)t->ptr, p - t->ptr);
                 Identifier *id = (Identifier *) sv->ptrvalue;
-
                 if (!id)
                 {   id = new Identifier(sv->lstring.string,TOKidentifier);
                     sv->ptrvalue = id;
@@ -708,6 +706,7 @@ void Lexer::scan(Token *t)
                         sprintf(time, "%.8s", p + 11);
                         sprintf(timestamp, "%.24s", p);
                     }
+
 #if DMDV1
                     if (mod && id == Id::FILE)
                     {
@@ -720,7 +719,7 @@ void Lexer::scan(Token *t)
                         t->uns64value = loc.linnum;
                     }
                     else
-#endif //if V1
+#endif
                     if (id == Id::DATE)
                     {
                         t->ustring = (unsigned char *)date;
@@ -745,7 +744,6 @@ void Lexer::scan(Token *t)
                         t->ustring = (unsigned char *)timestamp;
                      Lstr:
                         t->value = TOKstring;
-                     Llen:
                         t->postfix = 0;
                         t->len = strlen((char *)t->ustring);
                     }
@@ -756,7 +754,7 @@ void Lexer::scan(Token *t)
                         for (const char *p = global.version + 1; 1; p++)
                         {
                             char c = *p;
-                            if (isdigit(c))
+                            if (isdigit((unsigned char)c))
                                 minor = minor * 10 + c - '0';
                             else if (c == '.')
                             {   major = minor;
@@ -1337,7 +1335,7 @@ unsigned Lexer::escapeSequence()
                         }
                     }
                     if (ndigits != 2 && !utf_isValidDchar(v))
-                        {       error("invalid UTF character \\U%08x", v);
+                    {   error("invalid UTF character \\U%08x", v);
                         v = '?';        // recover with valid UTF character
                     }
                     c = v;
@@ -1998,7 +1996,6 @@ TOK Lexer::number(Token *t)
     };
     enum FLAGS flags = FLAGS_decimal;
 
-    int i;
     int base;
     unsigned c;
     unsigned char *start;
@@ -2243,7 +2240,7 @@ done:
                 p += 2, r = 16;
             else if (p[1] == 'b' || p[1] == 'B')
                 p += 2, r = 2;
-            else if (isdigit(p[1]))
+            else if (isdigit((unsigned char)p[1]))
                 p += 1, r = 8;
         }
 
@@ -2260,7 +2257,7 @@ done:
                 break;
             if (d >= r)
                 break;
-           uinteger_t n2 = n * r;
+            uinteger_t n2 = n * r;
             //printf("n2 / r = %llx, n = %llx\n", n2/r, n);
             if (n2 / r != n || n2 + d < n)
             {
@@ -2365,9 +2362,9 @@ done:
             break;
 
         default:
-#ifdef DEBUG
-            printf("%x\n",flags);
-#endif
+            #ifdef DEBUG
+                printf("%x\n",flags);
+            #endif
             assert(0);
     }
     t->uns64value = n;
@@ -2507,7 +2504,7 @@ done:
             real_t::parse((char *)stringbuffer.data, real_t::Float);
 #else
             {   // Only interested in errno return
-                float f = strtof((char *)stringbuffer.data, NULL);
+                double d = strtof((char *)stringbuffer.data, NULL);
                 // Assign to f to keep gcc warnings at bay
             }
 #endif
@@ -2584,7 +2581,10 @@ void Lexer::pragma()
 
     scan(&tok);
     if (tok.value == TOKint32v || tok.value == TOKint64v)
-        linnum = tok.uns64value - 1;
+    {   linnum = tok.uns64value - 1;
+        if (linnum != tok.uns64value - 1)
+            error("line number out of range");
+    }
     else
         goto Lerr;
 
@@ -2723,6 +2723,7 @@ void Lexer::getDocComment(Token *t, unsigned lineComment)
     /* ct tells us which kind of comment it is: '/', '*', or '+'
      */
     unsigned char ct = t->ptr[2];
+
     /* Start of comment text skips over / * *, / + +, or / / /
      */
     unsigned char *q = t->ptr + 3;      // start of comment text
@@ -2753,8 +2754,8 @@ void Lexer::getDocComment(Token *t, unsigned lineComment)
     /* Comment is now [q .. qend].
      * Canonicalize it into buf[].
      */
-     OutBuffer buf;
-     int linestart = 0;
+    OutBuffer buf;
+    int linestart = 0;
 
     for (; q < qend; q++)
     {
@@ -2838,7 +2839,8 @@ void Lexer::getDocComment(Token *t, unsigned lineComment)
 
 unsigned char *Lexer::combineComments(unsigned char *c1, unsigned char *c2)
 {
-        //printf("Lexer::combineComments('%s', '%s')\n", c1, c2);
+    //printf("Lexer::combineComments('%s', '%s')\n", c1, c2);
+
     unsigned char *c = c2;
 
     if (c1)
@@ -2855,9 +2857,6 @@ unsigned char *Lexer::combineComments(unsigned char *c1, unsigned char *c2)
             }
             memcpy(c + len1, c2, len2);
             c[len1 + len2] = 0;
-            c[len1] = '\n';
-            memcpy(c + len1 + 1, c2, len2);
-            c[len1 + 1 + len2] = 0;
         }
     }
     return c;
@@ -3020,6 +3019,7 @@ static Keyword keywords[] =
     //{ "manifest",     TOKmanifest     },
 
     // Added after 1.0
+    {   "__argTypes",   TOKargTypes     },
     {   "ref",          TOKref          },
     {   "macro",        TOKmacro        },
 #if DMDV2
