@@ -1,9 +1,10 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
+// http://www.dsource.org/projects/dmd/browser/branches/dmd-1.x/src/mars.c
 // License for redistribution is by either the Artistic License
 // in artistic.txt, or the GNU General Public License in gnu.txt.
 // See the included readme.txt for details.
@@ -16,7 +17,7 @@
 
 #ifndef IN_GCC
 
-#if linux || __APPLE__ || __FreeBSD__ || __sun&&__SVR4
+#if linux || __APPLE__ || __FreeBSD__ || __OpenBSD__ || __sun&&__SVR4
 #include <errno.h>
 #endif
 #endif
@@ -42,6 +43,7 @@
 long __cdecl __ehfilter(LPEXCEPTION_POINTERS ep);
 #endif
 
+
 int response_expand(int *pargc, char ***pargv);
 void browse(const char *url);
 void getenv_setargv(const char *envvar, int *pargc, char** *pargv);
@@ -64,7 +66,7 @@ Global::Global()
 #ifndef IN_GCC
 #if TARGET_WINDOS
     obj_ext  = "obj";
-#elif TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_SOLARIS
+#elif TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_OPENBSD || TARGET_SOLARIS
     obj_ext  = "o";
 #elif TARGET_NET
 #else
@@ -77,7 +79,7 @@ Global::Global()
 #ifndef IN_GCC
 #if TARGET_WINDOS
     lib_ext  = "lib";
-#elif TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_SOLARIS
+#elif TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_OPENBSD || TARGET_SOLARIS
     lib_ext  = "a";
 #elif TARGET_NET
 #else
@@ -90,7 +92,7 @@ Global::Global()
 #ifndef IN_GCC
 #if TARGET_WINDOS
     dll_ext  = "dll";
-#elif TARGET_LINUX || TARGET_FREEBSD || TARGET_SOLARIS
+#elif TARGET_LINUX || TARGET_FREEBSD || TARGET_OPENBSD || TARGET_SOLARIS
     dll_ext  = "so";
 #elif TARGET_OSX
     dll_ext = "dylib";
@@ -101,13 +103,13 @@ Global::Global()
     dll_ext  = "so";
 #endif
 
-    copyright = "Copyright (c) 1999-2010 by Digital Mars";
+    copyright = "Copyright (c) 1999-2011 by Digital Mars";
     written = "written by Walter Bright"
 #if TARGET_NET
     "\nMSIL back-end (alpha release) by Cristian L. Vlasceanu and associates.";
 #endif
     ;
-    version = "v1.063";
+    version = "v1.070";
     global.structalign = 8;
 
     memset(&params, 0, sizeof(Param));
@@ -116,7 +118,6 @@ Global::Global()
 char *Loc::toChars()
 {
     OutBuffer buf;
-    char *p;
 
     if (filename)
     {
@@ -195,7 +196,7 @@ void verror(Loc loc, const char *format, va_list ap)
 #endif
         fprintf(stdmsg, "\n");
         fflush(stdmsg);
-        halt();
+//halt();
     }
     global.errors++;
 }
@@ -206,6 +207,11 @@ void vwarning(Loc loc, const char *format, va_list ap)
     {
         char *p = loc.toChars();
 
+#if IN_GCC
+        if (global.params.warnings == 1 && !global.warnings)
+            fprintf(stdmsg, "cc1d: all warnings being treated as errors\n");
+#endif
+
         if (*p)
             fprintf(stdmsg, "%s: ", p);
         mem.free(p);
@@ -265,7 +271,8 @@ void usage()
 #else
     const char fpic[] = "";
 #endif
-    printf("Digital Mars D Compiler %s\n%s %s\n",
+    printf("DMD%s D Compiler %s\n%s %s\n",
+        sizeof(size_t) == 4 ? "32" : "64",
         global.version, global.copyright, global.written);
     printf("\
 Documentation: http://www.digitalmars.com/d/1.0/index.html\n\
@@ -327,9 +334,8 @@ Usage:\n\
 
 int main(int argc, char *argv[])
 {
-    int i;
-    Array files;
-    Array libmodules;
+    Strings files;
+    Strings libmodules;
     char *p;
     Module *m;
     int status = EXIT_SUCCESS;
@@ -337,6 +343,10 @@ int main(int argc, char *argv[])
     int setdebuglib = 0;
     const char *inifilename = NULL;
 
+#ifdef DEBUG
+    printf("DMD %s DEBUG\n", global.version);
+#endif
+
     // Check for malformed input
     if (argc < 1 || !argv)
     {
@@ -344,7 +354,7 @@ int main(int argc, char *argv[])
         error("missing or null command line arguments");
         fatal();
     }
-    for (i = 0; i < argc; i++)
+    for (size_t i = 0; i < argc; i++)
     {
         if (!argv[i])
             goto Largs;
@@ -369,14 +379,17 @@ int main(int argc, char *argv[])
     global.params.Dversion = 2;
     global.params.quiet = 1;
 
-    global.params.linkswitches = new Array();
-    global.params.libfiles = new Array();
-    global.params.objfiles = new Array();
-    global.params.ddocfiles = new Array();
+    global.params.linkswitches = new Strings();
+    global.params.libfiles = new Strings();
+    global.params.objfiles = new Strings();
+    global.params.ddocfiles = new Strings();
+
+    // Default to -m32 for 32 bit dmd, -m64 for 64 bit dmd
+    global.params.is64bit = (sizeof(size_t) == 8);
 
 #if TARGET_WINDOS
     global.params.defaultlibname = "phobos";
-#elif TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_SOLARIS
+#elif TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_OPENBSD || TARGET_SOLARIS
     global.params.defaultlibname = "phobos";
 #elif TARGET_NET
 #else
@@ -394,20 +407,24 @@ int main(int argc, char *argv[])
     VersionCondition::addPredefinedGlobalIdent("D_NET");
 #endif
 #elif TARGET_LINUX
-        VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("Posix");
     VersionCondition::addPredefinedGlobalIdent("linux");
     global.params.isLinux = 1;
 #elif TARGET_OSX
-     VersionCondition::addPredefinedGlobalIdent("Posix");
-     VersionCondition::addPredefinedGlobalIdent("OSX");
-     global.params.isOSX = 1;
+    VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("OSX");
+    global.params.isOSX = 1;
 
-     // For legacy compatibility
-     VersionCondition::addPredefinedGlobalIdent("darwin");
+    // For legacy compatibility
+    VersionCondition::addPredefinedGlobalIdent("darwin");
 #elif TARGET_FREEBSD
-     VersionCondition::addPredefinedGlobalIdent("Posix");
-     VersionCondition::addPredefinedGlobalIdent("FreeBSD");
-     global.params.isFreeBSD = 1;
+    VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("FreeBSD");
+    global.params.isFreeBSD = 1;
+#elif TARGET_OPENBSD
+    VersionCondition::addPredefinedGlobalIdent("Posix");
+    VersionCondition::addPredefinedGlobalIdent("OpenBSD");
+    global.params.isFreeBSD = 1;
 #elif TARGET_SOLARIS
     VersionCondition::addPredefinedGlobalIdent("Posix");
     VersionCondition::addPredefinedGlobalIdent("Solaris");
@@ -416,7 +433,6 @@ int main(int argc, char *argv[])
 #error "fix this"
 #endif
 
-    VersionCondition::addPredefinedGlobalIdent("X86");
     VersionCondition::addPredefinedGlobalIdent("LittleEndian");
     //VersionCondition::addPredefinedGlobalIdent("D_Bits");
 #if DMDV2
@@ -426,7 +442,7 @@ int main(int argc, char *argv[])
 
 #if _WIN32
     inifilename = inifile(argv[0], "sc.ini");
-#elif linux || __APPLE__ || __FreeBSD__ || __sun&&__SVR4
+#elif linux || __APPLE__ || __FreeBSD__ || __OpenBSD__ || __sun&&__SVR4
     inifilename = inifile(argv[0], "dmd.conf");
 #else
 #error "fix this"
@@ -434,13 +450,13 @@ int main(int argc, char *argv[])
     getenv_setargv("DFLAGS", &argc, &argv);
 
 #if 0
-    for (i = 0; i < argc; i++)
+    for (size_t i = 0; i < argc; i++)
     {
         printf("argv[%d] = '%s'\n", i, argv[i]);
     }
 #endif
 
-    for (i = 1; i < argc; i++)
+    for (size_t i = 1; i < argc; i++)
     {
         p = argv[i];
         if (*p == '-')
@@ -451,7 +467,7 @@ int main(int argc, char *argv[])
                 global.params.link = 0;
             else if (strcmp(p + 1, "cov") == 0)
                 global.params.cov = 1;
-#if TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_SOLARIS
+#if TARGET_LINUX || TARGET_OSX || TARGET_FREEBSD || TARGET_OPENBSD || TARGET_SOLARIS
             else if (strcmp(p + 1, "fPIC") == 0)
                 global.params.pic = 1;
 #endif
@@ -472,9 +488,9 @@ int main(int argc, char *argv[])
                 global.params.trace = 1;
             }
             else if (strcmp(p + 1, "m32") == 0)
-                global.params.isX86_64 = 0;
+                global.params.is64bit = 0;
             else if (strcmp(p + 1, "m64") == 0)
-                global.params.isX86_64 = 1;
+                global.params.is64bit = 1;
             else if (strcmp(p + 1, "profile") == 0)
                 global.params.trace = 1;
             else if (strcmp(p + 1, "v") == 0)
@@ -494,7 +510,7 @@ int main(int argc, char *argv[])
             }
             else if (strcmp(p + 1, "w") == 0)
                 global.params.warnings = 1;
-                else if (strcmp(p + 1, "wi") == 0)
+            else if (strcmp(p + 1, "wi") == 0)
                 global.params.warnings = 2;
             else if (strcmp(p + 1, "O") == 0)
                 global.params.optimize = 1;
@@ -554,7 +570,6 @@ int main(int argc, char *argv[])
                         goto Lerror;
                 }
             }
-#ifdef _DH
             else if (p[1] == 'H')
             {   global.params.doHdrGeneration = 1;
                 switch (p[2])
@@ -578,7 +593,6 @@ int main(int argc, char *argv[])
                         goto Lerror;
                 }
             }
-#endif
             else if (p[1] == 'X')
             {   global.params.doXGeneration = 1;
                 switch (p[2])
@@ -617,13 +631,13 @@ int main(int argc, char *argv[])
             else if (p[1] == 'I')
             {
                 if (!global.params.imppath)
-                    global.params.imppath = new Array();
+                    global.params.imppath = new Strings();
                 global.params.imppath->push(p + 2);
             }
             else if (p[1] == 'J')
             {
                 if (!global.params.fileImppath)
-                    global.params.fileImppath = new Array();
+                    global.params.fileImppath = new Strings();
                 global.params.fileImppath->push(p + 2);
             }
             else if (memcmp(p + 1, "debug", 5) == 0 && p[6] != 'l')
@@ -634,7 +648,7 @@ int main(int argc, char *argv[])
                 //      -debug=identifier
                 if (p[6] == '=')
                 {
-                    if (isdigit(p[7]))
+                    if (isdigit((unsigned char)p[7]))
                     {   long level;
 
                         errno = 0;
@@ -660,7 +674,7 @@ int main(int argc, char *argv[])
                 //      -version=identifier
                 if (p[8] == '=')
                 {
-                    if (isdigit(p[9]))
+                    if (isdigit((unsigned char)p[9]))
                     {   long level;
 
                         errno = 0;
@@ -703,7 +717,7 @@ int main(int argc, char *argv[])
             }
             else if (memcmp(p + 1, "debuglib=", 9) == 0)
             {
-            setdebuglib = 1;
+                setdebuglib = 1;
                 global.params.debuglibname = p + 1 + 9;
             }
             else if (memcmp(p + 1, "deps=", 5) == 0)
@@ -742,6 +756,13 @@ int main(int argc, char *argv[])
 #else
                 browse("http://www.digitalmars.com/d/2.0/dmd-freebsd.html");
 #endif
+#endif
+#if __OpenBSD__
+#if DMDV1
+                browse("http://www.digitalmars.com/d/1.0/dmd-openbsd.html");
+#else
+                browse("http://www.digitalmars.com/d/2.0/dmd-openbsd.html");
+#endif
 #endif
                 exit(EXIT_SUCCESS);
             }
@@ -795,6 +816,7 @@ int main(int argc, char *argv[])
 
     if (!setdebuglib)
         global.params.debuglibname = global.params.defaultlibname;
+
 #if TARGET_OSX
     global.params.pic = 1;
 #endif
@@ -843,7 +865,15 @@ int main(int argc, char *argv[])
         global.params.objname = NULL;
 
         // Haven't investigated handling these options with multiobj
-        if (!global.params.cov && !global.params.trace)
+        if (!global.params.cov && !global.params.trace
+#if 0 && TARGET_WINDOS
+            /* multiobj causes class/struct debug info to be attached to init-data,
+             * but this will not be linked into the executable, so this info is lost.
+             * Bugzilla 4014
+             */
+            && !global.params.symdebug
+#endif
+           )
             global.params.multiobj = 1;
     }
     else if (global.params.run)
@@ -860,7 +890,7 @@ int main(int argc, char *argv[])
             //fatal();
         }
     }
-    if (global.params.isX86_64)
+    if (global.params.is64bit)
     {
         VersionCondition::addPredefinedGlobalIdent("D_InlineAsm_X86_64");
         VersionCondition::addPredefinedGlobalIdent("X86_64");
@@ -874,7 +904,7 @@ int main(int argc, char *argv[])
         VersionCondition::addPredefinedGlobalIdent("D_InlineAsm");
         VersionCondition::addPredefinedGlobalIdent("D_InlineAsm_X86");
         VersionCondition::addPredefinedGlobalIdent("X86");
-#if TARGET_WINDOSs
+#if TARGET_WINDOS
         VersionCondition::addPredefinedGlobalIdent("Win32");
 #endif
     }
@@ -882,7 +912,7 @@ int main(int argc, char *argv[])
         VersionCondition::addPredefinedGlobalIdent("D_Ddoc");
     if (global.params.cov)
         VersionCondition::addPredefinedGlobalIdent("D_Coverage");
-        if (global.params.pic)
+    if (global.params.pic)
         VersionCondition::addPredefinedGlobalIdent("D_PIC");
 #if DMDV2
     if (global.params.useUnitTests)
@@ -895,8 +925,6 @@ int main(int argc, char *argv[])
     Module::init();
     initPrecedence();
 
-    backend_init();
-
     if (global.params.verbose)
     {   printf("binary    %s\n", argv[0]);
         printf("version   %s\n", global.version);
@@ -908,15 +936,15 @@ int main(int argc, char *argv[])
     // Build import search path
     if (global.params.imppath)
     {
-        for (i = 0; i < global.params.imppath->dim; i++)
+        for (size_t i = 0; i < global.params.imppath->dim; i++)
         {
-            char *path = (char *)global.params.imppath->data[i];
-            Array *a = FileName::splitPath(path);
+            char *path = (*global.params.imppath)[i];
+            Strings *a = FileName::splitPath(path);
 
             if (a)
             {
                 if (!global.path)
-                    global.path = new Array();
+                    global.path = new Strings();
                 global.path->append(a);
             }
         }
@@ -925,34 +953,34 @@ int main(int argc, char *argv[])
     // Build string import search path
     if (global.params.fileImppath)
     {
-        for (i = 0; i < global.params.fileImppath->dim; i++)
+        for (size_t i = 0; i < global.params.fileImppath->dim; i++)
         {
-            char *path = (char *)global.params.fileImppath->data[i];
-            Array *a = FileName::splitPath(path);
+            char *path = global.params.fileImppath->tdata()[i];
+            Strings *a = FileName::splitPath(path);
 
             if (a)
             {
                 if (!global.filePath)
-                    global.filePath = new Array();
+                    global.filePath = new Strings();
                 global.filePath->append(a);
             }
         }
     }
 
     // Create Modules
-    Array modules;
+    Modules modules;
     modules.reserve(files.dim);
     int firstmodule = 1;
-    for (i = 0; i < files.dim; i++)
+    for (size_t i = 0; i < files.dim; i++)
     {
         char *ext;
         char *name;
 
-        p = (char *) files.data[i];
+        p = files.tdata()[i];
 
 #if _WIN32
         // Convert / to \ so linker will work
-        for (int i = 0; p[i]; i++)
+        for (size_t i = 0; p[i]; i++)
         {
             if (p[i] == '/')
                 p[i] = '\\';
@@ -964,48 +992,49 @@ int main(int argc, char *argv[])
         if (ext)
         {   /* Deduce what to do with a file based on its extension
              */
-                if (FileName::equals(ext, global.obj_ext))
+            if (FileName::equals(ext, global.obj_ext))
             {
-                global.params.objfiles->push(files.data[i]);
-                libmodules.push(files.data[i]);
+                global.params.objfiles->push(files.tdata()[i]);
+                libmodules.push(files.tdata()[i]);
                 continue;
             }
 
+            if (FileName::equals(ext, global.lib_ext))
             {
-                global.params.libfiles->push(files.data[i]);
-                libmodules.push(files.data[i]);
+                global.params.libfiles->push(files.tdata()[i]);
+                libmodules.push(files.tdata()[i]);
                 continue;
             }
 
             if (strcmp(ext, global.ddoc_ext) == 0)
             {
-                global.params.ddocfiles->push(files.data[i]);
+                global.params.ddocfiles->push(files.tdata()[i]);
                 continue;
             }
 
             if (FileName::equals(ext, global.json_ext))
             {
                 global.params.doXGeneration = 1;
-                global.params.xfilename = (char *)files.data[i];
+                global.params.xfilename = files.tdata()[i];
                 continue;
             }
 
             if (FileName::equals(ext, global.map_ext))
             {
-                global.params.mapfile = (char *)files.data[i];
+                global.params.mapfile = files.tdata()[i];
                 continue;
             }
 
 #if TARGET_WINDOS
             if (FileName::equals(ext, "res"))
             {
-                global.params.resfile = (char *)files.data[i];
+                global.params.resfile = files.tdata()[i];
                 continue;
             }
 
             if (FileName::equals(ext, "def"))
             {
-                global.params.deffile = (char *)files.data[i];
+                global.params.deffile = files.tdata()[i];
                 continue;
             }
 
@@ -1036,7 +1065,7 @@ int main(int argc, char *argv[])
                     strcmp(name, ".") == 0)
                 {
                 Linvalid:
-                    error("invalid file name '%s'", (char *)files.data[i]);
+                    error("invalid file name '%s'", files.tdata()[i]);
                     fatal();
                 }
             }
@@ -1056,7 +1085,7 @@ int main(int argc, char *argv[])
          */
 
         Identifier *id = Lexer::idPool(name);
-        m = new Module((char *) files.data[i], id, global.params.doDocComments, global.params.doHdrGeneration);
+        m = new Module(files.tdata()[i], id, global.params.doDocComments, global.params.doHdrGeneration);
         modules.push(m);
 
         if (firstmodule)
@@ -1074,35 +1103,36 @@ int main(int argc, char *argv[])
 #if ASYNCREAD
     // Multi threaded
     AsyncRead *aw = AsyncRead::create(modules.dim);
-    for (i = 0; i < modules.dim; i++)
+    for (size_t i = 0; i < modules.dim; i++)
     {
-        m = (Module *)modules.data[i];
+        m = modules.tdata()[i];
         aw->addFile(m->srcfile);
     }
     aw->start();
 #else
     // Single threaded
-    for (i = 0; i < modules.dim; i++)
+    for (size_t i = 0; i < modules.dim; i++)
     {
-        m = (Module *)modules.data[i];
+        m = modules.tdata()[i];
         m->read(0);
     }
 #endif
 
     // Parse files
-    int anydocfiles = 0;
-    for (i = 0; i < modules.dim; i++)
+    bool anydocfiles = false;
+    size_t filecount = modules.dim;
+    for (size_t filei = 0, modi = 0; filei < filecount; filei++, modi++)
     {
-        m = (Module *)modules.data[i];
+        m = modules.tdata()[modi];
         if (global.params.verbose)
             printf("parse     %s\n", m->toChars());
         if (!Module::rootModule)
             Module::rootModule = m;
         m->importedFrom = m;
-        if (!global.params.oneobj || i == 0 || m->isDocFile)
+        if (!global.params.oneobj || modi == 0 || m->isDocFile)
             m->deleteObjFile();
 #if ASYNCREAD
-        if (aw->read(i))
+        if (aw->read(filei))
         {
             error("cannot read file %s", m->srcfile->name->toChars());
         }
@@ -1110,17 +1140,17 @@ int main(int argc, char *argv[])
         m->parse();
         if (m->isDocFile)
         {
-            anydocfiles = 1;
+            anydocfiles = true;
             m->gendocfile();
 
             // Remove m from list of modules
-            modules.remove(i);
-            i--;
+            modules.remove(modi);
+            modi--;
 
             // Remove m's object file from list of object files
-            for (int j = 0; j < global.params.objfiles->dim; j++)
+            for (size_t j = 0; j < global.params.objfiles->dim; j++)
             {
-                if (m->objfile->name->str == global.params.objfiles->data[j])
+                if (m->objfile->name->str == global.params.objfiles->tdata()[j])
                 {
                     global.params.objfiles->remove(j);
                     break;
@@ -1134,6 +1164,7 @@ int main(int argc, char *argv[])
 #if ASYNCREAD
     AsyncRead::dispose(aw);
 #endif
+
     if (anydocfiles && modules.dim &&
         (global.params.oneobj || global.params.objname))
     {
@@ -1142,7 +1173,6 @@ int main(int argc, char *argv[])
     }
     if (global.errors)
         fatal();
-#ifdef _DH
     if (global.params.doHdrGeneration)
     {
         /* Generate 'header' import files.
@@ -1150,9 +1180,9 @@ int main(int argc, char *argv[])
          * line switches and what else is imported, they are generated
          * before any semantic analysis.
          */
-        for (i = 0; i < modules.dim; i++)
+        for (size_t i = 0; i < modules.dim; i++)
         {
-            m = (Module *)modules.data[i];
+            m = modules.tdata()[i];
             if (global.params.verbose)
                 printf("import    %s\n", m->toChars());
             m->genhdrfile();
@@ -1160,12 +1190,11 @@ int main(int argc, char *argv[])
     }
     if (global.errors)
         fatal();
-#endif
 
     // load all unconditional imports for better symbol resolving
-    for (i = 0; i < modules.dim; i++)
+    for (size_t i = 0; i < modules.dim; i++)
     {
-       m = (Module *)modules.data[i];
+       m = modules.tdata()[i];
        if (global.params.verbose)
            printf("importall %s\n", m->toChars());
        m->importAll(0);
@@ -1173,10 +1202,12 @@ int main(int argc, char *argv[])
     if (global.errors)
        fatal();
 
+    backend_init();
+
     // Do semantic analysis
-    for (i = 0; i < modules.dim; i++)
+    for (size_t i = 0; i < modules.dim; i++)
     {
-        m = (Module *)modules.data[i];
+        m = modules.tdata()[i];
         if (global.params.verbose)
             printf("semantic  %s\n", m->toChars());
         m->semantic();
@@ -1188,9 +1219,9 @@ int main(int argc, char *argv[])
     Module::runDeferredSemantic();
 
     // Do pass 2 semantic analysis
-    for (i = 0; i < modules.dim; i++)
+    for (size_t i = 0; i < modules.dim; i++)
     {
-        m = (Module *)modules.data[i];
+        m = modules.tdata()[i];
         if (global.params.verbose)
             printf("semantic2 %s\n", m->toChars());
         m->semantic2();
@@ -1199,9 +1230,9 @@ int main(int argc, char *argv[])
         fatal();
 
     // Do pass 3 semantic analysis
-    for (i = 0; i < modules.dim; i++)
+    for (size_t i = 0; i < modules.dim; i++)
     {
-        m = (Module *)modules.data[i];
+        m = modules.tdata()[i];
         if (global.params.verbose)
             printf("semantic3 %s\n", m->toChars());
         m->semantic3();
@@ -1209,7 +1240,7 @@ int main(int argc, char *argv[])
     if (global.errors)
         fatal();
 
-        if (global.params.moduleDeps != NULL)
+    if (global.params.moduleDeps != NULL)
     {
         assert(global.params.moduleDepsFile != NULL);
 
@@ -1219,6 +1250,7 @@ int main(int argc, char *argv[])
         deps.writev();
     }
 
+
     // Scan for functions to inline
     if (global.params.useInline)
     {
@@ -1231,9 +1263,9 @@ int main(int argc, char *argv[])
         {
             // Do pass 3 semantic analysis on all imported modules,
             // since otherwise functions in them cannot be inlined
-            for (i = 0; i < Module::amodules.dim; i++)
+            for (size_t i = 0; i < Module::amodules.dim; i++)
             {
-                m = (Module *)Module::amodules.data[i];
+                m = Module::amodules.tdata()[i];
                 if (global.params.verbose)
                     printf("semantic3 %s\n", m->toChars());
                 m->semantic3();
@@ -1242,14 +1274,15 @@ int main(int argc, char *argv[])
                 fatal();
         }
 
-        for (i = 0; i < modules.dim; i++)
+        for (size_t i = 0; i < modules.dim; i++)
         {
-            m = (Module *)modules.data[i];
+            m = modules.tdata()[i];
             if (global.params.verbose)
                 printf("inline scan %s\n", m->toChars());
             m->inlineScan();
         }
     }
+
     // Do not attempt to generate output files if errors or warnings occurred
     if (global.errors || global.warnings)
         fatal();
@@ -1261,9 +1294,9 @@ int main(int argc, char *argv[])
         library->setFilename(global.params.objdir, global.params.libname);
 
         // Add input object and input library files to output library
-        for (int i = 0; i < libmodules.dim; i++)
+        for (size_t i = 0; i < libmodules.dim; i++)
         {
-            char *p = (char *)libmodules.data[i];
+            char *p = libmodules.tdata()[i];
             library->addObject(p, NULL, 0);
         }
     }
@@ -1275,9 +1308,9 @@ int main(int argc, char *argv[])
 
     if (global.params.oneobj)
     {
-        for (i = 0; i < modules.dim; i++)
+        for (size_t i = 0; i < modules.dim; i++)
         {
-            m = (Module *)modules.data[i];
+            m = modules.tdata()[i];
             if (global.params.verbose)
                 printf("code      %s\n", m->toChars());
             if (i == 0)
@@ -1288,14 +1321,14 @@ int main(int argc, char *argv[])
         }
         if (!global.errors && modules.dim)
         {
-            obj_end(library, ((Module *)modules.data[0])->objfile);
+            obj_end(library, modules.tdata()[0]->objfile);
         }
     }
     else
     {
-        for (i = 0; i < modules.dim; i++)
+        for (size_t i = 0; i < modules.dim; i++)
         {
-            m = (Module *)modules.data[i];
+            m = modules.tdata()[i];
             if (global.params.verbose)
                 printf("code      %s\n", m->toChars());
             if (global.params.obj)
@@ -1350,9 +1383,9 @@ int main(int argc, char *argv[])
 
                 /* Delete .obj files and .exe file
                  */
-                for (i = 0; i < modules.dim; i++)
+                for (size_t i = 0; i < modules.dim; i++)
                 {
-                    Module *m = (Module *)modules.data[i];
+                    Module *m = modules.tdata()[i];
                     m->deleteObjFile();
                     if (global.params.oneobj)
                         break;
@@ -1388,13 +1421,13 @@ void getenv_setargv(const char *envvar, int *pargc, char** *pargv)
     env = mem.strdup(env);      // create our own writable copy
 
     int argc = *pargc;
-    Array *argv = new Array();
+    Strings *argv = new Strings();
     argv->setDim(argc);
 
-    for (int i = 0; i < argc; i++)
-        argv->data[i] = (void *)(*pargv)[i];
+    for (size_t i = 0; i < argc; i++)
+        argv->tdata()[i] = (*pargv)[i];
 
-    int j = 1;                  // leave argv[0] alone
+    size_t j = 1;               // leave argv[0] alone
     while (1)
     {
         int wildcard = 1;       // do wildcard expansion
@@ -1468,7 +1501,7 @@ void getenv_setargv(const char *envvar, int *pargc, char** *pargv)
 
 Ldone:
     *pargc = argc;
-    *pargv = (char **)argv->data;
+    *pargv = argv->tdata();
 }
 
 #if WINDOWS_SEH
