@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -48,9 +48,9 @@ ClassDeclaration *Module::moduleinfo;
 
 Module *Module::rootModule;
 DsymbolTable *Module::modules;
-Array Module::amodules;
+Modules Module::amodules;
 
-Array Module::deferred; // deferred Dsymbol's needing semantic() run on them
+Dsymbols Module::deferred; // deferred Dsymbol's needing semantic() run on them
 unsigned Module::dprogress;
 
 void Module::init()
@@ -62,8 +62,6 @@ Module::Module(char *filename, Identifier *ident, int doDocComment, int doHdrGen
         : Package(ident)
 {
     FileName *srcfilename;
-    FileName *cfilename;
-    FileName *hfilename;
     FileName *objfilename;
     FileName *symfilename;
 
@@ -79,7 +77,7 @@ Module::Module(char *filename, Identifier *ident, int doDocComment, int doHdrGen
 #ifdef IN_GCC
     strictlyneedmoduleinfo = 0;
 #endif
-        selfimports = 0;
+    selfimports = 0;
     insearch = 0;
     searchCacheIdent = NULL;
     searchCacheSymbol = NULL;
@@ -252,7 +250,7 @@ const char *Module::kind()
     return "module";
 }
 
-Module *Module::load(Loc loc, Array *packages, Identifier *ident)
+Module *Module::load(Loc loc, Identifiers *packages, Identifier *ident)
 {   Module *m;
     char *filename;
 
@@ -266,10 +264,9 @@ Module *Module::load(Loc loc, Array *packages, Identifier *ident)
     if (packages && packages->dim)
     {
         OutBuffer buf;
-        int i;
 
-        for (i = 0; i < packages->dim; i++)
-        {   Identifier *pid = (Identifier *)packages->data[i];
+        for (size_t i = 0; i < packages->dim; i++)
+        {   Identifier *pid = packages->tdata()[i];
 
             buf.writestring(pid->toChars());
 #if _WIN32
@@ -330,7 +327,7 @@ Module *Module::load(Loc loc, Array *packages, Identifier *ident)
         if (packages)
         {
             for (size_t i = 0; i < packages->dim; i++)
-            {   Identifier *pid = (Identifier *)packages->data[i];
+            {   Identifier *pid = packages->tdata()[i];
                 printf("%s.", pid->toChars());
             }
         }
@@ -357,14 +354,14 @@ void Module::read(Loc loc)
              */
             if (global.path)
             {
-                for (int i = 0; i < global.path->dim; i++)
+                for (size_t i = 0; i < global.path->dim; i++)
                 {
-                    char *p = (char *)global.path->data[i];
-                    fprintf(stdmsg, "import path[%d] = %s\n", i, p);
+                    char *p = global.path->tdata()[i];
+                    fprintf(stdmsg, "import path[%zd] = %s\n", i, p);
                 }
             }
-        else
-            fprintf(stdmsg, "Specify path to file '%s' with -I switch\n", srcfile->toChars());
+            else
+                fprintf(stdmsg, "Specify path to file '%s' with -I switch\n", srcfile->toChars());
         }
         fatal();
     }
@@ -372,7 +369,7 @@ void Module::read(Loc loc)
 
 inline unsigned readwordLE(unsigned short *p)
 {
-#if __I86__
+#if LITTLE_ENDIAN
     return *p;
 #else
     return (((unsigned char *)p)[1] << 8) | ((unsigned char *)p)[0];
@@ -386,7 +383,7 @@ inline unsigned readwordBE(unsigned short *p)
 
 inline unsigned readlongLE(unsigned *p)
 {
-#if __I86__
+#if LITTLE_ENDIAN
     return *p;
 #else
     return ((unsigned char *)p)[0] |
@@ -688,9 +685,9 @@ void Module::importAll(Scope *prevsc)
     {
         // Add all symbols into module's symbol table
         symtab = new DsymbolTable();
-        for (int i = 0; i < members->dim; i++)
+        for (size_t i = 0; i < members->dim; i++)
         {
-            Dsymbol *s = (Dsymbol *)members->data[i];
+            Dsymbol *s = members->tdata()[i];
             s->addMember(NULL, sc->scopesym, 1);
         }
     }
@@ -702,14 +699,14 @@ void Module::importAll(Scope *prevsc)
      * before any semantic() on any of them.
      */
     setScope(sc);               // remember module scope for semantic
-    for (int i = 0; i < members->dim; i++)
-    {   Dsymbol *s = (Dsymbol *)members->data[i];
+    for (size_t i = 0; i < members->dim; i++)
+    {   Dsymbol *s = members->tdata()[i];
         s->setScope(sc);
     }
 
-    for (int i = 0; i < members->dim; i++)
+    for (size_t i = 0; i < members->dim; i++)
     {
-        Dsymbol *s = (Dsymbol *)members->data[i];
+        Dsymbol *s = members->tdata()[i];
         s->importAll(sc);
     }
 
@@ -743,30 +740,10 @@ void Module::semantic()
         Import *im = new Import(0, NULL, Id::object, NULL, 0);
         members->shift(im);
     }
-#ifdef IN_GCC
-    else
-    {
-        /* va_list is a pain.  If va_list involves a struct, add the
-           struct declaration to the "object" module.  This depends on
-           the GCC backend not naming the struct something that will
-           cause a conflict or define "va_list" without going through
-           std.stdarg. */
-        Type * t = d_gcc_builtin_va_list_d_type;
-        while (t) {
-            if (t->ty == Tstruct) {
-                StructDeclaration * sd = ((TypeStruct *) t)->sym;
-                sd->parent = this;
-                members->push(sd);
-                break;
-            }
-            t = t->next;
-        }
-    }
-#endif
 
     // Add all symbols into module's symbol table
     symtab = new DsymbolTable();
-    for (int i = 0; i < members->dim; i++)
+    for (size_t i = 0; i < members->dim; i++)
     {   Dsymbol *s = (Dsymbol *)members->data[i];
         s->addMember(NULL, sc->scopesym, 1);
     }
@@ -776,23 +753,23 @@ void Module::semantic()
      * If this works out well, it can be extended to all modules
      * before any semantic() on any of them.
      */
-    for (int i = 0; i < members->dim; i++)
+    for (size_t i = 0; i < members->dim; i++)
     {   Dsymbol *s = (Dsymbol *)members->data[i];
         s->setScope(sc);
     }
 #endif
 
     // Do semantic() on members that don't depend on others
-    for (int i = 0; i < members->dim; i++)
-    {   Dsymbol *s = (Dsymbol *)members->data[i];
+    for (size_t i = 0; i < members->dim; i++)
+    {   Dsymbol *s = members->tdata()[i];
 
         //printf("\tModule('%s'): '%s'.semantic0()\n", toChars(), s->toChars());
         s->semantic0(sc);
     }
 
     // Pass 1 semantic routines: do public side of the definition
-    for (int i = 0; i < members->dim; i++)
-    {   Dsymbol *s = (Dsymbol *)members->data[i];
+    for (size_t i = 0; i < members->dim; i++)
+    {   Dsymbol *s = members->tdata()[i];
 
         //printf("\tModule('%s'): '%s'.semantic()\n", toChars(), s->toChars());
         s->semantic(sc);
@@ -808,13 +785,12 @@ void Module::semantic()
 }
 
 void Module::semantic2()
-{   int i;
-
+{
     if (deferred.dim)
     {
-        for (int i = 0; i < deferred.dim; i++)
+        for (size_t i = 0; i < deferred.dim; i++)
         {
-            Dsymbol *sd = (Dsymbol *)deferred.data[i];
+            Dsymbol *sd = deferred.tdata()[i];
 
             sd->error("unable to resolve forward reference in definition");
         }
@@ -833,10 +809,10 @@ void Module::semantic2()
     //printf("Module = %p\n", sc.scopesym);
 
     // Pass 2 semantic routines: do initializers and function bodies
-    for (i = 0; i < members->dim; i++)
+    for (size_t i = 0; i < members->dim; i++)
     {   Dsymbol *s;
 
-        s = (Dsymbol *)members->data[i];
+        s = members->tdata()[i];
         s->semantic2(sc);
     }
 
@@ -847,8 +823,7 @@ void Module::semantic2()
 }
 
 void Module::semantic3()
-{   int i;
-
+{
     //printf("Module::semantic3('%s'): parent = %p\n", toChars(), parent);
     if (semanticstarted >= 3)
         return;
@@ -862,10 +837,10 @@ void Module::semantic3()
     //printf("Module = %p\n", sc.scopesym);
 
     // Pass 3 semantic routines: do initializers and function bodies
-    for (i = 0; i < members->dim; i++)
+    for (size_t i = 0; i < members->dim; i++)
     {   Dsymbol *s;
 
-        s = (Dsymbol *)members->data[i];
+        s = members->tdata()[i];
         //printf("Module %s: %s.semantic3()\n", toChars(), s->toChars());
         s->semantic3(sc);
     }
@@ -887,8 +862,8 @@ void Module::inlineScan()
     // gets imported, it is unaffected by context.
     //printf("Module = %p\n", sc.scopesym);
 
-    for (int i = 0; i < members->dim; i++)
-    {   Dsymbol *s = (Dsymbol *)members->data[i];
+    for (size_t i = 0; i < members->dim; i++)
+    {   Dsymbol *s = members->tdata()[i];
         //if (global.params.verbose)
             //printf("inline scan symbol %s\n", s->toChars());
 
@@ -910,8 +885,8 @@ void Module::gensymfile()
     buf.printf("// Sym file generated from '%s'", srcfile->toChars());
     buf.writenl();
 
-    for (int i = 0; i < members->dim; i++)
-    {   Dsymbol *s = (Dsymbol *)members->data[i];
+    for (size_t i = 0; i < members->dim; i++)
+    {   Dsymbol *s = members->tdata()[i];
 
         s->toCBuffer(&buf, &hgs);
     }
@@ -971,8 +946,8 @@ Dsymbol *Module::symtabInsert(Dsymbol *s)
 
 void Module::clearCache()
 {
-    for (int i = 0; i < amodules.dim; i++)
-    {   Module *m = (Module *)amodules.data[i];
+    for (size_t i = 0; i < amodules.dim; i++)
+    {   Module *m = amodules.tdata()[i];
         m->searchCacheIdent = NULL;
     }
 }
@@ -984,9 +959,9 @@ void Module::clearCache()
 void Module::addDeferredSemantic(Dsymbol *s)
 {
     // Don't add it if it is already there
-    for (int i = 0; i < deferred.dim; i++)
+    for (size_t i = 0; i < deferred.dim; i++)
     {
-        Dsymbol *sd = (Dsymbol *)deferred.data[i];
+        Dsymbol *sd = deferred.tdata()[i];
 
         if (sd == s)
             return;
@@ -1012,7 +987,7 @@ void Module::runDeferredSemantic()
     //if (deferred.dim) printf("+Module::runDeferredSemantic('%s'), len = %d\n", toChars(), deferred.dim);
     nested++;
 
-        size_t len;
+    size_t len;
     do
     {
         dprogress = 0;
@@ -1031,10 +1006,10 @@ void Module::runDeferredSemantic()
             todo = (Dsymbol **)alloca(len * sizeof(Dsymbol *));
             assert(todo);
         }
-        memcpy(todo, deferred.data, len * sizeof(Dsymbol *));
+        memcpy(todo, deferred.tdata(), len * sizeof(Dsymbol *));
         deferred.setDim(0);
 
-        for (int i = 0; i < len; i++)
+        for (size_t i = 0; i < len; i++)
         {
             Dsymbol *s = todo[i];
 
@@ -1058,13 +1033,13 @@ int Module::imports(Module *m)
     //printf("%s Module::imports(%s)\n", toChars(), m->toChars());
     int aimports_dim = aimports.dim;
 #if 0
-    for (int i = 0; i < aimports.dim; i++)
+    for (size_t i = 0; i < aimports.dim; i++)
     {   Module *mi = (Module *)aimports.data[i];
         printf("\t[%d] %s\n", i, mi->toChars());
     }
 #endif
-    for (int i = 0; i < aimports.dim; i++)
-    {   Module *mi = (Module *)aimports.data[i];
+    for (size_t i = 0; i < aimports.dim; i++)
+    {   Module *mi = aimports.tdata()[i];
         if (mi == m)
             return TRUE;
         if (!mi->insearch)
@@ -1087,16 +1062,16 @@ int Module::selfImports()
     //printf("Module::selfImports() %s\n", toChars());
     if (!selfimports)
     {
-        for (int i = 0; i < amodules.dim; i++)
-        {   Module *mi = (Module *)amodules.data[i];
+        for (size_t i = 0; i < amodules.dim; i++)
+        {   Module *mi = amodules.tdata()[i];
             //printf("\t[%d] %s\n", i, mi->toChars());
             mi->insearch = 0;
         }
 
         selfimports = imports(this) + 1;
 
-        for (int i = 0; i < amodules.dim; i++)
-        {   Module *mi = (Module *)amodules.data[i];
+        for (size_t i = 0; i < amodules.dim; i++)
+        {   Module *mi = amodules.tdata()[i];
             //printf("\t[%d] %s\n", i, mi->toChars());
             mi->insearch = 0;
         }
@@ -1104,9 +1079,10 @@ int Module::selfImports()
     return selfimports - 1;
 }
 
+
 /* =========================== ModuleDeclaration ===================== */
 
-ModuleDeclaration::ModuleDeclaration(Array *packages, Identifier *id)
+ModuleDeclaration::ModuleDeclaration(Identifiers *packages, Identifier *id)
 {
     this->packages = packages;
     this->id = id;
@@ -1115,12 +1091,11 @@ ModuleDeclaration::ModuleDeclaration(Array *packages, Identifier *id)
 char *ModuleDeclaration::toChars()
 {
     OutBuffer buf;
-    int i;
 
     if (packages && packages->dim)
     {
-        for (i = 0; i < packages->dim; i++)
-        {   Identifier *pid = (Identifier *)packages->data[i];
+        for (size_t i = 0; i < packages->dim; i++)
+        {   Identifier *pid = packages->tdata()[i];
 
             buf.writestring(pid->toChars());
             buf.writeByte('.');
@@ -1145,7 +1120,7 @@ const char *Package::kind()
 }
 
 
-DsymbolTable *Package::resolve(Array *packages, Dsymbol **pparent, Package **ppkg)
+DsymbolTable *Package::resolve(Identifiers *packages, Dsymbol **pparent, Package **ppkg)
 {
     DsymbolTable *dst = Module::modules;
     Dsymbol *parent = NULL;
@@ -1155,10 +1130,9 @@ DsymbolTable *Package::resolve(Array *packages, Dsymbol **pparent, Package **ppk
         *ppkg = NULL;
 
     if (packages)
-    {   int i;
-
-        for (i = 0; i < packages->dim; i++)
-        {   Identifier *pid = (Identifier *)packages->data[i];
+    {
+        for (size_t i = 0; i < packages->dim; i++)
+        {   Identifier *pid = packages->tdata()[i];
             Dsymbol *p;
 
             p = dst->lookup(pid);
