@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -46,7 +46,7 @@ static real_t zero;     // work around DMC bug for now
  * return that initializer.
  */
 
-Expression *fromConstInitializer(Expression *e1)
+Expression *fromConstInitializer(int result, Expression *e1)
 {
     //printf("fromConstInitializer(%s)\n", e1->toChars());
     if (e1->op == TOKvar)
@@ -54,7 +54,13 @@ Expression *fromConstInitializer(Expression *e1)
         VarDeclaration *v = ve->var->isVarDeclaration();
         if (v && !v->originalType && v->scope)  // semantic() not yet run
             v->semantic (v->scope);
-        if (v && v->isConst() && v->init)
+        if (!v || !v->type)
+            return e1;
+        Type * tb = v->type->toBasetype();
+        if (v->isConst() && v->init
+            && (result & WANTinterpret || (tb->isscalar() ||
+            ((result & WANTexpand) && (tb->ty != Tsarray && tb->ty != Tstruct))))
+        )
         {   Expression *ei = v->init->toExpression();
             if (ei && ei->type)
                 e1 = ei;
@@ -72,10 +78,7 @@ Expression *Expression::optimize(int result)
 
 Expression *VarExp::optimize(int result)
 {
-    if (result & WANTinterpret)
-    {
-        return fromConstInitializer(this);
-    }
+    return fromConstInitializer(result, this);
     return this;
 }
 
@@ -395,7 +398,7 @@ Expression *CastExp::optimize(int result)
 
     e1 = e1->optimize(result);
     if (result & WANTinterpret)
-        e1 = fromConstInitializer(e1);
+        e1 = fromConstInitializer(result, e1);
 
     if ((e1->op == TOKstring || e1->op == TOKarrayliteral) &&
         (type->ty == Tpointer || type->ty == Tarray) &&
@@ -421,7 +424,7 @@ Expression *CastExp::optimize(int result)
         // See if we can remove an unnecessary cast
         ClassDeclaration *cdfrom;
         ClassDeclaration *cdto;
-        target_ptrdiff_t offset;
+        int offset;
 
         cdfrom = e1->type->isClassHandle();
         cdto   = type->isClassHandle();
@@ -468,7 +471,7 @@ Expression *BinExp::optimize(int result)
             dinteger_t i2 = e2->toInteger();
             d_uns64 sz = e1->type->size() * 8;
             if (i2 < 0 || i2 > sz)
-            {   error("shift assign by %jd is outside the range 0..%zu", i2, sz);
+            {   error("shift assign by %"PRIdMAX" is outside the range 0..%"PRIuSIZE, i2, sz);
                 e2 = new IntegerExp(0);
             }
         }
@@ -561,9 +564,9 @@ Expression *shift_optimize(int result, BinExp *e, Expression *(*shift)(Type *, E
     if (e->e2->isConst() == 1)
     {
         dinteger_t i2 = e->e2->toInteger();
-        target_size_t sz = e->e1->type->size() * 8;
+        d_uns64 sz = e->e1->type->size() * 8;
         if (i2 < 0 || i2 > sz)
-        {   e->error("shift by %"PRIdMAX" is outside the range 0..%"PRIuTSIZE, i2, sz);
+        {   e->error("shift by %"PRIdMAX" is outside the range 0..%"PRIuSIZE, i2, sz);
             e->e2 = new IntegerExp(0);
         }
         if (e->e1->isConst() == 1)
@@ -664,7 +667,7 @@ Expression *ArrayLengthExp::optimize(int result)
 {   Expression *e;
 
     //printf("ArrayLengthExp::optimize(result = %d) %s\n", result, toChars());
-    e1 = e1->optimize(WANTvalue | (result & WANTinterpret));
+    e1 = e1->optimize(WANTvalue | WANTexpand | (result & WANTinterpret));
     e = this;
     if (e1->op == TOKstring || e1->op == TOKarrayliteral || e1->op == TOKassocarrayliteral)
     {
@@ -681,8 +684,8 @@ Expression *EqualExp::optimize(int result)
     e2 = e2->optimize(WANTvalue | (result & WANTinterpret));
     e = this;
 
-    Expression *e1 = fromConstInitializer(this->e1);
-    Expression *e2 = fromConstInitializer(this->e2);
+    Expression *e1 = fromConstInitializer(result, this->e1);
+    Expression *e2 = fromConstInitializer(result, this->e2);
 
     e = Equal(op, type, e1, e2);
     if (e == EXP_CANT_INTERPRET)
@@ -692,11 +695,11 @@ Expression *EqualExp::optimize(int result)
 
 Expression *IdentityExp::optimize(int result)
 {
-
     //printf("IdentityExp::optimize(result = %d) %s\n", result, toChars());
     e1 = e1->optimize(WANTvalue | (result & WANTinterpret));
     e2 = e2->optimize(WANTvalue | (result & WANTinterpret));
     Expression *e = this;
+
     if ((this->e1->isConst()     && this->e2->isConst()) ||
         (this->e1->op == TOKnull && this->e2->op == TOKnull))
     {
@@ -707,13 +710,40 @@ Expression *IdentityExp::optimize(int result)
     return e;
 }
 
+
+/* It is possible for constant folding to change an array expression of
+ * unknown length, into one where the length is known.
+ * If the expression 'arr' is a literal, set lengthVar to be its length.
+ */
+void setLengthVarIfKnown(VarDeclaration *lengthVar, Expression *arr)
+{
+    if (!lengthVar)
+        return;
+    if (lengthVar->init && !lengthVar->init->isVoidInitializer())
+        return; // we have previously calculated the length
+    size_t len;
+    if (arr->op == TOKstring)
+        len = ((StringExp *)arr)->len;
+    else if (arr->op == TOKarrayliteral)
+        len = ((ArrayLiteralExp *)arr)->elements->dim;
+    else
+        return; // we don't know the length yet
+
+    Expression *dollar = new IntegerExp(0, len, Type::tsize_t);
+    lengthVar->init = new ExpInitializer(0, dollar);
+    lengthVar->storage_class |= STCstatic | STCconst;
+}
+
+
 Expression *IndexExp::optimize(int result)
 {   Expression *e;
 
     //printf("IndexExp::optimize(result = %d) %s\n", result, toChars());
     Expression *e1 = this->e1->optimize(WANTvalue | (result & WANTinterpret));
     if (result & WANTinterpret)
-        e1 = fromConstInitializer(e1);
+        e1 = fromConstInitializer(result, e1);
+    // We might know $ now
+    setLengthVarIfKnown(lengthVar, e1);
     e2 = e2->optimize(WANTvalue | (result & WANTinterpret));
     e = Index(type, e1, e2);
     if (e == EXP_CANT_INTERPRET)
@@ -721,6 +751,7 @@ Expression *IndexExp::optimize(int result)
     return e;
 }
 
+
 Expression *SliceExp::optimize(int result)
 {   Expression *e;
 
@@ -737,12 +768,15 @@ Expression *SliceExp::optimize(int result)
         return e;
     }
     if (result & WANTinterpret)
-        e1 = fromConstInitializer(e1);
+        e1 = fromConstInitializer(result, e1);
+    // We might know $ now
+    setLengthVarIfKnown(lengthVar, e1);
     lwr = lwr->optimize(WANTvalue | (result & WANTinterpret));
     upr = upr->optimize(WANTvalue | (result & WANTinterpret));
     e = Slice(type, e1, lwr, upr);
     if (e == EXP_CANT_INTERPRET)
         e = this;
+    //printf("-SliceExp::optimize() %s\n", e->toChars());
     return e;
 }
 
