@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2009 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -47,16 +47,16 @@ AggregateDeclaration::AggregateDeclaration(Loc loc, Identifier *id)
     inv = NULL;
     aggNew = NULL;
     aggDelete = NULL;
-
+#if IN_GCC
     attributes = NULL;
+#endif
 
     stag = NULL;
     sinit = NULL;
 #if DMDV2
-    dtor = NULL;
-
     ctor = NULL;
     defaultCtor = NULL;
+    aliasthis = NULL;
 #endif
 }
 
@@ -85,13 +85,12 @@ void AggregateDeclaration::semantic2(Scope *sc)
 }
 
 void AggregateDeclaration::semantic3(Scope *sc)
-{   int i;
-
+{
     //printf("AggregateDeclaration::semantic3(%s)\n", toChars());
     if (members)
     {
         sc = sc->push(this);
-        for (i = 0; i < members->dim; i++)
+        for (size_t i = 0; i < members->dim; i++)
         {
             Dsymbol *s = (Dsymbol *)members->data[i];
             s->semantic3(sc);
@@ -101,12 +100,11 @@ void AggregateDeclaration::semantic3(Scope *sc)
 }
 
 void AggregateDeclaration::inlineScan()
-{   int i;
-
+{
     //printf("AggregateDeclaration::inlineScan(%s)\n", toChars());
     if (members)
     {
-        for (i = 0; i < members->dim; i++)
+        for (size_t i = 0; i < members->dim; i++)
         {
             Dsymbol *s = (Dsymbol *)members->data[i];
             //printf("inline scan aggregate symbol '%s'\n", s->toChars());
@@ -115,7 +113,7 @@ void AggregateDeclaration::inlineScan()
     }
 }
 
-target_size_t AggregateDeclaration::size(Loc loc)
+unsigned AggregateDeclaration::size(Loc loc)
 {
     //printf("AggregateDeclaration::size() = %d\n", structsize);
     if (!members)
@@ -145,15 +143,15 @@ int AggregateDeclaration::isDeprecated()
  */
 
 void AggregateDeclaration::alignmember(
-        target_size_t salign,   // struct alignment that is in effect
-        target_size_t size,             // alignment requirement of field
-        target_size_t *poffset)
+        unsigned salign,        // struct alignment that is in effect
+        unsigned size,          // alignment requirement of field
+        unsigned *poffset)
 {
     //printf("salign = %d, size = %d, offset = %d\n",salign,size,offset);
     if (salign > 1)
     {
         assert(size != 3);
-        int sa = size;
+        unsigned sa = size;
         if (sa == 0 || salign < sa)
             sa = salign;
         *poffset = (*poffset + sa - 1) & ~(sa - 1);
@@ -164,9 +162,9 @@ void AggregateDeclaration::alignmember(
 
 void AggregateDeclaration::addField(Scope *sc, VarDeclaration *v)
 {
-    target_size_t memsize;      // size of member
-    target_size_t memalignsize; // size of member for alignment purposes
-    target_size_t xalign;       // alignment boundaries
+    unsigned memsize;           // size of member
+    unsigned memalignsize;      // size of member for alignment purposes
+    unsigned xalign;            // alignment boundaries
 
     //printf("AggregateDeclaration::addField('%s') %s\n", v->toChars(), toChars());
 
@@ -205,7 +203,7 @@ void AggregateDeclaration::addField(Scope *sc, VarDeclaration *v)
     if (sc->offset > structsize)
         structsize = sc->offset;
 #else
-    target_size_t ofs = sc->offset;
+    unsigned ofs = sc->offset;
     alignmember(xalign, memalignsize, &ofs);
     v->offset = ofs;
     ofs += memsize;
@@ -214,17 +212,71 @@ void AggregateDeclaration::addField(Scope *sc, VarDeclaration *v)
     if (!isUnionDeclaration())
         sc->offset = ofs;
 #endif
-    if (sc->structalign < memalignsize)
+    if (global.params.is64bit && sc->structalign == 8 && memalignsize == 16)
+        /* Not sure how to handle this */
+        ;
+    else if (sc->structalign < memalignsize)
         memalignsize = sc->structalign;
     if (alignsize < memalignsize)
         alignsize = memalignsize;
-    //printf("\talignsize = %d\n", alignsize);
+    //printf("\t%s: alignsize = %d\n", toChars(), alignsize);
 
     v->storage_class |= STCfield;
     //printf(" addField '%s' to '%s' at offset %d, size = %d\n", v->toChars(), toChars(), v->offset, memsize);
     fields.push(v);
 }
 
+/****************************************
+ * If field[indx] is not part of a union, return indx.
+ * Otherwise, return the lowest field index of the union.
+ */
+int AggregateDeclaration::firstFieldInUnion(int indx)
+{
+    if (isUnionDeclaration())
+        return 0;
+    VarDeclaration * vd = (VarDeclaration *)fields.data[indx];
+    int firstNonZero = indx; // first index in the union with non-zero size
+    for (; ;)
+    {
+        if (indx == 0)
+            return firstNonZero;
+        VarDeclaration * v = (VarDeclaration *)fields.data[indx - 1];
+        if (v->offset != vd->offset)
+            return firstNonZero;
+        --indx;
+        /* If it is a zero-length field, it's ambiguous: we don't know if it is
+         * in the union unless we find an earlier non-zero sized field with the
+         * same offset.
+         */
+        if (v->size(loc) != 0)
+            firstNonZero = indx;
+    }
+}
+
+/****************************************
+ * Count the number of fields starting at firstIndex which are part of the
+ * same union as field[firstIndex]. If not a union, return 1.
+ */
+int AggregateDeclaration::numFieldsInUnion(int firstIndex)
+{
+    VarDeclaration * vd = (VarDeclaration *)fields.data[firstIndex];
+    /* If it is a zero-length field, AND we can't find an earlier non-zero
+     * sized field with the same offset, we assume it's not part of a union.
+     */
+    if (vd->size(loc) == 0 && !isUnionDeclaration() &&
+        firstFieldInUnion(firstIndex) == firstIndex)
+        return 1;
+    int count = 1;
+    for (size_t i = firstIndex+1; i < fields.dim; ++i)
+    {
+        VarDeclaration * v = (VarDeclaration *)fields.data[i];
+        // If offsets are different, they are not in the same union
+        if (v->offset != vd->offset)
+            break;
+        ++count;
+    }
+    return count;
+}
 
 /********************************* StructDeclaration ****************************/
 
@@ -236,6 +288,7 @@ StructDeclaration::StructDeclaration(Loc loc, Identifier *id)
     hasIdentityAssign = 0;
     cpctor = NULL;
     postblit = NULL;
+    eq = NULL;
 #endif
 
     // For forward references
@@ -255,12 +308,12 @@ Dsymbol *StructDeclaration::syntaxCopy(Dsymbol *s)
 }
 
 void StructDeclaration::semantic(Scope *sc)
-{   int i;
+{
     Scope *sc2;
 
-    //printf("+StructDeclaration::semantic(this=%p, '%s')\n", this, toChars());
+    //printf("+StructDeclaration::semantic(this=%p, '%s', sizeok = %d)\n", this, toChars(), sizeok);
 
-    //static int count; if (++count == 20) *(char*)0=0;
+    //static int count; if (++count == 20) halt();
 
     assert(type);
     if (!members)                       // if forward reference
@@ -271,7 +324,7 @@ void StructDeclaration::semantic(Scope *sc)
         {   //printf("already completed\n");
             scope = NULL;
             return;             // semantic() already completed
-    }
+        }
     }
     else
         symtab = new DsymbolTable();
@@ -284,7 +337,6 @@ void StructDeclaration::semantic(Scope *sc)
     }
 
     unsigned dprogress_save = Module::dprogress;
-
 #ifdef IN_GCC
     methods.setDim(0);
 #endif
@@ -309,14 +361,16 @@ void StructDeclaration::semantic(Scope *sc)
     else if (storage_class & STCconst)
         type = type->constOf();
 #endif
+#if IN_GCC
     if (attributes)
         attributes->append(sc->attributes);
     else
         attributes = sc->attributes;
+#endif
 
     if (sizeok == 0)            // if not already done the addMember step
     {
-        for (i = 0; i < members->dim; i++)
+        for (size_t i = 0; i < members->dim; i++)
         {
             Dsymbol *s = (Dsymbol *)members->data[i];
             //printf("adding member '%s' to '%s'\n", s->toChars(), this->toChars());
@@ -327,19 +381,21 @@ void StructDeclaration::semantic(Scope *sc)
     sizeok = 0;
     sc2 = sc->push(this);
     sc2->stc = 0;
+#if IN_GCC
     sc2->attributes = NULL;
+#endif
     sc2->parent = this;
     if (isUnionDeclaration())
         sc2->inunion = 1;
     sc2->protection = PROTpublic;
     sc2->explicitProtection = 0;
 
-    int members_dim = members->dim;
+    size_t members_dim = members->dim;
 
     /* Set scope so if there are forward references, we still might be able to
      * resolve individual members like enums.
      */
-    for (int i = 0; i < members_dim; i++)
+    for (size_t i = 0; i < members_dim; i++)
     {   Dsymbol *s = (Dsymbol *)members->data[i];
         /* There are problems doing this in the general case because
          * Scope keeps track of things like 'offset'
@@ -351,7 +407,7 @@ void StructDeclaration::semantic(Scope *sc)
         }
     }
 
-    for (i = 0; i < members_dim; i++)
+    for (size_t i = 0; i < members_dim; i++)
     {
         Dsymbol *s = (Dsymbol *)members->data[i];
         s->semantic(sc2);
@@ -363,6 +419,10 @@ void StructDeclaration::semantic(Scope *sc)
 #endif
     }
 
+#if DMDV1
+    /* This doesn't work for DMDV2 because (ref S) and (S) parameter
+     * lists will overload the same.
+     */
     /* The TypeInfo_Struct is expecting an opEquals and opCmp with
      * a parameter that is a pointer to the struct. But if there
      * isn't one, but is an opEquals or opCmp with a value, write
@@ -420,7 +480,42 @@ void StructDeclaration::semantic(Scope *sc)
 
         id = Id::cmp;
     }
+#endif
 #if DMDV2
+    /* Try to find the opEquals function. Build it if necessary.
+     */
+    TypeFunction *tfeqptr;
+    {   // bool opEquals(const T*) const;
+        Parameters *parameters = new Parameters;
+#if STRUCTTHISREF
+        // bool opEquals(ref const T) const;
+        Parameter *param = new Parameter(STCref, type->constOf(), NULL, NULL);
+#else
+        // bool opEquals(const T*) const;
+        Parameter *param = new Parameter(STCin, type->pointerTo(), NULL, NULL);
+#endif
+
+        parameters->push(param);
+        tfeqptr = new TypeFunction(parameters, Type::tbool, 0, LINKd);
+        tfeqptr->mod = MODconst;
+        tfeqptr = (TypeFunction *)tfeqptr->semantic(0, sc2);
+
+        Dsymbol *s = search_function(this, Id::eq);
+        FuncDeclaration *fdx = s ? s->isFuncDeclaration() : NULL;
+        if (fdx)
+        {
+            eq = fdx->overloadExactMatch(tfeqptr);
+            if (!eq)
+                fdx->error("type signature should be %s not %s", tfeqptr->toChars(), fdx->type->toChars());
+        }
+
+        TemplateDeclaration *td = s ? s->isTemplateDeclaration() : NULL;
+        // BUG: should also check that td is a function template, not just a template
+
+        if (!eq && !td)
+            eq = buildOpEquals(sc2);
+    }
+
     dtor = buildDtor(sc2);
     postblit = buildPostBlit(sc2);
     cpctor = buildCpCtor(sc2);
@@ -465,7 +560,7 @@ void StructDeclaration::semantic(Scope *sc)
 
     // Determine if struct is all zeros or not
     zeroInit = 1;
-    for (i = 0; i < fields.dim; i++)
+    for (size_t i = 0; i < fields.dim; i++)
     {
         Dsymbol *s = (Dsymbol *)fields.data[i];
         VarDeclaration *vd = s->isVarDeclaration();
@@ -491,7 +586,7 @@ void StructDeclaration::semantic(Scope *sc)
     /* Look for special member functions.
      */
 #if DMDV2
-    ctor =   (CtorDeclaration *)search(0, Id::ctor, 0);
+    ctor = search(0, Id::ctor, 0);
 #endif
     inv =    (InvariantDeclaration *)search(0, Id::classInvariant, 0);
     aggNew =       (NewDeclaration *)search(0, Id::classNew,       0);
@@ -508,7 +603,7 @@ Dsymbol *StructDeclaration::search(Loc loc, Identifier *ident, int flags)
 {
     //printf("%s.StructDeclaration::search('%s')\n", toChars(), ident->toChars());
 
-    if (scope)
+    if (scope && !symtab)
         semantic(scope);
 
     if (!members || !symtab)
@@ -521,8 +616,7 @@ Dsymbol *StructDeclaration::search(Loc loc, Identifier *ident, int flags)
 }
 
 void StructDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
-{   int i;
-
+{
     buf->printf("%s ", kind());
     if (!isAnonymous())
         buf->writestring(toChars());
@@ -535,7 +629,7 @@ void StructDeclaration::toCBuffer(OutBuffer *buf, HdrGenState *hgs)
     buf->writenl();
     buf->writeByte('{');
     buf->writenl();
-    for (i = 0; i < members->dim; i++)
+    for (size_t i = 0; i < members->dim; i++)
     {
         Dsymbol *s = (Dsymbol *)members->data[i];
 
