@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -22,17 +22,12 @@
  * be written to the data segment.
  */
 
-#undef integer_t
 #include        <stdio.h>
 #include        <string.h>
 #include        <time.h>
 #include        <assert.h>
 #include        <complex.h>
 
-#ifdef __APPLE__
-#define integer_t dmd_integer_t
-#endif
-
 #include        "lexer.h"
 #include        "mtype.h"
 #include        "expression.h"
@@ -58,38 +53,36 @@ extern Symbol *static_sym();
 /* ================================================================ */
 
 #ifdef IN_GCC
-static dt_t *createTsarrayDt(dt_t * elem_or_all, Type *t)
+static dt_t *createTsarrayDt(dt_t * dt, Type *t)
 {
-    assert(elem_or_all != NULL);
-    target_size_t eoa_size = dt_size(elem_or_all);
+    assert(dt != NULL);
+    size_t eoa_size = dt_size(dt);
     if (eoa_size == t->size())
-    {
-        return elem_or_all;
-    }
+        return dt;
     else
     {
         TypeSArray * tsa = (TypeSArray *) t->toBasetype();
         assert(tsa->ty == Tsarray);
 
-        target_size_t dim = tsa->dim->toInteger();
+        size_t dim = tsa->dim->toInteger();
         dt_t * adt = NULL;
         dt_t ** padt = & adt;
 
         if (eoa_size * dim == eoa_size)
         {
-            for (target_size_t i = 0; i < dim; i++)
-                padt = dtcontainer(padt, NULL, elem_or_all);
+            for (size_t i = 0; i < dim; i++)
+                padt = dtcontainer(padt, NULL, dt);
         }
         else
         {
             assert(tsa->size(0) % eoa_size == 0);
-            for (target_size_t i = 0; i < dim; i++)
+            for (size_t i = 0; i < dim; i++)
                 padt = dtcontainer(padt, NULL,
-                    createTsarrayDt(elem_or_all, tsa->next));
+                    createTsarrayDt(dt, tsa->next));
         }
-        dt_t * fdt = NULL;
-        dtcontainer(& fdt, t, adt);
-        return fdt;
+        dt_t * cdt = NULL;
+        dtcontainer(& cdt, t, adt);
+        return cdt;
     }
 }
 #endif
@@ -116,25 +109,23 @@ dt_t *VoidInitializer::toDt()
 dt_t *StructInitializer::toDt()
 {
     Array dts;
-    unsigned i;
-    unsigned j;
     dt_t *dt;
     dt_t *d;
     dt_t **pdtend;
-    target_size_t offset;
+    unsigned offset;
 
     //printf("StructInitializer::toDt('%s')\n", toChars());
     dts.setDim(ad->fields.dim);
     dts.zero();
 
-    for (i = 0; i < vars.dim; i++)
+    for (size_t i = 0; i < vars.dim; i++)
     {
         VarDeclaration *v = (VarDeclaration *)vars.data[i];
         Initializer *val = (Initializer *)value.data[i];
 
         //printf("vars[%d] = %s\n", i, v->toChars());
 
-        for (j = 0; 1; j++)
+        for (size_t j = 0; 1; j++)
         {
             assert(j < dts.dim);
             //printf(" adfield[%d] = %s\n", j, ((VarDeclaration *)ad->fields.data[j])->toChars());
@@ -151,7 +142,7 @@ dt_t *StructInitializer::toDt()
     dt = NULL;
     pdtend = &dt;
     offset = 0;
-    for (j = 0; j < dts.dim; j++)
+    for (size_t j = 0; j < dts.dim; j++)
     {
         VarDeclaration *v = (VarDeclaration *)ad->fields.data[j];
 
@@ -160,16 +151,14 @@ dt_t *StructInitializer::toDt()
         {   // An instance specific initializer was not provided.
             // Look to see if there's a default initializer from the
             // struct definition
-            VarDeclaration *v = (VarDeclaration *)ad->fields.data[j];
-
             if (v->init)
             {
                 d = v->init->toDt();
             }
             else if (v->offset >= offset)
             {
-                target_size_t k;
-                target_size_t offset2 = v->offset + v->type->size();
+                unsigned k;
+                unsigned offset2 = v->offset + v->type->size();
                 // Make sure this field does not overlap any explicitly
                 // initialized field.
                 for (k = j + 1; 1; k++)
@@ -191,23 +180,16 @@ dt_t *StructInitializer::toDt()
             if (v->offset < offset)
                 error(loc, "duplicate union initialization for %s", v->toChars());
             else
-            {   target_size_t sz = dt_size(d);
-                target_size_t vsz = v->type->size();
-                target_size_t voffset = v->offset;
+            {   size_t sz = dt_size(d);
+                size_t vsz = v->type->size();
+                size_t voffset = v->offset;
 
-#ifdef IN_GCC
-                if (offset < voffset)
-                    pdtend = dtnzeros(pdtend, voffset - offset);
-                if (v->type->toBasetype()->ty == Tsarray)
-                {
-                    d = createTsarrayDt(d, v->type);
-                    sz = dt_size(d);
-                    assert(sz <= vsz);
+                if (sz > vsz)
+                {   assert(v->type->ty == Tsarray && vsz == 0);
+                    error(loc, "zero length array %s has non-zero length initializer", v->toChars());
                 }
-                pdtend = dtcat(pdtend, d);
-                offset = voffset + sz;
-#else
-                target_size_t dim = 1;
+
+                unsigned dim = 1;
                 for (Type *vt = v->type->toBasetype();
                      vt->ty == Tsarray;
                      vt = vt->next->toBasetype())
@@ -217,7 +199,7 @@ dt_t *StructInitializer::toDt()
                 //printf("sz = %d, dim = %d, vsz = %d\n", sz, dim, vsz);
                 assert(sz == vsz || sz * dim <= vsz);
 
-                for (target_size_t i = 0; i < dim; i++)
+                for (size_t i = 0; i < dim; i++)
                 {
                     if (offset < voffset)
                         pdtend = dtnzeros(pdtend, voffset - offset);
@@ -235,7 +217,6 @@ dt_t *StructInitializer::toDt()
                     if (sz == vsz)
                         break;
                 }
-#endif
             }
         }
     }
@@ -244,7 +225,7 @@ dt_t *StructInitializer::toDt()
 
 #ifdef IN_GCC
     dt_t * cdt = NULL;
-    dtcontainer(& cdt, ad->type, dt);
+    dtcontainer(&cdt, ad->type, dt);
     dt = cdt;
 #endif
     return dt;
@@ -263,7 +244,6 @@ dt_t *ArrayInitializer::toDt()
     Array dts;
     unsigned size;
     unsigned length;
-    unsigned i;
     dt_t *dt;
     dt_t *d;
     dt_t **pdtend;
@@ -275,7 +255,7 @@ dt_t *ArrayInitializer::toDt()
     size = tn->size();
 
     length = 0;
-    for (i = 0; i < index.dim; i++)
+    for (size_t i = 0; i < index.dim; i++)
     {   Expression *idx;
         Initializer *val;
 
@@ -289,8 +269,6 @@ dt_t *ArrayInitializer::toDt()
         dt = val->toDt();
         if (dts.data[length])
             error(loc, "duplicate initializations for index %d", length);
-        if (tn->ty == Tsarray)
-            dt = createTsarrayDt(dt, tb->next);
         dts.data[length] = (void *)dt;
         length++;
     }
@@ -314,7 +292,7 @@ dt_t *ArrayInitializer::toDt()
 
     d = NULL;
     pdtend = &d;
-    for (i = 0; i < dim; i++)
+    for (size_t i = 0; i < dim; i++)
     {
         dt = (dt_t *)dts.data[i];
 #ifdef IN_GCC
@@ -324,7 +302,7 @@ dt_t *ArrayInitializer::toDt()
             pdtend = dtcat(pdtend, dt);
         else
         {
-            for (int j = 0; j < n; j++)
+            for (size_t j = 0; j < n; j++)
                 pdtend = edefault->toDt(pdtend);
         }
 #endif
@@ -344,11 +322,11 @@ dt_t *ArrayInitializer::toDt()
                     pdtend = dtnzeros(pdtend, size * (tadim - dim));
                 else
                 {
-                    for (i = dim; i < tadim; i++)
+                    for (size_t i = dim; i < tadim; i++)
 #ifdef IN_GCC
                         pdtend = dtcontainer(pdtend, NULL, sadefault);
 #else
-                    {   for (int j = 0; j < n; j++)
+                    {   for (size_t j = 0; j < n; j++)
                             pdtend = edefault->toDt(pdtend);
                     }
 #endif
@@ -371,15 +349,14 @@ dt_t *ArrayInitializer::toDt()
 
         case Tpointer:
         case Tarray:
-            // Create symbol, and then refer to it
-            Symbol *s;
-            s = static_sym();
+        {   // Create symbol, and then refer to it
+            Symbol *s = static_sym();
             s->Sdt = d;
             outdata(s);
 
             d = NULL;
             if (tb->ty == Tarray)
-                dtdword(&d, dim);
+                dtsize_t(&d, dim);
             dtxoff(&d, s, 0, TYnptr);
 #ifdef IN_GCC
             dt_t * cdt;
@@ -391,6 +368,7 @@ dt_t *ArrayInitializer::toDt()
             }
 #endif
             break;
+        }
 
         default:
             assert(0);
@@ -404,12 +382,13 @@ dt_t *ArrayInitializer::toDtBit()
 #if DMDV1
     unsigned size;
     unsigned length;
-    unsigned i;
     unsigned tadim;
     dt_t *d;
     dt_t **pdtend;
     Type *tb = type->toBasetype();
 
+    //printf("ArrayInitializer::toDtBit('%s')\n", toChars());
+
     Bits databits;
     Bits initbits;
 
@@ -440,7 +419,7 @@ dt_t *ArrayInitializer::toDtBit()
     size = sizeof(databits.data[0]);
 
     length = 0;
-    for (i = 0; i < index.dim; i++)
+    for (size_t i = 0; i < index.dim; i++)
     {   Expression *idx;
         Initializer *val;
         Expression *eval;
@@ -481,9 +460,6 @@ dt_t *ArrayInitializer::toDtBit()
         {
             if (dim > tadim)
             {
-#ifdef DEBUG
-                printf("2: ");
-#endif
                 error(loc, "too many initializers, %d, for array[%d]", dim, tadim);
             }
             else
@@ -505,7 +481,7 @@ dt_t *ArrayInitializer::toDtBit()
 
             d = NULL;
             if (tb->ty == Tarray)
-                dtdword(&d, dim);
+                dtsize_t(&d, dim);
             dtxoff(&d, s, 0, TYnptr);
             break;
 
@@ -521,6 +497,7 @@ dt_t *ArrayInitializer::toDtBit()
 
 dt_t *ExpInitializer::toDt()
 {
+    //printf("ExpInitializer::toDt() %s\n", exp->toChars());
     dt_t *dt = NULL;
 
     exp = exp->optimize(WANTvalue);
@@ -652,25 +629,26 @@ dt_t **StringExp::toDt(dt_t **pdt)
     switch (t->ty)
     {
         case Tarray:
-            dt_t * adt; adt = NULL;
-            dtdword(& adt, len);
-#ifndef IN_GCC
-            dtabytes(& adt, TYnptr, 0, (len + 1) * sz, (char *)string);
-            pdt = dcat(pdt, adt);
+            dt_t *adt; adt = NULL;
+            dtsize_t(&adt, len);
+#ifdef IN_GCC
+            dtawords(&adt, len + 1, string, sz);
+            dtcontainer(pdt, type, adt);
 #else
-            dtawords(& adt, len + 1, string, sz);
-            pdt = dtcontainer(pdt, type, adt);
+            dtabytes(&adt, TYnptr, 0, (len + 1) * sz, (char *)string);
+            dtcat(pdt, adt);
 #endif
+            
             break;
 
         case Tsarray:
         {   TypeSArray *tsa = (TypeSArray *)type;
             dinteger_t dim;
 
-#ifndef IN_GCC
-            pdt = dtnbytes(pdt, len * sz, (const char *)string);
-#else
+#ifdef IN_GCC
             pdt = dtnwords(pdt, len, string, sz);
+#else
+            pdt = dtnbytes(pdt, len * sz, (const char *)string);
 #endif
             if (tsa->dim)
             {
@@ -684,10 +662,10 @@ dt_t **StringExp::toDt(dt_t **pdt)
             break;
         }
         case Tpointer:
-#ifndef IN_GCC
-            pdt = dtabytes(pdt, TYnptr, 0, (len + 1) * sz, (char *)string);
-#else
+#ifdef IN_GCC
             pdt = dtawords(pdt, len + 1, string, sz);
+#else
+            pdt = dtabytes(pdt, TYnptr, 0, (len + 1) * sz, (char *)string);
 #endif
             break;
 
@@ -707,14 +685,14 @@ dt_t **ArrayLiteralExp::toDt(dt_t **pdt)
 
     d = NULL;
     pdtend = &d;
-    for (int i = 0; i < elements->dim; i++)
+    for (size_t i = 0; i < elements->dim; i++)
     {   Expression *e = (Expression *)elements->data[i];
 
         pdtend = e->toDt(pdtend);
     }
 #ifdef IN_GCC
-    dt_t * cdt = NULL;
-    dtcontainer(& cdt, type, d);
+    dt_t *cdt = NULL;
+    dtcontainer(&cdt, type, d);
     d = cdt;
 #endif
     Type *t = type->toBasetype();
@@ -727,9 +705,9 @@ dt_t **ArrayLiteralExp::toDt(dt_t **pdt)
 
         case Tpointer:
         case Tarray:
-            dt_t * adt; adt = NULL;
+            dt_t *adt; adt = NULL;
             if (t->ty == Tarray)
-                dtdword(& adt, elements->dim);
+                dtsize_t(&adt, elements->dim);
             if (d)
             {
                 // Create symbol, and then refer to it
@@ -738,10 +716,10 @@ dt_t **ArrayLiteralExp::toDt(dt_t **pdt)
                 s->Sdt = d;
                 outdata(s);
 
-                dtxoff(& adt, s, 0, TYnptr);
+                dtxoff(&adt, s, 0, TYnptr);
             }
             else
-                dtdword(& adt, 0);
+                dtsize_t(&adt, 0);
 #ifdef IN_GCC
             if (t->ty == Tarray)
                 dtcontainer(pdt, type, adt);
@@ -760,19 +738,17 @@ dt_t **ArrayLiteralExp::toDt(dt_t **pdt)
 dt_t **StructLiteralExp::toDt(dt_t **pdt)
 {
     Array dts;
-    unsigned i;
-    unsigned j;
     dt_t *dt;
     dt_t *d;
     dt_t *sdt = NULL;
-    target_size_t offset;
+    unsigned  offset;
 
     //printf("StructLiteralExp::toDt() %s)\n", toChars());
     dts.setDim(sd->fields.dim);
     dts.zero();
     assert(elements->dim <= sd->fields.dim);
 
-    for (i = 0; i < elements->dim; i++)
+    for (size_t i = 0; i < elements->dim; i++)
     {
         Expression *e = (Expression *)elements->data[i];
         if (!e)
@@ -783,7 +759,7 @@ dt_t **StructLiteralExp::toDt(dt_t **pdt)
     }
 
     offset = 0;
-    for (j = 0; j < dts.dim; j++)
+    for (size_t j = 0; j < dts.dim; j++)
     {
         VarDeclaration *v = (VarDeclaration *)sd->fields.data[j];
 
@@ -792,19 +768,16 @@ dt_t **StructLiteralExp::toDt(dt_t **pdt)
         {   // An instance specific initializer was not provided.
             // Look to see if there's a default initializer from the
             // struct definition
-            VarDeclaration *v = (VarDeclaration *)sd->fields.data[j];
-
             if (v->init)
             {
                 d = v->init->toDt();
             }
             else if (v->offset >= offset)
             {
-                target_size_t k;
-                target_size_t offset2 = v->offset + v->type->size();
+                unsigned offset2 = v->offset + v->type->size();
                 // Make sure this field (v) does not overlap any explicitly
                 // initialized field.
-                for (k = j + 1; 1; k++)
+                for (size_t k = j + 1; 1; k++)
                 {
                     if (k == dts.dim)           // didn't find any overlap
                     {
@@ -823,24 +796,16 @@ dt_t **StructLiteralExp::toDt(dt_t **pdt)
             if (v->offset < offset)
                 error("duplicate union initialization for %s", v->toChars());
             else
-            {   target_size_t sz = dt_size(d);
-                target_size_t vsz = v->type->size();
-                target_size_t voffset = v->offset;
-                assert(sz <= vsz);
+            {   size_t sz = dt_size(d);
+                size_t vsz = v->type->size();
+                size_t voffset = v->offset;
 
-#ifdef IN_GCC
-                if (offset < voffset)
-                    dtnzeros(& sdt, voffset - offset);
-                if (v->type->toBasetype()->ty == Tsarray)
-                {
-                    d = createTsarrayDt(d, v->type);
-                    sz = dt_size(d);
-                    assert(sz <= vsz);
+                if (sz > vsz)
+                {   assert(v->type->ty == Tsarray && vsz == 0);
+                    error("zero length array %s has non-zero length initializer", v->toChars());
                 }
-                dtcat(& sdt, d);
-                offset = voffset + sz;
-#else
-                target_size_t dim = 1;
+
+                unsigned dim = 1;
                 Type *vt;
                 for (vt = v->type->toBasetype();
                      vt->ty == Tsarray;
@@ -849,10 +814,13 @@ dt_t **StructLiteralExp::toDt(dt_t **pdt)
                     dim *= tsa->dim->toInteger();
                 }
 
-                for (target_size_t i = 0; i < dim; i++)
+                //printf("sz = %d, dim = %d, vsz = %d\n", sz, dim, vsz);
+                assert(sz == vsz || sz * dim <= vsz);
+
+                for (size_t i = 0; i < dim; i++)
                 {
                     if (offset < voffset)
-                        dtnzeros(& sdt, voffset - offset);
+                        dtnzeros(&sdt, voffset - offset);
                     if (!d)
                     {
                         if (v->init)
@@ -860,25 +828,23 @@ dt_t **StructLiteralExp::toDt(dt_t **pdt)
                         else
                             vt->toDt(&d);
                     }
-                    dtcat(& sdt, d);
+                    dtcat(&sdt, d);
                     d = NULL;
                     offset = voffset + sz;
                     voffset += vsz / dim;
                     if (sz == vsz)
                         break;
                 }
-#endif
             }
         }
     }
     if (offset < sd->structsize)
-        dtnzeros(& sdt, sd->structsize - offset);
+        dtnzeros(&sdt, sd->structsize - offset);
 #ifdef IN_GCC
     dtcontainer(pdt, type, sdt);
 #else
-    pdt = dtcat(pdt, sdt);
+    dtcat(pdt, sdt);
 #endif
-
     return pdt;
 }
 
@@ -910,7 +876,7 @@ dt_t **VarExp::toDt(dt_t **pdt)
     VarDeclaration *v = var->isVarDeclaration();
     if (v && v->isConst() && type->toBasetype()->ty != Tsarray && v->init)
     {
-    if (v->inuse)
+        if (v->inuse)
         {
             error("recursive reference %s", toChars());
             return pdt;
@@ -934,6 +900,18 @@ dt_t **VarExp::toDt(dt_t **pdt)
     return pdt;
 }
 
+dt_t **FuncExp::toDt(dt_t **pdt)
+{
+    //printf("FuncExp::toDt() %d\n", op);
+    Symbol *s = fd->toSymbol();
+    if (fd->isNested())
+    {   error("non-constant nested delegate literal expression %s", toChars());
+        return NULL;
+    }
+    fd->toObjFile(0);
+    return dtxoff(pdt, s, 0, TYnptr);
+}
+
 /* ================================================================= */
 
 // Generate the data for the static initializer.
@@ -944,7 +922,7 @@ void ClassDeclaration::toDt(dt_t **pdt)
 
     // Put in first two members, the vtbl[] and the monitor
     dtxoff(pdt, toVtblSymbol(), 0, TYnptr);
-    dtdword(pdt, 0);                    // monitor
+    dtsize_t(pdt, 0);                    // monitor
 
     // Put in the rest
     toDt2(pdt, this);
@@ -955,7 +933,6 @@ void ClassDeclaration::toDt(dt_t **pdt)
 void ClassDeclaration::toDt2(dt_t **pdt, ClassDeclaration *cd)
 {
     unsigned offset;
-    unsigned i;
     dt_t *dt;
     unsigned csymoffset;
 
@@ -975,7 +952,7 @@ void ClassDeclaration::toDt2(dt_t **pdt, ClassDeclaration *cd)
     }
 
     // Note equivalence of this loop to struct's
-    for (i = 0; i < fields.dim; i++)
+    for (size_t i = 0; i < fields.dim; i++)
     {
         VarDeclaration *v = (VarDeclaration *)fields.data[i];
         Initializer *init;
@@ -988,14 +965,7 @@ void ClassDeclaration::toDt2(dt_t **pdt, ClassDeclaration *cd)
             ExpInitializer *ei = init->isExpInitializer();
             Type *tb = v->type->toBasetype();
             if (ei && tb->ty == Tsarray)
-            {
-#ifdef IN_GCC
-                dt = init->toDt();
-                dt = createTsarrayDt(dt, v->type);
-#else
                 ((TypeSArray *)tb)->toDtElem(&dt, ei->exp);
-#endif
-            }
             else
                 dt = init->toDt();
         }
@@ -1020,7 +990,7 @@ void ClassDeclaration::toDt2(dt_t **pdt, ClassDeclaration *cd)
     // Interface vptr initializations
     toSymbol();                                         // define csym
 
-    for (i = 0; i < vtblInterfaces->dim; i++)
+    for (size_t i = 0; i < vtblInterfaces->dim; i++)
     {   BaseClass *b = (BaseClass *)vtblInterfaces->data[i];
 
 #if 1 || INTERFACE_VIRTUAL
@@ -1052,16 +1022,13 @@ void ClassDeclaration::toDt2(dt_t **pdt, ClassDeclaration *cd)
 
 void StructDeclaration::toDt(dt_t **pdt)
 {
-#if 0
-    // %% ?
     if (zeroInit)
     {
         dtnzeros(pdt, structsize);
         return;
     }
-#endif
+
     unsigned offset;
-    unsigned i;
     dt_t *dt;
     dt_t *sdt = NULL;
 
@@ -1069,7 +1036,7 @@ void StructDeclaration::toDt(dt_t **pdt)
     offset = 0;
 
     // Note equivalence of this loop to class's
-    for (i = 0; i < fields.dim; i++)
+    for (size_t i = 0; i < fields.dim; i++)
     {
         VarDeclaration *v = (VarDeclaration *)fields.data[i];
         Initializer *init;
@@ -1082,14 +1049,7 @@ void StructDeclaration::toDt(dt_t **pdt)
             ExpInitializer *ei = init->isExpInitializer();
             Type *tb = v->type->toBasetype();
             if (ei && tb->ty == Tsarray)
-            {
-#ifdef IN_GCC
-                dt = init->toDt();
-                dt = createTsarrayDt(dt, v->type);
-#else
                 ((TypeSArray *)tb)->toDtElem(&dt, ei->exp);
-#endif
-            }
             else
                 dt = init->toDt();
         }
@@ -1102,21 +1062,20 @@ void StructDeclaration::toDt(dt_t **pdt)
             else
             {
                 if (offset < v->offset)
-                    dtnzeros(& sdt, v->offset - offset);
-                dtcat(& sdt, dt);
+                    dtnzeros(&sdt, v->offset - offset);
+                dtcat(&sdt, dt);
                 offset = v->offset + v->type->size();
             }
         }
     }
 
     if (offset < structsize)
-        dtnzeros(& sdt, structsize - offset);
+        dtnzeros(&sdt, structsize - offset);
 #ifdef IN_GCC
     dtcontainer(pdt, type, sdt);
 #else
     dtcat(pdt, sdt);
 #endif
-
     dt_optimize(*pdt);
 }
 
@@ -1136,7 +1095,6 @@ dt_t **TypeSArray::toDt(dt_t **pdt)
 
 dt_t **TypeSArray::toDtElem(dt_t **pdt, Expression *e)
 {
-    int i;
     unsigned len;
 
     //printf("TypeSArray::toDtElem()\n");
@@ -1147,74 +1105,53 @@ dt_t **TypeSArray::toDtElem(dt_t **pdt, Expression *e)
             pdt = &((*pdt)->DTnext);
         Type *tnext = next;
         Type *tbn = tnext->toBasetype();
-        while (tbn->ty == Tsarray)
+        while (tbn->ty == Tsarray && (!e || tbn != e->type->nextOf()))
         {   TypeSArray *tsa = (TypeSArray *)tbn;
 
             len *= tsa->dim->toInteger();
-            tnext = tbn->next;
+            tnext = tbn->nextOf();
             tbn = tnext->toBasetype();
         }
         if (!e)                         // if not already supplied
             e = tnext->defaultInit();   // use default initializer
-        if (tbn->ty == Tbit)
+        dt_t *adt = NULL;
+        dt_t **padt = & adt;
+        e->toDt(padt);
+        dt_optimize(*padt);
+        // These first four cases are OK for GDC
+        if (e->op == TOKstring)
+            len /= ((StringExp *)e)->len;
+        if (e->op == TOKarrayliteral)
+            len /= ((ArrayLiteralExp *)e)->elements->dim;
+        // These first two cases are okay for GDC too
+        if ((*padt)->dt == DT_azeros && !(*padt)->DTnext)
         {
-            Bits databits;
-
-            databits.resize(len);
-            if (e->toInteger())
-                databits.set();
-#ifdef IN_GCC
-            pdt = dtnbits(pdt, databits.allocdim * sizeof(databits.data[0]),
-                (char *)databits.data, sizeof(databits.data[0]));
-#else
-            pdt = dtnbytes(pdt, databits.allocdim * sizeof(databits.data[0]),
-                (char *)databits.data);
-#endif
+            (*padt)->DTazeros *= len;
+            pdt = dtcat(pdt, adt);
+        }
+        else if ((*padt)->dt == DT_1byte && (*padt)->DTonebyte == 0 && !(*padt)->DTnext)
+        {
+            (*padt)->dt = DT_azeros;
+            (*padt)->DTazeros = len;
+            pdt = dtcat(pdt, adt);
         }
         else
         {
-            dt_t *adt = NULL;
-            dt_t **padt = & adt;
-            /* problem...?
-            if (tbn->ty == Tstruct)
-                tnext->toDt(pdt);
-            else
-                e->toDt(pdt);
-            */
-            e->toDt(padt);
-            dt_optimize(*padt);
-
-            // These first two cases are okay for GDC too
-            if ((*padt)->dt == DT_azeros && !(*padt)->DTnext)
-            {
-                (*padt)->DTazeros *= len;
-                pdt = dtcat(pdt, adt);
-            }
-            else if ((*padt)->dt == DT_1byte && (*padt)->DTonebyte == 0 && !(*padt)->DTnext)
-            {
-                (*padt)->dt = DT_azeros;
-                (*padt)->DTazeros = len;
-                pdt = dtcat(pdt, adt);
-            }
-            else if (e->op != TOKstring)
-            {
-#ifdef IN_GCC
-                if(e->type->ty != Terror)
-                    pdt = dtcat(pdt, createTsarrayDt(adt, this));
+#if IN_GCC
+            pdt = dtcat(pdt, createTsarrayDt(adt, this));
 #else
-                for (i = 1; i < len; i++)
-                {
-                    if (tbn->ty == Tstruct)
-                    {   padt = tnext->toDt(padt);
-                        while (*padt)
-                            adt = &((*padt)->DTnext);
-                    }
-                    else
-                        padt = e->toDt(padt);
+            for (size_t i = 1; i < len; i++)
+            {
+                if (tbn->ty == Tstruct)
+                {   padt = tnext->toDt(padt);
+                    while (*padt)
+                        padt = &((*padt)->DTnext);
                 }
-                pdt = dtcat(pdt, adt);
-#endif
+                else
+                    padt = e->toDt(padt);
             }
+            pdt = dtcat(pdt, adt);
+#endif
         }
     }
     return pdt;
