@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2010 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -37,7 +37,6 @@
 #include "rmem.h"
 #endif
 #ifndef IN_GCC
-#ifndef TARGET_NET
 #include "cc.h"
 #include "global.h"
 #include "oper.h"
@@ -46,7 +45,6 @@
 #include "dt.h"
 #include "cgcv.h"
 #include "outbuf.h"
-#endif // TARGET_NET
 #include "irstate.h"
 #else
 #include "symbol.h"
@@ -76,7 +74,8 @@ Expression *Type::getInternalTypeInfo(Scope *sc)
     {
         case Tsarray:
 #if 0
-            t = t->next->arrayOf();     // convert to corresponding dynamic array type
+            // convert to corresponding dynamic array type
+            t = t->nextOf()->mutableOf()->arrayOf();
 #endif
             break;
 
@@ -135,6 +134,8 @@ Expression *Type::getTypeInfo(Scope *sc)
             t->vtinfo = new TypeInfoConstDeclaration(t);
         else if (t->isImmutable())
             t->vtinfo = new TypeInfoInvariantDeclaration(t);
+        else if (t->isWild())
+            t->vtinfo = new TypeInfoWildDeclaration(t);
         else
 #endif
             t->vtinfo = t->getTypeInfoDeclaration();
@@ -157,6 +158,8 @@ Expression *Type::getTypeInfo(Scope *sc)
             }
         }
     }
+    if (!vtinfo)
+        vtinfo = t->vtinfo;     // Types aren't merged, but we can share the vtinfo's
     Expression *e = new VarExp(0, t->vtinfo);
     e = e->addressOf(sc);
     e->type = t->vtinfo->type;          // do this so we don't get redundant dereference
@@ -237,7 +240,7 @@ void TypeInfoDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoDeclaration::toDt() %s\n", toChars());
     dtxoff(pdt, Type::typeinfo->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 }
 
 #if DMDV2
@@ -245,7 +248,7 @@ void TypeInfoConstDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoConstDeclaration::toDt() %s\n", toChars());
     dtxoff(pdt, Type::typeinfoconst->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Const
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
     Type *tm = tinfo->mutableOf();
     tm = tm->merge();
     tm->getTypeInfo(NULL);
@@ -256,7 +259,7 @@ void TypeInfoInvariantDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoInvariantDeclaration::toDt() %s\n", toChars());
     dtxoff(pdt, Type::typeinfoinvariant->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Invariant
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
     Type *tm = tinfo->mutableOf();
     tm = tm->merge();
     tm->getTypeInfo(NULL);
@@ -267,16 +270,24 @@ void TypeInfoSharedDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoSharedDeclaration::toDt() %s\n", toChars());
     dtxoff(pdt, Type::typeinfoshared->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Shared
-    dtdword(pdt, 0);                        // monitor
-    Type *tm;
-    if (tinfo->isConst())               // it was 'shared const'
-        tm = tinfo->constOf();
-    else                                // it was just 'shared'
-        tm = tinfo->mutableOf();
+    dtsize_t(pdt, 0);                        // monitor
+    Type *tm = tinfo->unSharedOf();
     tm = tm->merge();
     tm->getTypeInfo(NULL);
     dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
 }
+
+void TypeInfoWildDeclaration::toDt(dt_t **pdt)
+{
+    //printf("TypeInfoWildDeclaration::toDt() %s\n", toChars());
+    dtxoff(pdt, Type::typeinfowild->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Wild
+    dtsize_t(pdt, 0);                        // monitor
+    Type *tm = tinfo->mutableOf();
+    tm = tm->merge();
+    tm->getTypeInfo(NULL);
+    dtxoff(pdt, tm->vtinfo->toSymbol(), 0, TYnptr);
+}
+
 #endif
 
 void TypeInfoTypedefDeclaration::toDt(dt_t **pdt)
@@ -284,7 +295,7 @@ void TypeInfoTypedefDeclaration::toDt(dt_t **pdt)
     //printf("TypeInfoTypedefDeclaration::toDt() %s\n", toChars());
 
     dtxoff(pdt, Type::typeinfotypedef->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Typedef
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Ttypedef);
 
@@ -305,18 +316,18 @@ void TypeInfoTypedefDeclaration::toDt(dt_t **pdt)
 
     const char *name = sd->toPrettyChars();
     size_t namelen = strlen(name);
-    dtdword(pdt, namelen);
+    dtsize_t(pdt, namelen);
     dtabytes(pdt, TYnptr, 0, namelen + 1, name);
 
     // void[] init;
     if (tinfo->isZeroInit() || !sd->init)
     {   // 0 initializer, or the same as the base type
-        dtdword(pdt, 0);        // init.length
-        dtdword(pdt, 0);        // init.ptr
+        dtsize_t(pdt, 0);        // init.length
+        dtsize_t(pdt, 0);        // init.ptr
     }
     else
     {
-        dtdword(pdt, sd->type->size()); // init.length
+        dtsize_t(pdt, sd->type->size()); // init.length
         dtxoff(pdt, sd->toInitializer(), 0, TYnptr);    // init.ptr
     }
 }
@@ -325,7 +336,7 @@ void TypeInfoEnumDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoEnumDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfoenum->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Enum
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tenum);
 
@@ -343,18 +354,18 @@ void TypeInfoEnumDeclaration::toDt(dt_t **pdt)
 
     const char *name = sd->toPrettyChars();
     size_t namelen = strlen(name);
-    dtdword(pdt, namelen);
+    dtsize_t(pdt, namelen);
     dtabytes(pdt, TYnptr, 0, namelen + 1, name);
 
     // void[] init;
     if (!sd->defaultval || tinfo->isZeroInit())
     {   // 0 initializer, or the same as the base type
-        dtdword(pdt, 0);        // init.length
-        dtdword(pdt, 0);        // init.ptr
+        dtsize_t(pdt, 0);        // init.length
+        dtsize_t(pdt, 0);        // init.ptr
     }
     else
     {
-        dtdword(pdt, sd->type->size()); // init.length
+        dtsize_t(pdt, sd->type->size()); // init.length
         dtxoff(pdt, sd->toInitializer(), 0, TYnptr);    // init.ptr
     }
 }
@@ -363,7 +374,7 @@ void TypeInfoPointerDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoPointerDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfopointer->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Pointer
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tpointer);
 
@@ -377,7 +388,7 @@ void TypeInfoArrayDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoArrayDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfoarray->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Array
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tarray);
 
@@ -391,7 +402,7 @@ void TypeInfoStaticArrayDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoStaticArrayDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfostaticarray->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_StaticArray
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tsarray);
 
@@ -400,14 +411,14 @@ void TypeInfoStaticArrayDeclaration::toDt(dt_t **pdt)
     tc->next->getTypeInfo(NULL);
     dtxoff(pdt, tc->next->vtinfo->toSymbol(), 0, TYnptr); // TypeInfo for array of type
 
-    dtdword(pdt, tc->dim->toInteger());         // length
+    dtsize_t(pdt, tc->dim->toInteger());         // length
 }
 
 void TypeInfoAssociativeArrayDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoAssociativeArrayDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfoassociativearray->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_AssociativeArray
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Taarray);
 
@@ -418,13 +429,18 @@ void TypeInfoAssociativeArrayDeclaration::toDt(dt_t **pdt)
 
     tc->index->getTypeInfo(NULL);
     dtxoff(pdt, tc->index->vtinfo->toSymbol(), 0, TYnptr); // TypeInfo for array of type
+
+#if DMDV2
+    tc->getImpl()->type->getTypeInfo(NULL);
+    dtxoff(pdt, tc->getImpl()->type->vtinfo->toSymbol(), 0, TYnptr);    // impl
+#endif
 }
 
 void TypeInfoFunctionDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoFunctionDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfofunction->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Function
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tfunction);
 
@@ -432,13 +448,19 @@ void TypeInfoFunctionDeclaration::toDt(dt_t **pdt)
 
     tc->next->getTypeInfo(NULL);
     dtxoff(pdt, tc->next->vtinfo->toSymbol(), 0, TYnptr); // TypeInfo for function return value
+
+    const char *name = tinfo->deco;
+    assert(name);
+    size_t namelen = strlen(name);
+    dtsize_t(pdt, namelen);
+    dtabytes(pdt, TYnptr, 0, namelen + 1, name);
 }
 
 void TypeInfoDelegateDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoDelegateDeclaration::toDt()\n");
     dtxoff(pdt, Type::typeinfodelegate->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Delegate
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tdelegate);
 
@@ -446,6 +468,12 @@ void TypeInfoDelegateDeclaration::toDt(dt_t **pdt)
 
     tc->next->nextOf()->getTypeInfo(NULL);
     dtxoff(pdt, tc->next->nextOf()->vtinfo->toSymbol(), 0, TYnptr); // TypeInfo for delegate return value
+
+    const char *name = tinfo->deco;
+    assert(name);
+    size_t namelen = strlen(name);
+    dtsize_t(pdt, namelen);
+    dtabytes(pdt, TYnptr, 0, namelen + 1, name);
 }
 
 void TypeInfoStructDeclaration::toDt(dt_t **pdt)
@@ -454,8 +482,12 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
 
     unsigned offset = Type::typeinfostruct->structsize;
 
+    // If using obsolete 32 bit object.d
+    if (!global.params.is64bit && !Type::typeinfostruct->search(NULL, Id::m_align, 0))
+        offset += 4;                         // include space for m_align
+
     dtxoff(pdt, Type::typeinfostruct->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfo_Struct
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tstruct);
 
@@ -470,21 +502,25 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
      *  int function(void*,void*) xopCmp;
      *  char[] function(void*) xtoString;
      *  uint m_flags;
+     *  uint m_align;
+     *  version (X86_64)
+     *      TypeInfo m_arg1;
+     *      TypeInfo m_arg2;
      *
      *  name[]
      */
 
     const char *name = sd->toPrettyChars();
     size_t namelen = strlen(name);
-    dtdword(pdt, namelen);
+    dtsize_t(pdt, namelen);
     //dtabytes(pdt, TYnptr, 0, namelen + 1, name);
     dtxoff(pdt, toSymbol(), offset, TYnptr);
     offset += namelen + 1;
 
     // void[] init;
-    dtdword(pdt, sd->structsize);       // init.length
+    dtsize_t(pdt, sd->structsize);       // init.length
     if (sd->zeroInit)
-        dtdword(pdt, 0);                // NULL for 0 initialization
+        dtsize_t(pdt, 0);                // NULL for 0 initialization
     else
         dtxoff(pdt, sd->toInitializer(), 0, TYnptr);    // init.ptr
 
@@ -546,10 +582,10 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
             dtxoff(pdt, fd->toSymbol(), 0, TYnptr);
         else
             //fdx->error("must be declared as extern (D) uint toHash()");
-            dtdword(pdt, 0);
+            dtsize_t(pdt, 0);
     }
     else
-        dtdword(pdt, 0);
+        dtsize_t(pdt, 0);
 
     s = search_function(sd, Id::eq);
     fdx = s ? s->isFuncDeclaration() : NULL;
@@ -561,11 +597,11 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
                 dtxoff(pdt, fd->toSymbol(), 0, TYnptr);
             else
                 //fdx->error("must be declared as extern (D) int %s(%s*)", fdx->toChars(), sd->toChars());
-                dtdword(pdt, 0);
+                dtsize_t(pdt, 0);
         }
         else
             //fdx->error("must be declared as extern (D) int %s(%s*)", fdx->toChars(), sd->toChars());
-            dtdword(pdt, 0);
+            dtsize_t(pdt, 0);
 
         s = search_function(sd, Id::cmp);
         fdx = s ? s->isFuncDeclaration() : NULL;
@@ -579,13 +615,16 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
             dtxoff(pdt, fd->toSymbol(), 0, TYnptr);
         else
             //fdx->error("must be declared as extern (D) char[] toString()");
-            dtdword(pdt, 0);
+            dtsize_t(pdt, 0);
     }
     else
-        dtdword(pdt, 0);
+        dtsize_t(pdt, 0);
 
     // uint m_flags;
-    dti32(pdt, tc->hasPointers(), false);
+    dtdword(pdt, tc->hasPointers());
+
+    // uint m_align;
+    dtdword(pdt, tc->alignsize());
 
 #if DMDV2
     // xgetMembers
@@ -593,22 +632,39 @@ void TypeInfoStructDeclaration::toDt(dt_t **pdt)
     if (sgetmembers)
         dtxoff(pdt, sgetmembers->toSymbol(), 0, TYnptr);
     else
-        dtdword(pdt, 0);                        // xgetMembers
+        dtsize_t(pdt, 0);                        // xgetMembers
 
     // xdtor
     FuncDeclaration *sdtor = sd->dtor;
     if (sdtor)
         dtxoff(pdt, sdtor->toSymbol(), 0, TYnptr);
     else
-        dtdword(pdt, 0);                        // xdtor
+        dtsize_t(pdt, 0);                        // xdtor
 
     // xpostblit
     FuncDeclaration *spostblit = sd->postblit;
-    if (spostblit)
+    if (spostblit && !(spostblit->storage_class & STCdisable))
         dtxoff(pdt, spostblit->toSymbol(), 0, TYnptr);
     else
-        dtdword(pdt, 0);                        // xpostblit
+        dtsize_t(pdt, 0);                        // xpostblit
 #endif
+    if (global.params.is64bit)
+    {
+        TypeTuple *tup = tc->toArgTypes();
+        assert(tup->arguments->dim <= 2);
+        for (int i = 0; i < 2; i++)
+        {
+            if (i < tup->arguments->dim)
+            {
+                Type *targ = (tup->arguments->tdata()[i])->type;
+                targ = targ->merge();
+                targ->getTypeInfo(NULL);
+                dtxoff(pdt, targ->vtinfo->toSymbol(), 0, TYnptr);       // m_argi
+            }
+            else
+                dtsize_t(pdt, 0);                    // m_argi
+        }
+    }
 
     // name[]
     dtnbytes(pdt, namelen + 1, name);
@@ -618,7 +674,7 @@ void TypeInfoClassDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoClassDeclaration::toDt() %s\n", tinfo->toChars());
     dtxoff(pdt, Type::typeinfoclass->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfoClass
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tclass);
 
@@ -635,7 +691,7 @@ void TypeInfoInterfaceDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoInterfaceDeclaration::toDt() %s\n", tinfo->toChars());
     dtxoff(pdt, Type::typeinfointerface->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfoInterface
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Tclass);
 
@@ -652,14 +708,14 @@ void TypeInfoTupleDeclaration::toDt(dt_t **pdt)
 {
     //printf("TypeInfoTupleDeclaration::toDt() %s\n", tinfo->toChars());
     dtxoff(pdt, Type::typeinfotypelist->toVtblSymbol(), 0, TYnptr); // vtbl for TypeInfoInterface
-    dtdword(pdt, 0);                        // monitor
+    dtsize_t(pdt, 0);                        // monitor
 
     assert(tinfo->ty == Ttuple);
 
     TypeTuple *tu = (TypeTuple *)tinfo;
 
     size_t dim = tu->arguments->dim;
-    dtdword(pdt, dim);                      // elements.length
+    dtsize_t(pdt, dim);                      // elements.length
 
     dt_t *d = NULL;
     for (size_t i = 0; i < dim; i++)
@@ -777,7 +833,7 @@ int TypeClass::builtinTypeInfo()
  * Used to supply hidden _arguments[] value for variadic D functions.
  */
 
-Expression *createTypeInfoArray(Scope *sc, Expression *exps[], int dim)
+Expression *createTypeInfoArray(Scope *sc, Expression *exps[], unsigned dim)
 {
 #if 1
     /* Get the corresponding TypeInfo_Tuple and
