@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2007 by Digital Mars
+// Copyright (c) 1999-2010 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -23,19 +23,23 @@ struct LabelDsymbol;
 struct Initializer;
 struct Module;
 struct Condition;
-#ifdef _DH
 struct HdrGenState;
-#endif
 
 /**************************************************************/
 
 struct AttribDeclaration : Dsymbol
 {
-    Array *decl;	// array of Dsymbol's
+    Dsymbols *decl;     // array of Dsymbol's
 
-    AttribDeclaration(Array *decl);
-    virtual Array *include(Scope *sc, ScopeDsymbol *s);
+    AttribDeclaration(Dsymbols *decl);
+    virtual Dsymbols *include(Scope *sc, ScopeDsymbol *s);
     int addMember(Scope *sc, ScopeDsymbol *s, int memnum);
+    void setScopeNewSc(Scope *sc,
+        StorageClass newstc, enum LINK linkage, enum PROT protection, int explictProtection,
+        unsigned structalign);
+    void semanticNewSc(Scope *sc,
+        StorageClass newstc, enum LINK linkage, enum PROT protection, int explictProtection,
+        unsigned structalign);
     void semantic(Scope *sc);
     void semantic2(Scope *sc);
     void semantic3(Scope *sc);
@@ -48,28 +52,33 @@ struct AttribDeclaration : Dsymbol
     void checkCtorConstInit();
     void addLocalClass(ClassDeclarations *);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    void toJsonBuffer(OutBuffer *buf);
     AttribDeclaration *isAttribDeclaration() { return this; }
 
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
     int cvMember(unsigned char *p);
 };
 
 struct StorageClassDeclaration: AttribDeclaration
 {
-    unsigned stc;
+    StorageClass stc;
 
-    StorageClassDeclaration(unsigned stc, Array *decl);
+    StorageClassDeclaration(StorageClass stc, Dsymbols *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
+    void setScope(Scope *sc);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+
+    static void stcToCBuffer(OutBuffer *buf, StorageClass stc);
 };
 
 struct LinkDeclaration : AttribDeclaration
 {
     enum LINK linkage;
 
-    LinkDeclaration(enum LINK p, Array *decl);
+    LinkDeclaration(enum LINK p, Dsymbols *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
+    void setScope(Scope *sc);
     void semantic(Scope *sc);
     void semantic3(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -80,18 +89,23 @@ struct ProtDeclaration : AttribDeclaration
 {
     enum PROT protection;
 
-    ProtDeclaration(enum PROT p, Array *decl);
+    ProtDeclaration(enum PROT p, Dsymbols *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
+    void importAll(Scope *sc);
+    void setScope(Scope *sc);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+
+    static void protectionToCBuffer(OutBuffer *buf, enum PROT protection);
 };
 
 struct AlignDeclaration : AttribDeclaration
 {
     unsigned salign;
 
-    AlignDeclaration(unsigned sa, Array *decl);
+    AlignDeclaration(unsigned sa, Dsymbols *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
+    void setScope(Scope *sc);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
@@ -99,10 +113,9 @@ struct AlignDeclaration : AttribDeclaration
 struct AnonDeclaration : AttribDeclaration
 {
     int isunion;
-    Scope *scope;		// !=NULL means context to use
-    int sem;			// 1 if successful semantic()
+    int sem;                    // 1 if successful semantic()
 
-    AnonDeclaration(Loc loc, int isunion, Array *decl);
+    AnonDeclaration(Loc loc, int isunion, Dsymbols *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -111,29 +124,33 @@ struct AnonDeclaration : AttribDeclaration
 
 struct PragmaDeclaration : AttribDeclaration
 {
-    Expressions *args;		// array of Expression's
+    Expressions *args;          // array of Expression's
 
-    PragmaDeclaration(Loc loc, Identifier *ident, Expressions *args, Array *decl);
+    PragmaDeclaration(Loc loc, Identifier *ident, Expressions *args, Dsymbols *decl);
     Dsymbol *syntaxCopy(Dsymbol *s);
     void semantic(Scope *sc);
+    void setScope(Scope *sc);
     int oneMember(Dsymbol **ps);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     const char *kind();
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
 };
 
 struct ConditionalDeclaration : AttribDeclaration
 {
     Condition *condition;
-    Array *elsedecl;	// array of Dsymbol's for else block
+    Dsymbols *elsedecl; // array of Dsymbol's for else block
 
-    ConditionalDeclaration(Condition *condition, Array *decl, Array *elsedecl);
+    ConditionalDeclaration(Condition *condition, Dsymbols *decl, Dsymbols *elsedecl);
     Dsymbol *syntaxCopy(Dsymbol *s);
     int oneMember(Dsymbol **ps);
     void emitComment(Scope *sc);
-    Array *include(Scope *sc, ScopeDsymbol *s);
+    Dsymbols *include(Scope *sc, ScopeDsymbol *s);
     void addComment(unsigned char *comment);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    void toJsonBuffer(OutBuffer *buf);
+    void importAll(Scope *sc);
+    void setScope(Scope *sc);
 };
 
 struct StaticIfDeclaration : ConditionalDeclaration
@@ -141,10 +158,12 @@ struct StaticIfDeclaration : ConditionalDeclaration
     ScopeDsymbol *sd;
     int addisdone;
 
-    StaticIfDeclaration(Condition *condition, Array *decl, Array *elsedecl);
+    StaticIfDeclaration(Condition *condition, Dsymbols *decl, Dsymbols *elsedecl);
     Dsymbol *syntaxCopy(Dsymbol *s);
     int addMember(Scope *sc, ScopeDsymbol *s, int memnum);
     void semantic(Scope *sc);
+    void importAll(Scope *sc);
+    void setScope(Scope *sc);
     const char *kind();
 };
 
