@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -43,95 +43,120 @@ enum PROT;
 enum LINK;
 enum TOK;
 enum MATCH;
+enum PURE;
 
-enum STC
-{
-    STCundefined    = 0,
-    STCstatic	    = 1,
-    STCextern	    = 2,
-    STCconst	    = 4,
-    STCfinal	    = 8,
-    STCabstract     = 0x10,
-    STCparameter    = 0x20,
-    STCfield	    = 0x40,
-    STCoverride	    = 0x80,
-    STCauto         = 0x100,
-    STCsynchronized = 0x200,
-    STCdeprecated   = 0x400,
-    STCin           = 0x800,		// in parameter
-    STCout          = 0x1000,		// out parameter
-    STClazy	    = 0x2000,		// lazy parameter
-    STCforeach      = 0x4000,		// variable for foreach loop
-    STCcomdat       = 0x8000,		// should go into COMDAT record
-    STCvariadic     = 0x10000,		// variadic function argument
-    STCctorinit     = 0x20000,		// can only be set inside constructor
-    STCtemplateparameter = 0x40000,	// template parameter
-    STCscope	    = 0x80000,		// template parameter
-    STCinvariant    = 0x100000,
-    STCref	    = 0x200000,
-    STCinit	    = 0x400000,		// has explicit initializer
-    STCmanifest	    = 0x800000,		// manifest constant
-    STCnodtor	    = 0x1000000,	// don't run destructor
-    STCnothrow	    = 0x2000000,	// never throws exceptions
-    STCpure	    = 0x4000000,	// pure function
-    STCtls	    = 0x8000000,	// thread local
-    STCalias	    = 0x10000000,	// alias parameter
-    STCshared       = 0x20000000,	// accessible from multiple threads
-};
+#define STCundefined    0LL
+#define STCstatic       1LL
+#define STCextern       2LL
+#define STCconst        4LL
+#define STCfinal        8LL
+#define STCabstract     0x10LL
+#define STCparameter    0x20LL
+#define STCfield        0x40LL
+#define STCoverride     0x80LL
+#define STCauto         0x100LL
+#define STCsynchronized 0x200LL
+#define STCdeprecated   0x400LL
+#define STCin           0x800LL         // in parameter
+#define STCout          0x1000LL        // out parameter
+#define STClazy         0x2000LL        // lazy parameter
+#define STCforeach      0x4000LL        // variable for foreach loop
+#define STCcomdat       0x8000LL        // should go into COMDAT record
+#define STCvariadic     0x10000LL       // variadic function argument
+#define STCctorinit     0x20000LL       // can only be set inside constructor
+#define STCtemplateparameter  0x40000LL // template parameter
+#define STCscope        0x80000LL       // template parameter
+#define STCimmutable    0x100000LL
+#define STCref          0x200000LL
+#define STCinit         0x400000LL      // has explicit initializer
+#define STCmanifest     0x800000LL      // manifest constant
+#define STCnodtor       0x1000000LL     // don't run destructor
+#define STCnothrow      0x2000000LL     // never throws exceptions
+#define STCpure         0x4000000LL     // pure function
+#define STCtls          0x8000000LL     // thread local
+#define STCalias        0x10000000LL    // alias parameter
+#define STCshared       0x20000000LL    // accessible from multiple threads
+#define STCgshared      0x40000000LL    // accessible from multiple threads
+                                        // but not typed as "shared"
+#define STCwild         0x80000000LL    // for "wild" type constructor
+#define STC_TYPECTOR    (STCconst | STCimmutable | STCshared | STCwild)
+
+#define STCproperty     0x100000000LL
+#define STCsafe         0x200000000LL
+#define STCtrusted      0x400000000LL
+#define STCsystem       0x800000000LL
+#define STCctfe         0x1000000000LL  // can be used in CTFE, even if it is static
+#define STCdisable      0x2000000000LL  // for functions that are not callable
+#define STCresult       0x4000000000LL  // for result variables passed to out contracts
+#define STCnodefaultctor 0x8000000000LL  // must be set inside constructor
 
 struct Match
 {
-    int count;			// number of matches found
-    MATCH last;			// match level of lastf
-    FuncDeclaration *lastf;	// last matching function we found
-    FuncDeclaration *nextf;	// current matching function
-    FuncDeclaration *anyf;	// pick a func, any func, to use for error recovery
+    int count;                  // number of matches found
+    MATCH last;                 // match level of lastf
+    FuncDeclaration *lastf;     // last matching function we found
+    FuncDeclaration *nextf;     // current matching function
+    FuncDeclaration *anyf;      // pick a func, any func, to use for error recovery
 };
 
 void overloadResolveX(Match *m, FuncDeclaration *f,
-	Expression *ethis, Expressions *arguments);
+        Expression *ethis, Expressions *arguments);
 int overloadApply(FuncDeclaration *fstart,
-	int (*fp)(void *, FuncDeclaration *),
-	void *param);
+        int (*fp)(void *, FuncDeclaration *),
+        void *param);
+
+enum Semantic
+{
+    SemanticStart,      // semantic has not been run
+    SemanticIn,         // semantic() is in progress
+    SemanticDone,       // semantic() has been run
+    Semantic2Done,      // semantic2() has been run
+};
 
 /**************************************************************/
 
 struct Declaration : Dsymbol
 {
     Type *type;
-    Type *originalType;		// before semantic analysis
-    unsigned storage_class;
+    Type *originalType;         // before semantic analysis
+    StorageClass storage_class;
     enum PROT protection;
     enum LINK linkage;
-    Expressions * attributes; // GCC decl/type attributes
+    int inuse;                  // used to detect cycles
+#if IN_GCC
+    Expressions * attributes;   // GCC decl/type attributes
+#endif
+
+    enum Semantic sem;
 
     Declaration(Identifier *id);
     void semantic(Scope *sc);
     const char *kind();
-    target_size_t size(Loc loc);
+    unsigned size(Loc loc);
     void checkModify(Loc loc, Scope *sc, Type *t);
 
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
     void toDocBuffer(OutBuffer *buf);
 
     char *mangle();
     int isStatic() { return storage_class & STCstatic; }
-    virtual int isStaticConstructor();
-    virtual int isStaticDestructor();
     virtual int isDelete();
     virtual int isDataseg();
+    virtual int isThreadlocal();
     virtual int isCodeseg();
     int isCtorinit()     { return storage_class & STCctorinit; }
     int isFinal()        { return storage_class & STCfinal; }
     int isAbstract()     { return storage_class & STCabstract; }
     int isConst()        { return storage_class & STCconst; }
-    int isInvariant()    { return storage_class & STCinvariant; }
+    int isImmutable()    { return storage_class & STCimmutable; }
     int isAuto()         { return storage_class & STCauto; }
-    int isScope()        { return storage_class & (STCscope | STCauto); }
+    int isScope()        { return storage_class & STCscope; }
     int isSynchronized() { return storage_class & STCsynchronized; }
     int isParameter()    { return storage_class & STCparameter; }
     int isDeprecated()   { return storage_class & STCdeprecated; }
     int isOverride()     { return storage_class & STCoverride; }
+    StorageClass isResult()       { return storage_class & STCresult; }
 
     int isIn()    { return storage_class & STCin; }
     int isOut()   { return storage_class & STCout; }
@@ -147,9 +172,9 @@ struct Declaration : Dsymbol
 struct TupleDeclaration : Declaration
 {
     Objects *objects;
-    int isexp;			// 1: expression tuple
+    int isexp;                  // 1: expression tuple
 
-    TypeTuple *tupletype;	// !=NULL if this is a type tuple
+    TypeTuple *tupletype;       // !=NULL if this is a type tuple
 
     TupleDeclaration(Loc loc, Identifier *ident, Objects *objects);
     Dsymbol *syntaxCopy(Dsymbol *);
@@ -166,11 +191,6 @@ struct TypedefDeclaration : Declaration
 {
     Type *basetype;
     Initializer *init;
-    int sem;			// 0: semantic() has not been run
-				// 1: semantic() is in progress
-				// 2: semantic() has been run
-				// 3: semantic2() has been run
-    int inuse;			// used to detect typedef cycles
 
     TypedefDeclaration(Loc loc, Identifier *ident, Type *basetype, Initializer *init);
     Dsymbol *syntaxCopy(Dsymbol *);
@@ -180,14 +200,12 @@ struct TypedefDeclaration : Declaration
     const char *kind();
     Type *getType();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-#ifdef _DH
     Type *htype;
     Type *hbasetype;
-#endif
 
     void toDocBuffer(OutBuffer *buf);
 
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
     void toDebug();
     int cvMember(unsigned char *p);
 
@@ -202,7 +220,7 @@ struct TypedefDeclaration : Declaration
 struct AliasDeclaration : Declaration
 {
     Dsymbol *aliassym;
-    Dsymbol *overnext;		// next in overload list
+    Dsymbol *overnext;          // next in overload list
     int inSemantic;
 
     AliasDeclaration(Loc loc, Identifier *ident, Type *type);
@@ -214,10 +232,8 @@ struct AliasDeclaration : Declaration
     Type *getType();
     Dsymbol *toAlias();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-#ifdef _DH
     Type *htype;
     Dsymbol *haliassym;
-#endif
 
     void toDocBuffer(OutBuffer *buf);
 
@@ -229,18 +245,40 @@ struct AliasDeclaration : Declaration
 struct VarDeclaration : Declaration
 {
     Initializer *init;
-    target_size_t offset;
-    int noauto;			// no auto semantics
+    unsigned offset;
+    int noscope;                 // no auto semantics
+#if DMDV2
     FuncDeclarations nestedrefs; // referenced by these lexically nested functions
-    int inuse;
-    int ctorinit;		// it has been initialized in a ctor
-    int onstack;		// 1: it has been allocated on the stack
-				// 2: on stack, run destructor anyway
-    int canassign;		// it can be assigned to
-    Dsymbol *aliassym;		// if redone as alias to another symbol
-    Expression *value;		// when interpreting, this is the value
-				// (NULL if value not determinable)
-    Scope *scope;		// !=NULL means context to use
+    bool isargptr;              // if parameter that _argptr points to
+#else
+    int nestedref;              // referenced by a lexically nested function
+#if IN_GCC
+    FuncDeclarations nestedrefs; // referenced by these lexically nested functions
+#endif
+#endif
+    int ctorinit;               // it has been initialized in a ctor
+    int onstack;                // 1: it has been allocated on the stack
+                                // 2: on stack, run destructor anyway
+    int canassign;              // it can be assigned to
+    Dsymbol *aliassym;          // if redone as alias to another symbol
+
+    // When interpreting, these hold the value (NULL if value not determinable)
+    // The various functions are used only to detect compiler CTFE bugs
+    Expression *literalvalue;
+    Expression *getValue() { return literalvalue; }
+    void setValueNull();
+    void setValueWithoutChecking(Expression *newval);
+    void createRefValue(Expression *newval); // struct or array literal
+    void setRefValue(Expression *newval);
+    void setStackValue(Expression *newval);
+    void createStackValue(Expression *newval);
+
+#if DMDV2
+    VarDeclaration *rundtor;    // if !NULL, rundtor is tested at runtime to see
+                                // if the destructor should be run. Used to prevent
+                                // dtor calls on postblitted vars
+    Expression *edtor;          // if !=NULL, does the destruction of the variable
+#endif
 
     VarDeclaration(Loc loc, Type *t, Identifier *id, Initializer *init);
     Dsymbol *syntaxCopy(Dsymbol *);
@@ -248,17 +286,20 @@ struct VarDeclaration : Declaration
     void semantic2(Scope *sc);
     const char *kind();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-#ifdef _DH
     Type *htype;
     Initializer *hinit;
-#endif
+    AggregateDeclaration *isThis();
     int needThis();
     int isImportedSymbol();
     int isDataseg();
+    int isThreadlocal();
+    int isCTFE();
     int hasPointers();
+#if DMDV2
     int canTakeAddressOf();
     int needsAutoDtor();
-    Expression *callAutoDtor(Scope *sc);
+#endif
+    Expression *callScopeDtor(Scope *sc);
     ExpInitializer *getExpInitializer();
     Expression *getConstInitializer();
     void checkCtorConstInit();
@@ -266,14 +307,13 @@ struct VarDeclaration : Declaration
     Dsymbol *toAlias();
 
     Symbol *toSymbol();
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
     int cvMember(unsigned char *p);
 
     // Eliminate need for dynamic_cast
     VarDeclaration *isVarDeclaration() { return (VarDeclaration *)this; }
 };
 
-
 /**************************************************************/
 
 // This is a shell around a back end symbol
@@ -300,6 +340,7 @@ struct ClassInfoDeclaration : VarDeclaration
     void semantic(Scope *sc);
 
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
 
     Symbol *toSymbol();
 };
@@ -313,6 +354,7 @@ struct ModuleInfoDeclaration : VarDeclaration
     void semantic(Scope *sc);
 
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
 
     Symbol *toSymbol();
 };
@@ -326,9 +368,10 @@ struct TypeInfoDeclaration : VarDeclaration
     void semantic(Scope *sc);
 
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
 
     Symbol *toSymbol();
-    void toObjFile(int multiobj);			// compile to .obj file
+    void toObjFile(int multiobj);                       // compile to .obj file
     virtual void toDt(dt_t **pdt);
 };
 
@@ -342,6 +385,7 @@ struct TypeInfoStructDeclaration : TypeInfoDeclaration
 struct TypeInfoClassDeclaration : TypeInfoDeclaration
 {
     TypeInfoClassDeclaration(Type *tinfo);
+    Symbol *toSymbol();
 
     void toDt(dt_t **pdt);
 };
@@ -416,7 +460,7 @@ struct TypeInfoTupleDeclaration : TypeInfoDeclaration
     void toDt(dt_t **pdt);
 };
 
-#if V2
+#if DMDV2
 struct TypeInfoConstDeclaration : TypeInfoDeclaration
 {
     TypeInfoConstDeclaration(Type *tinfo);
@@ -430,114 +474,152 @@ struct TypeInfoInvariantDeclaration : TypeInfoDeclaration
 
     void toDt(dt_t **pdt);
 };
+
+struct TypeInfoSharedDeclaration : TypeInfoDeclaration
+{
+    TypeInfoSharedDeclaration(Type *tinfo);
+
+    void toDt(dt_t **pdt);
+};
+
+struct TypeInfoWildDeclaration : TypeInfoDeclaration
+{
+    TypeInfoWildDeclaration(Type *tinfo);
+
+    void toDt(dt_t **pdt);
+};
 #endif
 
 /**************************************************************/
 
 struct ThisDeclaration : VarDeclaration
 {
-    ThisDeclaration(Type *t);
+    ThisDeclaration(Loc loc, Type *t);
     Dsymbol *syntaxCopy(Dsymbol *);
+    ThisDeclaration *isThisDeclaration() { return this; }
 };
 
 enum ILS
 {
-    ILSuninitialized,	// not computed yet
-    ILSno,		// cannot inline
-    ILSyes,		// can inline
+    ILSuninitialized,   // not computed yet
+    ILSno,              // cannot inline
+    ILSyes,             // can inline
 };
 
 /**************************************************************/
-#if V2
+#if DMDV2
 
 enum BUILTIN
 {
-    BUILTINunknown = -1,	// not known if this is a builtin
-    BUILTINnot,			// this is not a builtin
-    BUILTINsin,			// std.math.sin
-    BUILTINcos,			// std.math.cos
-    BUILTINtan,			// std.math.tan
-    BUILTINsqrt,		// std.math.sqrt
-    BUILTINfabs,		// std.math.fabs
+    BUILTINunknown = -1,        // not known if this is a builtin
+    BUILTINnot,                 // this is not a builtin
+    BUILTINsin,                 // std.math.sin
+    BUILTINcos,                 // std.math.cos
+    BUILTINtan,                 // std.math.tan
+    BUILTINsqrt,                // std.math.sqrt
+    BUILTINfabs,                // std.math.fabs
+#if IN_GCC
+    BUILTINgcc,                 // GCC builtin
+#endif
 };
 
 Expression *eval_builtin(enum BUILTIN builtin, Expressions *arguments);
 
+#else
+enum BUILTIN { };
 #endif
 
 struct FuncDeclaration : Declaration
 {
-    Array *fthrows;			// Array of Type's of exceptions (not used)
+    Types *fthrows;                     // Array of Type's of exceptions (not used)
     Statement *frequire;
     Statement *fensure;
     Statement *fbody;
 
-    Identifier *outId;			// identifier for out statement
-    VarDeclaration *vresult;		// variable corresponding to outId
-    LabelDsymbol *returnLabel;		// where the return goes
+    FuncDeclarations foverrides;        // functions this function overrides
+    FuncDeclaration *fdrequire;         // function that does the in contract
+    FuncDeclaration *fdensure;          // function that does the out contract
 
-    DsymbolTable *localsymtab;		// used to prevent symbols in different
-					// scopes from having the same name
-    VarDeclaration *vthis;		// 'this' parameter (member and nested)
-    VarDeclaration *v_arguments;	// '_arguments' parameter
+    Identifier *outId;                  // identifier for out statement
+    VarDeclaration *vresult;            // variable corresponding to outId
+    LabelDsymbol *returnLabel;          // where the return goes
+
+    DsymbolTable *localsymtab;          // used to prevent symbols in different
+                                        // scopes from having the same name
+    VarDeclaration *vthis;              // 'this' parameter (member and nested)
+    VarDeclaration *v_arguments;        // '_arguments' parameter
 #if IN_GCC
-    VarDeclaration *v_arguments_var;	// '_arguments' variable
-    VarDeclaration *v_argptr;	        // '_argptr' variable
+    VarDeclaration *v_arguments_var;    // '_arguments' variable
+    VarDeclaration *v_argptr;           // '_argptr' variable
 #endif
-    Dsymbols *parameters;		// Array of VarDeclaration's for parameters
-    DsymbolTable *labtab;		// statement label symbol table
-    Declaration *overnext;		// next in overload list
-    Loc endloc;				// location of closing curly bracket
-    int vtblIndex;			// for member functions, index into vtbl[]
-    int naked;				// !=0 if naked
-    int inlineAsm;			// !=0 if has inline assembler
+    VarDeclaration *v_argsave;          // save area for args passed in registers for variadic functions
+    VarDeclarations *parameters;        // Array of VarDeclaration's for parameters
+    DsymbolTable *labtab;               // statement label symbol table
+    Declaration *overnext;              // next in overload list
+    Loc endloc;                         // location of closing curly bracket
+    int vtblIndex;                      // for member functions, index into vtbl[]
+    int naked;                          // !=0 if naked
     ILS inlineStatus;
-    int inlineNest;			// !=0 if nested inline
-    int cantInterpret;			// !=0 if cannot interpret function
-    int semanticRun;			// !=0 if semantic3() had been run
-					// this function's frame ptr
-    ForeachStatement *fes;		// if foreach body, this is the foreach
-    int introducing;			// !=0 if 'introducing' function
-    Type *tintro;			// if !=NULL, then this is the type
-					// of the 'introducing' function
-					// this one is overriding
-    int inferRetType;			// !=0 if return type is to be inferred
-    Scope *scope;			// !=NULL means context to use
+    int inlineNest;                     // !=0 if nested inline
+    int cantInterpret;                  // !=0 if cannot interpret function
+    int isArrayOp;                      // !=0 if array operation
+    enum PASS semanticRun;
+                                        // this function's frame ptr
+    ForeachStatement *fes;              // if foreach body, this is the foreach
+    int introducing;                    // !=0 if 'introducing' function
+    Type *tintro;                       // if !=NULL, then this is the type
+                                        // of the 'introducing' function
+                                        // this one is overriding
+    int inferRetType;                   // !=0 if return type is to be inferred
 
     // Things that should really go into Scope
-    int hasReturnExp;			// 1 if there's a return exp; statement
-					// 2 if there's a throw statement
-					// 4 if there's an assert(0)
-					// 8 if there's inline asm
+    int hasReturnExp;                   // 1 if there's a return exp; statement
+                                        // 2 if there's a throw statement
+                                        // 4 if there's an assert(0)
+                                        // 8 if there's inline asm
 
     // Support for NRVO (named return value optimization)
-    int nrvo_can;			// !=0 means we can do it
-    VarDeclaration *nrvo_var;		// variable to replace with shidden
-    Symbol *shidden;			// hidden pointer passed to function
+    int nrvo_can;                       // !=0 means we can do it
+    VarDeclaration *nrvo_var;           // variable to replace with shidden
+    Symbol *shidden;                    // hidden pointer passed to function
 
-#if V2
-    enum BUILTIN builtin;		// set if this is a known, builtin
-					// function we can evaluate at compile
-					// time
+#if DMDV2
+    enum BUILTIN builtin;               // set if this is a known, builtin
+                                        // function we can evaluate at compile
+                                        // time
 
-    int tookAddressOf;			// set if someone took the address of
-					// this function
-    Dsymbols closureVars;		// local variables in this function
-					// which are referenced by nested
-					// functions
+    int tookAddressOf;                  // set if someone took the address of
+                                        // this function
+    VarDeclarations closureVars;        // local variables in this function
+                                        // which are referenced by nested
+                                        // functions
+
+    unsigned flags;
+    #define FUNCFLAGpurityInprocess 1   // working on determining purity
+    #define FUNCFLAGsafetyInprocess 2   // working on determining safety
+    #define FUNCFLAGnothrowInprocess 4  // working on determining nothrow
 #else
-    int nestedFrameRef;			// !=0 if nested variables referenced
+    int nestedFrameRef;                 // !=0 if nested variables referenced
+#if IN_GCC
+    VarDeclarations frameVars;          // local variables in this function
+                                        // which are referenced by nested
+                                        // functions
+#endif
 #endif
 
-    FuncDeclaration(Loc loc, Loc endloc, Identifier *id, enum STC storage_class, Type *type);
+    FuncDeclaration(Loc loc, Loc endloc, Identifier *id, StorageClass storage_class, Type *type);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     void semantic2(Scope *sc);
     void semantic3(Scope *sc);
+    // called from semantic3
+    void varArgs(Scope *sc, TypeFunction*, VarDeclaration *&, VarDeclaration *&);
+    int equals(Object *o);
+
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     void bodyToCBuffer(OutBuffer *buf, HdrGenState *hgs);
     int overrides(FuncDeclaration *fd);
-    int findVtblIndex(Array *vtbl, int dim);
+    int findVtblIndex(Dsymbols *vtbl, int dim);
     int overloadInsert(Dsymbol *s);
     FuncDeclaration *overloadExactMatch(Type *t);
     FuncDeclaration *overloadResolve(Loc loc, Expression *ethis, Expressions *arguments, int flags = 0);
@@ -545,10 +627,11 @@ struct FuncDeclaration : Declaration
     LabelDsymbol *searchLabel(Identifier *ident);
     AggregateDeclaration *isThis();
     AggregateDeclaration *isMember2();
-    int getLevel(Loc loc, FuncDeclaration *fd);	// lexical nesting level difference
+    int getLevel(Loc loc, FuncDeclaration *fd); // lexical nesting level difference
     void appendExp(Expression *e);
     void appendState(Statement *s);
     char *mangle();
+    const char *toPrettyChars();
     int isMain();
     int isWinMain();
     int isDllMain();
@@ -558,35 +641,50 @@ struct FuncDeclaration : Declaration
     int isAbstract();
     int isCodeseg();
     int isOverloadable();
+    enum PURE isPure();
+    enum PURE isPureBypassingInference();
+    bool setImpure();
+    int isSafe();
+    int isTrusted();
+    bool setUnsafe();
     virtual int isNested();
     int needThis();
     virtual int isVirtual();
     virtual int isFinal();
     virtual int addPreInvariant();
     virtual int addPostInvariant();
-    Expression *interpret(InterState *istate, Expressions *arguments);
+    Expression *interpret(InterState *istate, Expressions *arguments, Expression *thisexp = NULL);
     void inlineScan();
     int canInline(int hasthis, int hdrscan = 0);
-    Expression *doInline(InlineScanState *iss, Expression *ethis, Array *arguments);
+    Expression *doInline(InlineScanState *iss, Expression *ethis, Expressions *arguments);
     const char *kind();
     void toDocBuffer(OutBuffer *buf);
     FuncDeclaration *isUnique();
     int needsClosure();
+    Statement *mergeFrequire(Statement *);
+    Statement *mergeFensure(Statement *);
+    Parameters *getParameters(int *pvarargs);
 
-    static FuncDeclaration *genCfunc(Type *treturn, const char *name,
-	Type *t1 = 0, Type *t2 = 0, Type *t3 = 0);
-    static FuncDeclaration *genCfunc(Type *treturn, Identifier *id,
-	Type *t1 = 0, Type *t2 = 0, Type *t3 = 0);
+    static FuncDeclaration *genCfunc(Type *treturn, const char *name, Type *t1 = NULL, Type *t2 = NULL, Type *t3 = NULL);
+    static FuncDeclaration *genCfunc(Type *treturn, Identifier *id, Type *t1 = NULL, Type *t2 = NULL, Type *t3 = NULL);
 
     Symbol *toSymbol();
-    Symbol *toThunkSymbol(target_ptrdiff_t offset);	// thunk version
-    void toObjFile(int multiobj);			// compile to .obj file
+    Symbol *toThunkSymbol(int offset);  // thunk version
+    void toObjFile(int multiobj);                       // compile to .obj file
     int cvMember(unsigned char *p);
     void buildClosure(IRState *irs);
 
     FuncDeclaration *isFuncDeclaration() { return this; }
 };
 
+#if DMDV2
+FuncDeclaration *resolveFuncCall(Scope *sc, Loc loc, Dsymbol *s,
+        Objects *tiargs,
+        Expression *ethis,
+        Expressions *arguments,
+        int flags);
+#endif
+
 struct FuncAliasDeclaration : FuncDeclaration
 {
     FuncDeclaration *funcalias;
@@ -600,10 +698,10 @@ struct FuncAliasDeclaration : FuncDeclaration
 
 struct FuncLiteralDeclaration : FuncDeclaration
 {
-    enum TOK tok;			// TOKfunction or TOKdelegate
+    enum TOK tok;                       // TOKfunction or TOKdelegate
 
     FuncLiteralDeclaration(Loc loc, Loc endloc, Type *type, enum TOK tok,
-	ForeachStatement *fes);
+        ForeachStatement *fes);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Dsymbol *syntaxCopy(Dsymbol *);
     int isNested();
@@ -614,10 +712,8 @@ struct FuncLiteralDeclaration : FuncDeclaration
 };
 
 struct CtorDeclaration : FuncDeclaration
-{   Arguments *arguments;
-    int varargs;
-
-    CtorDeclaration(Loc loc, Loc endloc, Arguments *arguments, int varargs);
+{
+    CtorDeclaration(Loc loc, Loc endloc, StorageClass stc, Type *type);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -626,12 +722,11 @@ struct CtorDeclaration : FuncDeclaration
     int isVirtual();
     int addPreInvariant();
     int addPostInvariant();
-    void toDocBuffer(OutBuffer *buf);
 
     CtorDeclaration *isCtorDeclaration() { return this; }
 };
 
-#if V2
+#if DMDV2
 struct PostBlitDeclaration : FuncDeclaration
 {
     PostBlitDeclaration(Loc loc, Loc endloc);
@@ -644,6 +739,7 @@ struct PostBlitDeclaration : FuncDeclaration
     int addPostInvariant();
     int overloadInsert(Dsymbol *s);
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
 
     PostBlitDeclaration *isPostBlitDeclaration() { return this; }
 };
@@ -656,11 +752,14 @@ struct DtorDeclaration : FuncDeclaration
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    const char *kind();
+    char *toChars();
     int isVirtual();
     int addPreInvariant();
     int addPostInvariant();
     int overloadInsert(Dsymbol *s);
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
 
     DtorDeclaration *isDtorDeclaration() { return this; }
 };
@@ -668,36 +767,60 @@ struct DtorDeclaration : FuncDeclaration
 struct StaticCtorDeclaration : FuncDeclaration
 {
     StaticCtorDeclaration(Loc loc, Loc endloc);
+    StaticCtorDeclaration(Loc loc, Loc endloc, const char *name);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     AggregateDeclaration *isThis();
-    int isStaticConstructor();
     int isVirtual();
     int addPreInvariant();
     int addPostInvariant();
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     StaticCtorDeclaration *isStaticCtorDeclaration() { return this; }
 };
 
-struct StaticDtorDeclaration : FuncDeclaration
+#if DMDV2
+struct SharedStaticCtorDeclaration : StaticCtorDeclaration
 {
-	VarDeclaration *vgate;	// 'gate' variable
+    SharedStaticCtorDeclaration(Loc loc, Loc endloc);
+    Dsymbol *syntaxCopy(Dsymbol *);
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+
+    SharedStaticCtorDeclaration *isSharedStaticCtorDeclaration() { return this; }
+};
+#endif
+
+struct StaticDtorDeclaration : FuncDeclaration
+{   VarDeclaration *vgate;      // 'gate' variable
+
     StaticDtorDeclaration(Loc loc, Loc endloc);
+    StaticDtorDeclaration(Loc loc, Loc endloc, const char *name);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     AggregateDeclaration *isThis();
-    int isStaticDestructor();
     int isVirtual();
     int addPreInvariant();
     int addPostInvariant();
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     StaticDtorDeclaration *isStaticDtorDeclaration() { return this; }
 };
 
+#if DMDV2
+struct SharedStaticDtorDeclaration : StaticDtorDeclaration
+{
+    SharedStaticDtorDeclaration(Loc loc, Loc endloc);
+    Dsymbol *syntaxCopy(Dsymbol *);
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+
+    SharedStaticDtorDeclaration *isSharedStaticDtorDeclaration() { return this; }
+};
+#endif
+
 struct InvariantDeclaration : FuncDeclaration
 {
     InvariantDeclaration(Loc loc, Loc endloc);
@@ -707,12 +830,12 @@ struct InvariantDeclaration : FuncDeclaration
     int addPreInvariant();
     int addPostInvariant();
     void emitComment(Scope *sc);
+    void toJsonBuffer(OutBuffer *buf);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     InvariantDeclaration *isInvariantDeclaration() { return this; }
 };
 
-
 struct UnitTestDeclaration : FuncDeclaration
 {
     UnitTestDeclaration(Loc loc, Loc endloc);
@@ -723,15 +846,16 @@ struct UnitTestDeclaration : FuncDeclaration
     int addPreInvariant();
     int addPostInvariant();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    void toJsonBuffer(OutBuffer *buf);
 
     UnitTestDeclaration *isUnitTestDeclaration() { return this; }
 };
 
 struct NewDeclaration : FuncDeclaration
-{   Arguments *arguments;
+{   Parameters *arguments;
     int varargs;
 
-    NewDeclaration(Loc loc, Loc endloc, Arguments *arguments, int varargs);
+    NewDeclaration(Loc loc, Loc endloc, Parameters *arguments, int varargs);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -745,9 +869,9 @@ struct NewDeclaration : FuncDeclaration
 
 
 struct DeleteDeclaration : FuncDeclaration
-{   Arguments *arguments;
+{   Parameters *arguments;
 
-    DeleteDeclaration(Loc loc, Loc endloc, Arguments *arguments);
+    DeleteDeclaration(Loc loc, Loc endloc, Parameters *arguments);
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -756,9 +880,7 @@ struct DeleteDeclaration : FuncDeclaration
     int isVirtual();
     int addPreInvariant();
     int addPostInvariant();
-#ifdef _DH
     DeleteDeclaration *isDeleteDeclaration() { return this; }
-#endif
 };
 
 #endif /* DMD_DECLARATION_H */
