@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2010 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -25,6 +25,7 @@ struct Identifier;
 struct Scope;
 struct DsymbolTable;
 struct Declaration;
+struct ThisDeclaration;
 struct TupleDeclaration;
 struct TypedefDeclaration;
 struct AliasDeclaration;
@@ -42,6 +43,8 @@ struct PostBlitDeclaration;
 struct DtorDeclaration;
 struct StaticCtorDeclaration;
 struct StaticDtorDeclaration;
+struct SharedStaticCtorDeclaration;
+struct SharedStaticDtorDeclaration;
 struct InvariantDeclaration;
 struct UnitTestDeclaration;
 struct NewDeclaration;
@@ -55,6 +58,7 @@ struct Type;
 struct TypeTuple;
 struct WithStatement;
 struct LabelDsymbol;
+struct ScopeDsymbol;
 struct TemplateDeclaration;
 struct TemplateInstance;
 struct TemplateMixin;
@@ -67,7 +71,10 @@ struct Expression;
 struct DeleteDeclaration;
 struct HdrGenState;
 struct OverloadSet;
-
+struct AA;
+#if TARGET_NET
+struct PragmaScope;
+#endif
 #if IN_GCC
 union tree_node;
 typedef union tree_node TYPE;
@@ -81,7 +88,7 @@ struct Classsym;
 enum PROT
 {
     PROTundefined,
-    PROTnone,		// no access
+    PROTnone,           // no access
     PROTprivate,
     PROTpackage,
     PROTprotected,
@@ -89,21 +96,33 @@ enum PROT
     PROTexport,
 };
 
+/* State of symbol in winding its way through the passes of the compiler
+ */
+enum PASS
+{
+    PASSinit,           // initial state
+    PASSsemantic,       // semantic() started
+    PASSsemanticdone,   // semantic() done
+    PASSsemantic2,      // semantic2() run
+    PASSsemantic3,      // semantic3() started
+    PASSsemantic3done,  // semantic3() done
+    PASSobj,            // toObjFile() run
+};
 
 struct Dsymbol : Object
 {
     Identifier *ident;
     Identifier *c_ident;
     Dsymbol *parent;
-    Symbol *csym;		// symbol for code generator
-    Symbol *isym;		// import version of csym
-    unsigned char *comment;	// documentation comment for this Dsymbol
-    Loc loc;			// where defined
+    Symbol *csym;               // symbol for code generator
+    Symbol *isym;               // import version of csym
+    unsigned char *comment;     // documentation comment for this Dsymbol
+    Loc loc;                    // where defined
+    Scope *scope;               // !=NULL means context to use for semantic()
 
     Dsymbol();
     Dsymbol(Identifier *);
     char *toChars();
-    char *toPrettyChars();
     char *locToChars();
     int equals(Object *o);
     int isAnonymous();
@@ -116,44 +135,51 @@ struct Dsymbol : Object
     Dsymbol *toParent2();
     TemplateInstance *inTemplateInstance();
 
-    int dyncast() { return DYNCAST_DSYMBOL; }	// kludge for template.isSymbol()
+    int dyncast() { return DYNCAST_DSYMBOL; }   // kludge for template.isSymbol()
 
-    static Array *arraySyntaxCopy(Array *a);
+    static Dsymbols *arraySyntaxCopy(Dsymbols *a);
 
+    virtual const char *toPrettyChars();
     virtual const char *kind();
-    virtual Dsymbol *toAlias();			// resolve real symbol
+    virtual Dsymbol *toAlias();                 // resolve real symbol
     virtual int addMember(Scope *sc, ScopeDsymbol *s, int memnum);
+    virtual void setScope(Scope *sc);
+    virtual void importAll(Scope *sc);
+    virtual void semantic0(Scope *sc);
     virtual void semantic(Scope *sc);
     virtual void semantic2(Scope *sc);
     virtual void semantic3(Scope *sc);
     virtual void inlineScan();
     virtual Dsymbol *search(Loc loc, Identifier *ident, int flags);
+    Dsymbol *search_correct(Identifier *id);
     Dsymbol *searchX(Loc loc, Scope *sc, Identifier *id);
     virtual int overloadInsert(Dsymbol *s);
-#ifdef _DH
     char *toHChars();
     virtual void toHBuffer(OutBuffer *buf, HdrGenState *hgs);
-#endif
     virtual void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     virtual void toDocBuffer(OutBuffer *buf);
-    virtual target_size_t size(Loc loc);
+    virtual void toJsonBuffer(OutBuffer *buf);
+    virtual unsigned size(Loc loc);
     virtual int isforwardRef();
     virtual void defineRef(Dsymbol *s);
-    virtual AggregateDeclaration *isThis();	// is a 'this' required to access the member
-    virtual ClassDeclaration *isClassMember();	// are we a member of a class?
-    virtual int isExport();			// is Dsymbol exported?
-    virtual int isImportedSymbol();		// is Dsymbol imported?
-    virtual int isDeprecated();			// is Dsymbol deprecated?
+    virtual AggregateDeclaration *isThis();     // is a 'this' required to access the member
+    AggregateDeclaration *isAggregateMember();  // are we a member of an aggregate?
+    ClassDeclaration *isClassMember();          // are we a member of a class?
+    virtual int isExport();                     // is Dsymbol exported?
+    virtual int isImportedSymbol();             // is Dsymbol imported?
+    virtual int isDeprecated();                 // is Dsymbol deprecated?
+#if DMDV2
     virtual int isOverloadable();
-    virtual LabelDsymbol *isLabel();		// is this a LabelDsymbol?
-    virtual AggregateDeclaration *isMember();	// is this symbol a member of an AggregateDeclaration?
-    virtual Type *getType();			// is this a type?
+#endif
+    virtual LabelDsymbol *isLabel();            // is this a LabelDsymbol?
+    virtual AggregateDeclaration *isMember();   // is this symbol a member of an AggregateDeclaration?
+    virtual Type *getType();                    // is this a type?
     virtual char *mangle();
-    virtual int needThis();			// need a 'this' pointer?
+    virtual int needThis();                     // need a 'this' pointer?
     virtual enum PROT prot();
-    virtual Dsymbol *syntaxCopy(Dsymbol *s);	// copy only syntax trees
+    virtual Dsymbol *syntaxCopy(Dsymbol *s);    // copy only syntax trees
     virtual int oneMember(Dsymbol **ps);
-    static int oneMembers(Array *members, Dsymbol **ps);
+    static int oneMembers(Dsymbols *members, Dsymbol **ps);
     virtual int hasPointers();
     virtual void addLocalClass(ClassDeclarations *) { }
     virtual void checkCtorConstInit() { }
@@ -164,14 +190,14 @@ struct Dsymbol : Object
 
     // Backend
 
-    virtual Symbol *toSymbol();			// to backend symbol
-    virtual void toObjFile(int multiobj);			// compile to .obj file
-    virtual int cvMember(unsigned char *p);	// emit cv debug info for member
+    virtual Symbol *toSymbol();                 // to backend symbol
+    virtual void toObjFile(int multiobj);                       // compile to .obj file
+    virtual int cvMember(unsigned char *p);     // emit cv debug info for member
 
-    Symbol *toImport();				// to backend import symbol
-    static Symbol *toImport(Symbol *s);		// to backend import symbol
+    Symbol *toImport();                         // to backend import symbol
+    static Symbol *toImport(Symbol *s);         // to backend import symbol
 
-    Symbol *toSymbolX(const char *prefix, int sclass, TYPE *t, const char *suffix);	// helper
+    Symbol *toSymbolX(const char *prefix, int sclass, TYPE *t, const char *suffix);     // helper
 
     // Eliminate need for dynamic_cast
     virtual Package *isPackage() { return NULL; }
@@ -181,6 +207,7 @@ struct Dsymbol : Object
     virtual TemplateInstance *isTemplateInstance() { return NULL; }
     virtual TemplateMixin *isTemplateMixin() { return NULL; }
     virtual Declaration *isDeclaration() { return NULL; }
+    virtual ThisDeclaration *isThisDeclaration() { return NULL; }
     virtual TupleDeclaration *isTupleDeclaration() { return NULL; }
     virtual TypedefDeclaration *isTypedefDeclaration() { return NULL; }
     virtual AliasDeclaration *isAliasDeclaration() { return NULL; }
@@ -193,6 +220,8 @@ struct Dsymbol : Object
     virtual DtorDeclaration *isDtorDeclaration() { return NULL; }
     virtual StaticCtorDeclaration *isStaticCtorDeclaration() { return NULL; }
     virtual StaticDtorDeclaration *isStaticDtorDeclaration() { return NULL; }
+    virtual SharedStaticCtorDeclaration *isSharedStaticCtorDeclaration() { return NULL; }
+    virtual SharedStaticDtorDeclaration *isSharedStaticDtorDeclaration() { return NULL; }
     virtual InvariantDeclaration *isInvariantDeclaration() { return NULL; }
     virtual UnitTestDeclaration *isUnitTestDeclaration() { return NULL; }
     virtual NewDeclaration *isNewDeclaration() { return NULL; }
@@ -206,23 +235,24 @@ struct Dsymbol : Object
     virtual ArrayScopeSymbol *isArrayScopeSymbol() { return NULL; }
     virtual Import *isImport() { return NULL; }
     virtual EnumDeclaration *isEnumDeclaration() { return NULL; }
-#ifdef _DH
     virtual DeleteDeclaration *isDeleteDeclaration() { return NULL; }
-#endif
     virtual SymbolDeclaration *isSymbolDeclaration() { return NULL; }
     virtual AttribDeclaration *isAttribDeclaration() { return NULL; }
     virtual OverloadSet *isOverloadSet() { return NULL; }
+#if TARGET_NET
+    virtual PragmaScope* isPragmaScope() { return NULL; }
+#endif
 };
 
 // Dsymbol that generates a scope
 
 struct ScopeDsymbol : Dsymbol
 {
-    Array *members;		// all Dsymbol's in this scope
-    DsymbolTable *symtab;	// members[] sorted into table
+    Dsymbols *members;          // all Dsymbol's in this scope
+    DsymbolTable *symtab;       // members[] sorted into table
 
-    Array *imports;		// imported ScopeDsymbol's
-    unsigned char *prots;	// PROT for each import
+    ScopeDsymbols *imports;     // imported ScopeDsymbol's
+    unsigned char *prots;       // array of PROT, one for each import
 
     ScopeDsymbol();
     ScopeDsymbol(Identifier *id);
@@ -235,11 +265,12 @@ struct ScopeDsymbol : Dsymbol
     Dsymbol *nameCollision(Dsymbol *s);
     const char *kind();
     FuncDeclaration *findGetMembers();
+    virtual Dsymbol *symtabInsert(Dsymbol *s);
 
     void emitMemberComments(Scope *sc);
 
-    static size_t dim(Array *members);
-    static Dsymbol *getNth(Array *members, size_t nth, size_t *pn = NULL);
+    static size_t dim(Dsymbols *members);
+    static Dsymbol *getNth(Dsymbols *members, size_t nth, size_t *pn = NULL);
 
     ScopeDsymbol *isScopeDsymbol() { return this; }
 };
@@ -260,9 +291,9 @@ struct WithScopeSymbol : ScopeDsymbol
 
 struct ArrayScopeSymbol : ScopeDsymbol
 {
-    Expression *exp;	// IndexExp or SliceExp
-    TypeTuple *type;	// for tuple[length]
-    TupleDeclaration *td;	// for tuples of objects
+    Expression *exp;    // IndexExp or SliceExp
+    TypeTuple *type;    // for tuple[length]
+    TupleDeclaration *td;       // for tuples of objects
     Scope *sc;
 
     ArrayScopeSymbol(Scope *sc, Expression *e);
@@ -275,21 +306,27 @@ struct ArrayScopeSymbol : ScopeDsymbol
 
 // Overload Sets
 
+#if DMDV2
 struct OverloadSet : Dsymbol
 {
-    Dsymbols a;		// array of Dsymbols
+    Dsymbols a;         // array of Dsymbols
 
     OverloadSet();
     void push(Dsymbol *s);
     OverloadSet *isOverloadSet() { return this; }
     const char *kind();
 };
+#endif
 
 // Table of Dsymbol's
 
 struct DsymbolTable : Object
 {
+#if STRINGTABLE
     StringTable *tab;
+#else
+    AA *tab;
+#endif
 
     DsymbolTable();
     ~DsymbolTable();
@@ -302,7 +339,7 @@ struct DsymbolTable : Object
 
     // Look for Dsymbol in table. If there, return it. If not, insert s and return that.
     Dsymbol *update(Dsymbol *s);
-    Dsymbol *insert(Identifier *ident, Dsymbol *s);	// when ident and s are not the same
+    Dsymbol *insert(Identifier *ident, Dsymbol *s);     // when ident and s are not the same
 };
 
 #endif /* DMD_DSYMBOL_H */
