@@ -28,16 +28,15 @@ int integral_promotion(int t)
 {
     switch (t)
     {
-	case Tchar:
-	case Twchar:
-	//case Tbit:
-	case Tbool:
-	case Tint8:
-	case Tuns8:
-	case Tint16:
-	case Tuns16:	return Tint32;
-	case Tdchar:	return Tuns32;
-	default:	return t;
+        case Tchar:
+        case Twchar:
+        case Tbool:
+        case Tint8:
+        case Tuns8:
+        case Tint16:
+        case Tuns16:    return Tint32;
+        case Tdchar:    return Tuns32;
+        default:        return t;
     }
 }
 
@@ -46,41 +45,18 @@ void init()
 
     // Set conversion tables
     for (i = 0; i < TMAX; i++)
-	for (j = 0; j < TMAX; j++)
-	{   impcnvResult[i][j] = Terror;
-	    impcnvType1[i][j] = Terror;
-	    impcnvType2[i][j] = Terror;
-	    impcnvWarn[i][j] = 0;
-	}
+        for (j = 0; j < TMAX; j++)
+        {   impcnvResult[i][j] = Terror;
+            impcnvType1[i][j] = Terror;
+            impcnvType2[i][j] = Terror;
+            impcnvWarn[i][j] = 0;
+        }
 
-#define X(t1,t2, nt1,nt2, rt)		\
-	impcnvResult[t1][t2] = rt;	\
-	impcnvType1[t1][t2] = nt1;	\
-	impcnvType2[t1][t2] = nt2;
+#define X(t1,t2, nt1,nt2, rt)           \
+        impcnvResult[t1][t2] = rt;      \
+        impcnvType1[t1][t2] = nt1;      \
+        impcnvType2[t1][t2] = nt2;
 
-    /* ======================= */
-
-#if 0
-    X(Tbit,Tbit,    Tint32,Tint32,  Tint32)
-    X(Tbit,Tint8,   Tint32,Tint32,  Tint32)
-    X(Tbit,Tuns8,   Tint32,Tint32,  Tint32)
-    X(Tbit,Tint16,  Tint32,Tint32,  Tint32)
-    X(Tbit,Tuns16,  Tint32,Tint32,  Tint32)
-    X(Tbit,Tint32,  Tint32,Tint32,  Tint32)
-    X(Tbit,Tuns32,  Tuns32,Tuns32,  Tuns32)
-    X(Tbit,Tint64,  Tint64,Tint64,  Tint64)
-    X(Tbit,Tuns64,  Tuns64,Tuns64,  Tuns64)
-
-    X(Tbit,Tfloat32,     Tfloat32,Tfloat32,     Tfloat32)
-    X(Tbit,Tfloat64,     Tfloat64,Tfloat64,     Tfloat64)
-    X(Tbit,Tfloat80,     Tfloat80,Tfloat80,     Tfloat80)
-    X(Tbit,Timaginary32, Tfloat32,Timaginary32, Tfloat32)
-    X(Tbit,Timaginary64, Tfloat64,Timaginary64, Tfloat64)
-    X(Tbit,Timaginary80, Tfloat80,Timaginary80, Tfloat80)
-    X(Tbit,Tcomplex32,   Tfloat32,Tcomplex32,   Tcomplex32)
-    X(Tbit,Tcomplex64,   Tfloat64,Tcomplex64,   Tcomplex64)
-    X(Tbit,Tcomplex80,   Tfloat80,Tcomplex80,   Tcomplex80)
-#endif
 
     /* ======================= */
 
@@ -327,18 +303,7 @@ void init()
 
 #undef X
 
-#define Y(t1,t2)	impcnvWarn[t1][t2] = 1;
-
-#if 0
-    Y(Tint8, Tbit)
-    Y(Tuns8, Tbit)
-    Y(Tint16, Tbit)
-    Y(Tuns16, Tbit)
-    Y(Tint32, Tbit)
-    Y(Tuns32, Tbit)
-    Y(Tint64, Tbit)
-    Y(Tuns64, Tbit)
-#endif
+#define Y(t1,t2)        impcnvWarn[t1][t2] = 1;
 
     Y(Tuns8, Tint8)
     Y(Tint16, Tint8)
@@ -393,19 +358,19 @@ void init()
     Y(Tint64, Tdchar)
     Y(Tuns64, Tdchar)
 
-    Y(Tint64, Tuns64)
-    Y(Tuns64, Tint64)
+//    Y(Tint64, Tuns64)
+//    Y(Tuns64, Tint64)
 
     for (i = 0; i < TMAX; i++)
-	for (j = 0; j < TMAX; j++)
-	{
-	    if (impcnvResult[i][j] == Terror)
-	    {
-		impcnvResult[i][j] = impcnvResult[j][i];
-		impcnvType1[i][j] = impcnvType2[j][i];
-		impcnvType2[i][j] = impcnvType1[j][i];
-	    }
-	}
+        for (j = 0; j < TMAX; j++)
+        {
+            if (impcnvResult[i][j] == Terror)
+            {
+                impcnvResult[i][j] = impcnvResult[j][i];
+                impcnvType1[i][j] = impcnvType2[j][i];
+                impcnvType2[i][j] = impcnvType1[j][i];
+            }
+        }
 }
 
 int main()
@@ -423,44 +388,44 @@ int main()
     fprintf(fp,"unsigned char Type::impcnvResult[TMAX][TMAX] =\n{\n");
     for (i = 0; i < TMAX; i++)
     {
-	for (j = 0; j < TMAX; j++)
-	{
-	    fprintf(fp, "%d,",impcnvResult[i][j]);
-	}
-	fprintf(fp, "\n");
+        for (j = 0; j < TMAX; j++)
+        {
+            fprintf(fp, "%d,",impcnvResult[i][j]);
+        }
+        fprintf(fp, "\n");
     }
     fprintf(fp,"};\n");
 
     fprintf(fp,"unsigned char Type::impcnvType1[TMAX][TMAX] =\n{\n");
     for (i = 0; i < TMAX; i++)
     {
-	for (j = 0; j < TMAX; j++)
-	{
-	    fprintf(fp, "%d,",impcnvType1[i][j]);
-	}
-	fprintf(fp, "\n");
+        for (j = 0; j < TMAX; j++)
+        {
+            fprintf(fp, "%d,",impcnvType1[i][j]);
+        }
+        fprintf(fp, "\n");
     }
     fprintf(fp,"};\n");
 
     fprintf(fp,"unsigned char Type::impcnvType2[TMAX][TMAX] =\n{\n");
     for (i = 0; i < TMAX; i++)
     {
-	for (j = 0; j < TMAX; j++)
-	{
-	    fprintf(fp, "%d,",impcnvType2[i][j]);
-	}
-	fprintf(fp, "\n");
+        for (j = 0; j < TMAX; j++)
+        {
+            fprintf(fp, "%d,",impcnvType2[i][j]);
+        }
+        fprintf(fp, "\n");
     }
     fprintf(fp,"};\n");
 
     fprintf(fp,"unsigned char Type::impcnvWarn[TMAX][TMAX] =\n{\n");
     for (i = 0; i < TMAX; i++)
     {
-	for (j = 0; j < TMAX; j++)
-	{
-	    fprintf(fp, "%d,",impcnvWarn[i][j]);
-	}
-	fprintf(fp, "\n");
+        for (j = 0; j < TMAX; j++)
+        {
+            fprintf(fp, "%d,",impcnvWarn[i][j]);
+        }
+        fprintf(fp, "\n");
     }
     fprintf(fp,"};\n");
 
