@@ -24,33 +24,32 @@ struct OutBuffer;
 struct Module;
 struct Package;
 struct AliasDeclaration;
-#ifdef _DH
 struct HdrGenState;
-#endif
 
 struct Import : Dsymbol
 {
-    Array *packages;		// array of Identifier's representing packages
-    Identifier *id;		// module Identifier
+    Identifiers *packages;      // array of Identifier's representing packages
+    Identifier *id;             // module Identifier
     Identifier *aliasId;
-    int isstatic;		// !=0 if static import
+    int isstatic;               // !=0 if static import
 
     // Pairs of alias=name to bind into current namespace
-    Array names;
-    Array aliases;
+    Identifiers names;
+    Identifiers aliases;
 
-    Array aliasdecls;		// AliasDeclarations for names/aliases
+    AliasDeclarations aliasdecls; // AliasDeclarations for names/aliases
 
     Module *mod;
-    Package *pkg;		// leftmost package/module
+    Package *pkg;               // leftmost package/module
 
-    Import(Loc loc, Array *packages, Identifier *id, Identifier *aliasId,
-	int isstatic);
+    Import(Loc loc, Identifiers *packages, Identifier *id, Identifier *aliasId,
+        int isstatic);
     void addAlias(Identifier *name, Identifier *alias);
 
     const char *kind();
-    Dsymbol *syntaxCopy(Dsymbol *s);	// copy only syntax trees
+    Dsymbol *syntaxCopy(Dsymbol *s);    // copy only syntax trees
     void load(Scope *sc);
+    void importAll(Scope *sc);
     void semantic(Scope *sc);
     void semantic2(Scope *sc);
     Dsymbol *toAlias();
