@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2006 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -19,17 +19,18 @@
 #include "root.h"
 
 
-class Macro
+struct Macro
 {
-    Macro *next;		// next in list
+  private:
+    Macro *next;                // next in list
 
-    unsigned char *name;	// macro name
-    size_t namelen;		// length of macro name
+    unsigned char *name;        // macro name
+    size_t namelen;             // length of macro name
 
-    unsigned char *text;	// macro replacement text
-    size_t textlen;		// length of replacement text
+    unsigned char *text;        // macro replacement text
+    size_t textlen;             // length of replacement text
 
-    int inuse;			// macro is in use (don't expand)
+    int inuse;                  // macro is in use (don't expand)
 
     Macro(unsigned char *name, size_t namelen, unsigned char *text, size_t textlen);
     Macro *search(unsigned char *name, size_t namelen);
@@ -38,7 +39,7 @@ class Macro
     static Macro *define(Macro **ptable, unsigned char *name, size_t namelen, unsigned char *text, size_t textlen);
 
     void expand(OutBuffer *buf, unsigned start, unsigned *pend,
-	unsigned char *arg, unsigned arglen);
+        unsigned char *arg, unsigned arglen);
 };
 
 #endif
