@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -32,6 +32,8 @@ struct PostBlitDeclaration;
 struct DtorDeclaration;
 struct StaticCtorDeclaration;
 struct StaticDtorDeclaration;
+struct SharedStaticCtorDeclaration;
+struct SharedStaticDtorDeclaration;
 struct ConditionalDeclaration;
 struct InvariantDeclaration;
 struct UnitTestDeclaration;
@@ -50,10 +52,10 @@ struct StaticAssert;
 
 enum ParseStatementFlags
 {
-    PSsemi = 1,		// empty ';' statements are allowed
-    PSscope = 2,	// start a new scope
-    PScurly = 4,	// { } statement is required
-    PScurlyscope = 8,	// { } starts a new scope
+    PSsemi = 1,         // empty ';' statements are allowed
+    PSscope = 2,        // start a new scope
+    PScurly = 4,        // { } statement is required
+    PScurlyscope = 8,   // { } starts a new scope
 };
 
 
@@ -61,17 +63,21 @@ struct Parser : Lexer
 {
     ModuleDeclaration *md;
     enum LINK linkage;
-    Loc endloc;			// set to location of last right curly
-    int inBrackets;		// inside [] of array index or slice
+    Loc endloc;                 // set to location of last right curly
+    int inBrackets;             // inside [] of array index or slice
+    Loc lookingForElse;         // location of lonely if looking for an else
 
     Parser(Module *module, unsigned char *base, unsigned length, int doDocComment);
 
-    Array *parseModule();
-    Array *parseDeclDefs(int once);
-    Array *parseAutoDeclarations(unsigned storageClass, unsigned char *comment);
-    Array *parseBlock();
+    Dsymbols *parseModule();
+    Dsymbols *parseDeclDefs(int once);
+    Dsymbols *parseAutoDeclarations(StorageClass storageClass, unsigned char *comment);
+    Dsymbols *parseBlock();
+    void composeStorageClass(StorageClass stc);
+    StorageClass parseAttribute();
+    StorageClass parsePostfix();
     Expression *parseConstraint();
-    TemplateDeclaration *parseTemplateDeclaration();
+    TemplateDeclaration *parseTemplateDeclaration(int ismixin);
     TemplateParameters *parseTemplateParameterList(int flag = 0);
     Dsymbol *parseMixin();
     Objects *parseTemplateArgumentList();
@@ -83,26 +89,29 @@ struct Parser : Lexer
     Condition *parseDebugCondition();
     Condition *parseVersionCondition();
     Condition *parseStaticIfCondition();
-    FuncDeclaration *parseCtor();
+    Dsymbol *parseCtor();
     PostBlitDeclaration *parsePostBlit();
     DtorDeclaration *parseDtor();
     StaticCtorDeclaration *parseStaticCtor();
     StaticDtorDeclaration *parseStaticDtor();
+    SharedStaticCtorDeclaration *parseSharedStaticCtor();
+    SharedStaticDtorDeclaration *parseSharedStaticDtor();
     InvariantDeclaration *parseInvariant();
     UnitTestDeclaration *parseUnitTest();
     NewDeclaration *parseNew();
     DeleteDeclaration *parseDelete();
-    Arguments *parseParameters(int *pvarargs);
+    Parameters *parseParameters(int *pvarargs);
     EnumDeclaration *parseEnum();
     Dsymbol *parseAggregate();
     BaseClasses *parseBaseClasses();
-    Import *parseImport(Array *decldefs, int isstatic);
+    Import *parseImport(Dsymbols *decldefs, int isstatic);
     Type *parseType(Identifier **pident = NULL, TemplateParameters **tpl = NULL);
     Type *parseBasicType();
     Type *parseBasicType2(Type *t);
-    Type *parseDeclarator(Type *t, Identifier **pident, TemplateParameters **tpl = NULL);
-    Array *parseDeclarations(unsigned storage_class);
+    Type *parseDeclarator(Type *t, Identifier **pident, TemplateParameters **tpl = NULL, StorageClass storage_class = 0);
+    Dsymbols *parseDeclarations(StorageClass storage_class, unsigned char *comment);
     void parseContracts(FuncDeclaration *f);
+    void checkDanglingElse(Loc elseloc);
     Statement *parseStatement(int flags);
     Statement *parseExtAsm(int expect_rparen);
     Initializer *parseInitializer();
@@ -110,6 +119,7 @@ struct Parser : Lexer
     void check(Loc loc, enum TOK value);
     void check(enum TOK value);
     void check(enum TOK value, const char *string);
+    void checkParens(enum TOK value, Expression *e);
     int isDeclaration(Token *t, int needId, enum TOK endtok, Token **pt);
     int isBasicType(Token **pt);
     int isDeclarator(Token **pt, int *haveId, enum TOK endtok);
@@ -117,6 +127,7 @@ struct Parser : Lexer
     int isExpression(Token **pt);
     int isTemplateInstance(Token *t, Token **pt);
     int skipParens(Token *t, Token **pt);
+    int skipAttributes(Token *t, Token **pt);
 
     Expression *parseExpression();
     Expression *parsePrimaryExp();
@@ -125,8 +136,10 @@ struct Parser : Lexer
     Expression *parseMulExp();
     Expression *parseAddExp();
     Expression *parseShiftExp();
+#if DMDV1
     Expression *parseRelExp();
     Expression *parseEqualExp();
+#endif
     Expression *parseCmpExp();
     Expression *parseAndExp();
     Expression *parseXorExp();
@@ -143,4 +156,31 @@ struct Parser : Lexer
     void addComment(Dsymbol *s, unsigned char *blockComment);
 };
 
+// Operator precedence - greater values are higher precedence
+
+enum PREC
+{
+    PREC_zero,
+    PREC_expr,
+    PREC_assign,
+    PREC_cond,
+    PREC_oror,
+    PREC_andand,
+    PREC_or,
+    PREC_xor,
+    PREC_and,
+    PREC_equal,
+    PREC_rel,
+    PREC_shift,
+    PREC_add,
+    PREC_mul,
+    PREC_pow,
+    PREC_unary,
+    PREC_primary,
+};
+
+extern enum PREC precedence[TOKMAX];
+
+void initPrecedence();
+
 #endif /* DMD_PARSE_H */
