@@ -1,9 +1,9 @@
 
 
-// Copyright (c) 1999-2006 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
-// www.digitalmars.com
+// http://www.digitalmars.com
 // License for redistribution is by either the Artistic License
 // in artistic.txt, or the GNU General Public License in gnu.txt.
 // See the included readme.txt for details.
@@ -19,6 +19,9 @@
 
 #include <stdlib.h>
 #include <stdarg.h>
+#ifdef DEBUG
+#include <assert.h>
+#endif
 
 #if __DMC__
 #pragma once
@@ -33,6 +36,9 @@ int wcharIsAscii(wchar_t *);
 char *wchar2ascii(wchar_t *, unsigned len);
 int wcharIsAscii(wchar_t *, unsigned len);
 
+#ifdef IN_GCC
+int binary(const char *p , const char **tab, int high);
+#endif
 int bstrcmp(unsigned char *s1, unsigned char *s2);
 char *bstr2str(unsigned char *b);
 #ifndef GCC_SAFE_DMD
@@ -45,6 +51,13 @@ void warning(const char *format, ...);
 #define TYPEDEFS
 
 #if _MSC_VER
+#include <float.h>  // for _isnan
+#include <malloc.h> // for alloca
+// According to VC 8.0 docs, long double is the same as double
+#define strtold strtod
+#define strtof  strtod
+#define isnan   _isnan
+
 typedef __int64 longlong;
 typedef unsigned __int64 ulonglong;
 #else
@@ -61,7 +74,12 @@ longlong randomx();
  */
 
 struct OutBuffer;
-struct Array;
+
+// Can't include arraytypes.h here, need to declare these directly.
+template <typename TYPE> struct ArrayBase;
+typedef ArrayBase<struct File> Files;
+typedef ArrayBase<char> Strings;
+
 
 struct Object
 {
@@ -99,14 +117,14 @@ struct Object
     /**
      * Marks pointers for garbage collector by calling mem.mark() for all pointers into heap.
      */
-    /*virtual*/		// not used, disable for now
-	void mark();
+    /*virtual*/         // not used, disable for now
+        void mark();
 };
 
 struct String : Object
 {
-    int ref;			// != 0 if this is a reference to someone else's string
-    char *str;			// the string itself
+    int ref;                    // != 0 if this is a reference to someone else's string
+    char *str;                  // the string itself
 
     String(char *str, int ref = 1);
 
@@ -129,7 +147,9 @@ struct FileName : String
     FileName(char *path, char *name);
     hash_t hashCode();
     int equals(Object *obj);
+    static int equals(const char *name1, const char *name2);
     int compare(Object *obj);
+    static int compare(const char *name1, const char *name2);
     static int absolute(const char *name);
     static char *ext(const char *);
     char *ext();
@@ -137,28 +157,30 @@ struct FileName : String
     static char *name(const char *);
     char *name();
     static char *path(const char *);
-    static char *replaceName(char *path, char *name);
+    static const char *replaceName(const char *path, const char *name);
 
     static char *combine(const char *path, const char *name);
-    static Array *splitPath(const char *path);
+    static Strings *splitPath(const char *path);
     static FileName *defaultExt(const char *name, const char *ext);
     static FileName *forceExt(const char *name, const char *ext);
     int equalsExt(const char *ext);
 
     void CopyTo(FileName *to);
-    static char *searchPath(Array *path, const char *name, int cwd);
+    static char *searchPath(Strings *path, const char *name, int cwd);
+    static char *safeSearchPath(Strings *path, const char *name);
     static int exists(const char *name);
     static void ensurePathExists(const char *path);
+    static char *canonicalName(const char *name);
 };
 
 struct File : Object
 {
-    int ref;			// != 0 if this is a reference to someone else's buffer
-    unsigned char *buffer;	// data for our file
-    unsigned len;		// amount of data in buffer[]
-    void *touchtime;		// system time to use for file
+    int ref;                    // != 0 if this is a reference to someone else's buffer
+    unsigned char *buffer;      // data for our file
+    unsigned len;               // amount of data in buffer[]
+    void *touchtime;            // system time to use for file
 
-    FileName *name;		// name of our file
+    FileName *name;             // name of our file
 
     File(char *);
     File(FileName *);
@@ -202,9 +224,9 @@ struct File : Object
     void writev();
 
     /* Return !=0 if file exists.
-     *	0:	file doesn't exist
-     *	1:	normal file
-     *	2:	directory
+     *  0:      file doesn't exist
+     *  1:      normal file
+     *  2:      directory
      */
 
     /* Append to file, return !=0 if error
@@ -219,9 +241,9 @@ struct File : Object
     void appendv();
 
     /* Return !=0 if file exists.
-     *	0:	file doesn't exist
-     *	1:	normal file
-     *	2:	directory
+     *  0:      file doesn't exist
+     *  1:      normal file
+     *  2:      directory
      */
 
     int exists();
@@ -230,13 +252,13 @@ struct File : Object
      * matching File's.
      */
 
-    static Array *match(char *);
-    static Array *match(FileName *);
+    static Files *match(char *);
+    static Files *match(FileName *);
 
     // Compare file times.
-    // Return	<0	this < f
-    //		=0	this == f
-    //		>0	this > f
+    // Return   <0      this < f
+    //          =0      this == f
+    //          >0      this > f
     int compareTime(File *f);
 
     // Read system file statistics
@@ -247,13 +269,13 @@ struct File : Object
 
     void setbuffer(void *buffer, unsigned len)
     {
-	this->buffer = (unsigned char *)buffer;
-	this->len = len;
+        this->buffer = (unsigned char *)buffer;
+        this->len = len;
     }
 
     void checkoffset(size_t offset, size_t nbytes);
 
-    void remove();		// delete file
+    void remove();              // delete file
 };
 
 struct OutBuffer : Object
@@ -264,7 +286,7 @@ struct OutBuffer : Object
 
     OutBuffer();
     ~OutBuffer();
-    void *extractData();
+    char *extractData();
     void mark();
 
     void reserve(unsigned nbytes);
@@ -276,7 +298,7 @@ struct OutBuffer : Object
     void writedstring(const char *string);
     void writedstring(const wchar_t *string);
     void prependstring(const char *string);
-    void writenl();			// write newline
+    void writenl();                     // write newline
     void writeByte(unsigned b);
     void writebyte(unsigned b) { writeByte(b); }
     void writeUTF8(unsigned b);
@@ -307,11 +329,17 @@ struct OutBuffer : Object
 struct Array : Object
 {
     unsigned dim;
-    unsigned allocdim;
     void **data;
 
+  private:
+    unsigned allocdim;
+    #define SMALLARRAYCAP       1
+    void *smallarray[SMALLARRAYCAP];    // inline storage for small arrays
+
+  public:
     Array();
     ~Array();
+    //Array(const Array&);
     void mark();
     char *toChars();
 
@@ -331,6 +359,48 @@ struct Array : Object
     Array *copy();
 };
 
+template <typename TYPE>
+struct ArrayBase : Array
+{
+    TYPE **tdata()
+    {
+        return (TYPE **)data;
+    }
+
+    TYPE*& operator[] (size_t index)
+    {
+#ifdef DEBUG
+        assert(index < dim);
+#endif
+        return ((TYPE **)data)[index];
+    }
+
+    void insert(size_t index, TYPE *v)
+    {
+        Array::insert(index, (void *)v);
+    }
+
+    void insert(size_t index, ArrayBase *a)
+    {
+        Array::insert(index, (Array *)a);
+    }
+
+    void append(ArrayBase *a)
+    {
+        Array::append((Array *)a);
+    }
+
+    void push(TYPE *a)
+    {
+        Array::push((void *)a);
+    }
+
+    ArrayBase *copy()
+    {
+        return (ArrayBase *)Array::copy();
+    }
+};
+
 struct Bits : Object
 {
     unsigned bitdim;
