@@ -1,6 +1,6 @@
 
 // Compiler implementation of the D programming language
-// Copyright (c) 1999-2008 by Digital Mars
+// Copyright (c) 1999-2011 by Digital Mars
 // All Rights Reserved
 // written by Walter Bright
 // http://www.digitalmars.com
@@ -19,6 +19,7 @@
 
 #include "arraytypes.h"
 #include "dsymbol.h"
+#include "lexer.h"
 
 struct OutBuffer;
 struct Scope;
@@ -26,7 +27,7 @@ struct Expression;
 struct LabelDsymbol;
 struct Identifier;
 struct IfStatement;
-struct DeclarationStatement;
+struct ExpStatement;
 struct DefaultStatement;
 struct VarDeclaration;
 struct Condition;
@@ -37,13 +38,16 @@ struct InlineDoState;
 struct InlineScanState;
 struct ReturnStatement;
 struct CompoundStatement;
-struct Argument;
+struct Parameter;
 struct StaticAssert;
 struct AsmStatement;
 struct GotoStatement;
 struct ScopeStatement;
 struct TryCatchStatement;
 struct TryFinallyStatement;
+struct CaseStatement;
+struct DefaultStatement;
+struct LabelStatement;
 struct HdrGenState;
 struct InterState;
 
@@ -61,17 +65,17 @@ struct elem;
 #endif
 struct code;
 
-/* How a statement exits
+/* How a statement exits; this is returned by blockExit()
  */
 enum BE
 {
-    BEnone =	 0,
+    BEnone =     0,
     BEfallthru = 1,
     BEthrow =    2,
     BEreturn =   4,
     BEgoto =     8,
-    BEhalt =	 0x10,
-    BEbreak =	 0x20,
+    BEhalt =     0x10,
+    BEbreak =    0x20,
     BEcontinue = 0x40,
     BEany = (BEfallthru | BEthrow | BEreturn | BEgoto | BEhalt),
 };
@@ -87,25 +91,23 @@ struct Statement : Object
     char *toChars();
 
     void error(const char *format, ...);
+    void warning(const char *format, ...);
     virtual void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    virtual TryCatchStatement *isTryCatchStatement() { return NULL; }
-    virtual GotoStatement *isGotoStatement() { return NULL; }
-    virtual AsmStatement *isAsmStatement() { return NULL; }
-#ifdef _DH
     int incontract;
-#endif
     virtual ScopeStatement *isScopeStatement() { return NULL; }
     virtual Statement *semantic(Scope *sc);
     Statement *semanticScope(Scope *sc, Statement *sbreak, Statement *scontinue);
+    Statement *semanticNoScope(Scope *sc);
     virtual int hasBreak();
     virtual int hasContinue();
     virtual int usesEH();
-    virtual int fallOffEnd();
-    virtual int blockExit();
+    virtual int blockExit(bool mustNotThrow);
     virtual int comeFrom();
-    virtual void scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
+    virtual int isEmpty();
+    virtual Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
     virtual Statements *flatten(Scope *sc);
     virtual Expression *interpret(InterState *istate);
+    virtual Statement *last();
 
     virtual int inlineCost(InlineCostState *ics);
     virtual Expression *doInline(InlineDoState *ids);
@@ -115,10 +117,21 @@ struct Statement : Object
     virtual void toIR(IRState *irs);
 
     // Avoid dynamic_cast
-    virtual DeclarationStatement *isDeclarationStatement() { return NULL; }
+    virtual ExpStatement *isExpStatement() { return NULL; }
     virtual CompoundStatement *isCompoundStatement() { return NULL; }
     virtual ReturnStatement *isReturnStatement() { return NULL; }
     virtual IfStatement *isIfStatement() { return NULL; }
+    virtual CaseStatement *isCaseStatement() { return NULL; }
+    virtual DefaultStatement *isDefaultStatement() { return NULL; }
+    virtual LabelStatement *isLabelStatement() { return NULL; }
+};
+
+struct PeelStatement : Statement
+{
+    Statement *s;
+
+    PeelStatement(Statement *s);
+    Statement *semantic(Scope *sc);
 };
 
 struct ExpStatement : Statement
@@ -126,18 +139,34 @@ struct ExpStatement : Statement
     Expression *exp;
 
     ExpStatement(Loc loc, Expression *exp);
+    ExpStatement(Loc loc, Dsymbol *s);
     Statement *syntaxCopy();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
-    int fallOffEnd();
-    int blockExit();
+    int blockExit(bool mustNotThrow);
+    int isEmpty();
+    Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
 
     int inlineCost(InlineCostState *ics);
     Expression *doInline(InlineDoState *ids);
     Statement *inlineScan(InlineScanState *iss);
 
     void toIR(IRState *irs);
+
+    ExpStatement *isExpStatement() { return this; }
+};
+
+struct DtorExpStatement : ExpStatement
+{
+    /* Wraps an expression that is the destruction of 'var'
+     */
+
+    VarDeclaration *var;
+
+    DtorExpStatement(Loc loc, Expression *exp, VarDeclaration *v);
+    Statement *syntaxCopy();
+    void toIR(IRState *irs);
 };
 
 struct CompileStatement : Statement
@@ -151,36 +180,24 @@ struct CompileStatement : Statement
     Statement *semantic(Scope *sc);
 };
 
-struct DeclarationStatement : ExpStatement
-{
-    // Doing declarations as an expression, rather than a statement,
-    // makes inlining functions much easier.
-
-    DeclarationStatement(Loc loc, Dsymbol *s);
-    DeclarationStatement(Loc loc, Expression *exp);
-    Statement *syntaxCopy();
-    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    void scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
-
-    DeclarationStatement *isDeclarationStatement() { return this; }
-};
-
 struct CompoundStatement : Statement
 {
     Statements *statements;
 
     CompoundStatement(Loc loc, Statements *s);
+    CompoundStatement(Loc loc, Statement *s1);
     CompoundStatement(Loc loc, Statement *s1, Statement *s2);
     Statement *syntaxCopy();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Statement *semantic(Scope *sc);
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
+    int isEmpty();
     Statements *flatten(Scope *sc);
     ReturnStatement *isReturnStatement();
     Expression *interpret(InterState *istate);
+    Statement *last();
 
     int inlineCost(InlineCostState *ics);
     Expression *doInline(InlineDoState *ids);
@@ -191,6 +208,13 @@ struct CompoundStatement : Statement
     CompoundStatement *isCompoundStatement() { return this; }
 };
 
+struct CompoundDeclarationStatement : CompoundStatement
+{
+    CompoundDeclarationStatement(Loc loc, Statements *s);
+    Statement *syntaxCopy();
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+};
+
 /* The purpose of this is so that continue will go to the next
  * of the statements, and break will go to the end of the statements.
  */
@@ -204,8 +228,7 @@ struct UnrolledLoopStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -229,9 +252,9 @@ struct ScopeStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
+    int isEmpty();
     Expression *interpret(InterState *istate);
 
     Statement *inlineScan(InlineScanState *iss);
@@ -250,8 +273,7 @@ struct WhileStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -272,8 +294,7 @@ struct DoStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -293,12 +314,11 @@ struct ForStatement : Statement
     ForStatement(Loc loc, Statement *init, Expression *condition, Expression *increment, Statement *body);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
-    void scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
+    Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -310,27 +330,27 @@ struct ForStatement : Statement
 
 struct ForeachStatement : Statement
 {
-    enum TOK op;		// TOKforeach or TOKforeach_reverse
-    Arguments *arguments;	// array of Argument*'s
+    enum TOK op;                // TOKforeach or TOKforeach_reverse
+    Parameters *arguments;      // array of Parameter*'s
     Expression *aggr;
     Statement *body;
 
     VarDeclaration *key;
     VarDeclaration *value;
 
-    FuncDeclaration *func;	// function we're lexically in
+    FuncDeclaration *func;      // function we're lexically in
 
-    Array cases;	// put breaks, continues, gotos and returns here
-    Array gotos;	// forward referenced goto's go here
+    Statements *cases;          // put breaks, continues, gotos and returns here
+    CompoundStatements *gotos;  // forward referenced goto's go here
 
-    ForeachStatement(Loc loc, enum TOK op, Arguments *arguments, Expression *aggr, Statement *body);
+    ForeachStatement(Loc loc, enum TOK op, Parameters *arguments, Expression *aggr, Statement *body);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
+    bool checkForArgTypes();
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -340,26 +360,25 @@ struct ForeachStatement : Statement
     void toIR(IRState *irs);
 };
 
-#if V2
+#if DMDV2
 struct ForeachRangeStatement : Statement
 {
-    enum TOK op;		// TOKforeach or TOKforeach_reverse
-    Argument *arg;		// loop index variable
+    enum TOK op;                // TOKforeach or TOKforeach_reverse
+    Parameter *arg;             // loop index variable
     Expression *lwr;
     Expression *upr;
     Statement *body;
 
     VarDeclaration *key;
 
-    ForeachRangeStatement(Loc loc, enum TOK op, Argument *arg,
-	Expression *lwr, Expression *upr, Statement *body);
+    ForeachRangeStatement(Loc loc, enum TOK op, Parameter *arg,
+        Expression *lwr, Expression *upr, Statement *body);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
@@ -372,21 +391,20 @@ struct ForeachRangeStatement : Statement
 
 struct IfStatement : Statement
 {
-    Argument *arg;
+    Parameter *arg;
     Expression *condition;
     Statement *ifbody;
     Statement *elsebody;
 
-    VarDeclaration *match;	// for MatchExpression results
+    VarDeclaration *match;      // for MatchExpression results
 
-    IfStatement(Loc loc, Argument *arg, Expression *condition, Statement *ifbody, Statement *elsebody);
+    IfStatement(Loc loc, Parameter *arg, Expression *condition, Statement *ifbody, Statement *elsebody);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     IfStatement *isIfStatement() { return this; }
 
     int inlineCost(InlineCostState *ics);
@@ -407,7 +425,7 @@ struct ConditionalStatement : Statement
     Statement *semantic(Scope *sc);
     Statements *flatten(Scope *sc);
     int usesEH();
-    int blockExit();
+    int blockExit(bool mustNotThrow);
 
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
@@ -415,15 +433,14 @@ struct ConditionalStatement : Statement
 struct PragmaStatement : Statement
 {
     Identifier *ident;
-    Expressions *args;		// array of Expression's
+    Expressions *args;          // array of Expression's
     Statement *body;
 
     PragmaStatement(Loc loc, Identifier *ident, Expressions *args, Statement *body);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
 
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
@@ -437,6 +454,7 @@ struct StaticAssertStatement : Statement
     StaticAssertStatement(StaticAssert *sa);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
+    int blockExit(bool mustNotThrow);
 
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
@@ -445,21 +463,21 @@ struct SwitchStatement : Statement
 {
     Expression *condition;
     Statement *body;
+    bool isFinal;
 
     DefaultStatement *sdefault;
     TryFinallyStatement *tf;
-    Array gotoCases;		// array of unresolved GotoCaseStatement's
-    Array *cases;		// array of CaseStatement's
-    int hasNoDefault;		// !=0 if no default statement
-    int hasVars;		// !=0 if has variable case values
+    GotoCaseStatements gotoCases;  // array of unresolved GotoCaseStatement's
+    CaseStatements *cases;         // array of CaseStatement's
+    int hasNoDefault;           // !=0 if no default statement
+    int hasVars;                // !=0 if has variable case values
 
-    SwitchStatement(Loc loc, Expression *c, Statement *b);
+    SwitchStatement(Loc loc, Expression *c, Statement *b, bool isFinal);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     int hasBreak();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
@@ -472,41 +490,58 @@ struct CaseStatement : Statement
 {
     Expression *exp;
     Statement *statement;
-    int index;		// which case it is (since we sort this)
-    block *cblock;	// back end: label for the block
+
+    int index;          // which case it is (since we sort this)
+    block *cblock;      // back end: label for the block
 
     CaseStatement(Loc loc, Expression *exp, Statement *s);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     int compare(Object *obj);
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    CaseStatement *isCaseStatement() { return this; }
 
     Statement *inlineScan(InlineScanState *iss);
 
     void toIR(IRState *irs);
 };
 
+#if DMDV2
+
+struct CaseRangeStatement : Statement
+{
+    Expression *first;
+    Expression *last;
+    Statement *statement;
+
+    CaseRangeStatement(Loc loc, Expression *first, Expression *last, Statement *s);
+    Statement *syntaxCopy();
+    Statement *semantic(Scope *sc);
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+};
+
+#endif
+
 struct DefaultStatement : Statement
 {
     Statement *statement;
 #if IN_GCC
-    block *cblock;	// back end: label for the block
+    block *cblock;      // back end: label for the block
 #endif
 
     DefaultStatement(Loc loc, Statement *s);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+    DefaultStatement *isDefaultStatement() { return this; }
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -521,8 +556,7 @@ struct GotoDefaultStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     void toIR(IRState *irs);
@@ -530,15 +564,14 @@ struct GotoDefaultStatement : Statement
 
 struct GotoCaseStatement : Statement
 {
-    Expression *exp;		// NULL, or which case to goto
-    CaseStatement *cs;		// case statement it resolves to
+    Expression *exp;            // NULL, or which case to goto
+    CaseStatement *cs;          // case statement it resolves to
 
     GotoCaseStatement(Loc loc, Expression *exp);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     void toIR(IRState *irs);
@@ -547,8 +580,7 @@ struct GotoCaseStatement : Statement
 struct SwitchErrorStatement : Statement
 {
     SwitchErrorStatement(Loc loc);
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     void toIR(IRState *irs);
@@ -562,8 +594,7 @@ struct ReturnStatement : Statement
     Statement *syntaxCopy();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Statement *semantic(Scope *sc);
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     Expression *interpret(InterState *istate);
 
     int inlineCost(InlineCostState *ics);
@@ -583,8 +614,7 @@ struct BreakStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     void toIR(IRState *irs);
@@ -598,8 +628,7 @@ struct ContinueStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Expression *interpret(InterState *istate);
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     void toIR(IRState *irs);
@@ -616,8 +645,7 @@ struct SynchronizedStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     Statement *inlineScan(InlineScanState *iss);
@@ -639,8 +667,8 @@ struct WithStatement : Statement
     Statement *semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
+    Expression *interpret(InterState *istate);
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -650,21 +678,20 @@ struct WithStatement : Statement
 struct TryCatchStatement : Statement
 {
     Statement *body;
-    Array *catches;
+    Catches *catches;
 
-    TryCatchStatement(Loc loc, Statement *body, Array *catches);
+    TryCatchStatement(Loc loc, Statement *body, Catches *catches);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     int hasBreak();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
+    Expression *interpret(InterState *istate);
 
     Statement *inlineScan(InlineScanState *iss);
 
     void toIR(IRState *irs);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    TryCatchStatement *isTryCatchStatement() { return this; }
 };
 
 struct Catch : Object
@@ -678,7 +705,7 @@ struct Catch : Object
     Catch(Loc loc, Type *t, Identifier *id, Statement *handler);
     Catch *syntaxCopy();
     void semantic(Scope *sc);
-    int blockExit();
+    int blockExit(bool mustNotThrow);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
 
@@ -694,8 +721,8 @@ struct TryFinallyStatement : Statement
     int hasBreak();
     int hasContinue();
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
+    Expression *interpret(InterState *istate);
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -709,11 +736,12 @@ struct OnScopeStatement : Statement
 
     OnScopeStatement(Loc loc, TOK tok, Statement *statement);
     Statement *syntaxCopy();
-    int blockExit();
+    int blockExit(bool mustNotThrow);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
     Statement *semantic(Scope *sc);
     int usesEH();
-    void scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
+    Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
+    Expression *interpret(InterState *istate);
 
     void toIR(IRState *irs);
 };
@@ -726,8 +754,8 @@ struct ThrowStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
+    Expression *interpret(InterState *istate);
 
     Statement *inlineScan(InlineScanState *iss);
 
@@ -742,8 +770,7 @@ struct VolatileStatement : Statement
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Statements *flatten(Scope *sc);
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     Statement *inlineScan(InlineScanState *iss);
@@ -751,6 +778,17 @@ struct VolatileStatement : Statement
     void toIR(IRState *irs);
 };
 
+struct DebugStatement : Statement
+{
+    Statement *statement;
+
+    DebugStatement(Loc loc, Statement *statement);
+    Statement *syntaxCopy();
+    Statement *semantic(Scope *sc);
+    Statements *flatten(Scope *sc);
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+};
+
 struct GotoStatement : Statement
 {
     Identifier *ident;
@@ -760,13 +798,11 @@ struct GotoStatement : Statement
     GotoStatement(Loc loc, Identifier *ident);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     Expression *interpret(InterState *istate);
 
     void toIR(IRState *irs);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    GotoStatement *isGotoStatement() { return this; }
 };
 
 struct LabelStatement : Statement
@@ -774,21 +810,22 @@ struct LabelStatement : Statement
     Identifier *ident;
     Statement *statement;
     TryFinallyStatement *tf;
-    block *lblock;		// back end
-    int isReturnLabel;
+    block *lblock;              // back end
+
+    Array *fwdrefs;            // forward references to this LabelStatement
 
     LabelStatement(Loc loc, Identifier *ident, Statement *statement);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
     Statements *flatten(Scope *sc);
     int usesEH();
-    int blockExit();
-    int fallOffEnd();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
     Expression *interpret(InterState *istate);
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 
     Statement *inlineScan(InlineScanState *iss);
+    LabelStatement *isLabelStatement() { return this; }
 
     void toIR(IRState *irs);
 };
@@ -808,19 +845,38 @@ struct AsmStatement : Statement
 {
     Token *tokens;
     code *asmcode;
-    unsigned asmalign;		// alignment of this statement
-    unsigned refparam;		// !=0 if function parameter is referenced
-    unsigned naked;		// !=0 if function is to be naked
-    unsigned regs;		// mask of registers modified
+    unsigned asmalign;          // alignment of this statement
+    unsigned refparam;          // !=0 if function parameter is referenced
+    unsigned naked;             // !=0 if function is to be naked
+    unsigned regs;              // mask of registers modified
 
     AsmStatement(Loc loc, Token *tokens);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
-    int blockExit();
+    int blockExit(bool mustNotThrow);
     int comeFrom();
+    Expression *interpret(InterState *istate);
 
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
-    virtual AsmStatement *isAsmStatement() { return this; }
+
+    void toIR(IRState *irs);
+};
+
+struct ImportStatement : Statement
+{
+    Dsymbols *imports;          // Array of Import's
+
+    ImportStatement(Loc loc, Dsymbols *imports);
+    Statement *syntaxCopy();
+    Statement *semantic(Scope *sc);
+    int blockExit(bool mustNotThrow);
+    int isEmpty();
+    Expression *interpret(InterState *istate);
+
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+
+    int inlineCost(InlineCostState *ics);
+    Expression *doInline(InlineDoState *ids);
 
     void toIR(IRState *irs);
 };
@@ -832,16 +888,18 @@ struct ExtAsmStatement : Statement
 {
     Expression *insnTemplate;
     Expressions *args;            
-    Array *argNames;        // of NULL or Identifier*
+    Identifiers *argNames;        // of NULL or Identifier*
     Expressions *argConstraints;  // of StringExp*
     unsigned nOutputArgs;
     Expressions *clobbers;        // of StringExp*
 
-    ExtAsmStatement(Loc loc, Expression *insnTemplate, Expressions *args, Array *argNames,
-	Expressions *argConstraints, int nOutputArgs, Expressions *clobbers);
+    ExtAsmStatement(Loc loc, Expression *insnTemplate, Expressions *args, Identifiers *argNames,
+                    Expressions *argConstraints, int nOutputArgs, Expressions *clobbers);
     Statement *syntaxCopy();
     Statement *semantic(Scope *sc);
-    int blockExit();
+    int blockExit(bool mustNotThrow);
+    void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
+
     void toIR(IRState *irs);
 };
 
