@@ -18,9 +18,7 @@
 #include "dsymbol.h"
 
 struct Expression;
-#ifdef _DH
 struct HdrGenState;
-#endif
 
 struct StaticAssert : Dsymbol
 {
@@ -36,7 +34,7 @@ struct StaticAssert : Dsymbol
     void inlineScan();
     int oneMember(Dsymbol **ps);
     void toObjFile(int multiobj);
-    char *kind();
+    const char *kind();
     void toCBuffer(OutBuffer *buf, HdrGenState *hgs);
 };
 
