@@ -1,16 +1,16 @@
 /* GDC -- D front-end for GCC
    Copyright (C) 2004 David Friedman
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
- 
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
- 
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -49,8 +49,6 @@ enum TypeType
     mTYvolatile = 0x2000
 };
 
-typedef int tym_t;
-
 enum TypeFlag
 {
     TFsizeunknown = 0x01,
@@ -59,30 +57,6 @@ enum TypeFlag
     TFfixed = 0x08
 };
 
-/*
-struct TYPE : Object
-{
-    tym_t        Tty;
-    mangle_t     Tmangle;
-    TYPE *       Tnext;
-    int          Tcount;
-    int          Tflags;
-    void *       Ttag;
-    void *       Tparamtypes;
-    tree         Ttree;
-
-    TYPE();
-};
-
-extern TYPE * tsvoid;
-
-extern TYPE * type_alloc(tym_t ty); // ty: combination of TypeType values
-extern TYPE * type_fake(tym_t ty);
-extern void   type_setcv(TYPE ** pp_type, int ty);
-extern void   type_setmangle(TYPE ** pp_type, mangle_t mangle);
-extern int/ *?* / type_paramsize(TYPE * t);
-*/
-
 enum SymbolStorageClass
 {
     SC_INVALID,
@@ -114,6 +88,7 @@ enum SymbolFlag
 
 enum SymbolSegment
 {
+    INVALID,
     DATA,
     CDATA,
     UDATA
@@ -131,36 +106,29 @@ enum OutputStage
 };
 
 struct FuncFrameInfo;
+typedef ArrayBase<struct Thunk> Thunks;
 
 struct Symbol : Object
 {
     Symbol();
 
-    char *Sident;
+    const char *Sident;
+    const char *prettyIdent;
     //unused in GCC//TYPE *Stype; // maybe type/TYPE ?
     SymbolStorageClass Sclass;
     SymbolFL           Sfl;
     SymbolSegment      Sseg;
     int                Sflags;
 
-    //unused in GCC//int Ssymnum;
-
     dt_t * Sdt;
 
-    // fake classsym....
-    //unused in GCC//Symbol * Sstruct;
-    //unused in GCC//int      Sstructalign;
-
     // Specific to GNU backend
     tree     Stree;
     tree     ScontextDecl; // The DECL_CONTEXT to use for child declarations, but see IRState::declContext
-    unsigned Sunique;      // For conflicting symbol names
-#if V2
-    tree     SclosureField; // FIELD_DECL in closure frame struct that this variable is allocated in -- Eventually move back into Stree once everything works right
-#endif
+    tree     SframeField;  // FIELD_DECL in frame struct that this variable is allocated in -- Eventually move back into Stree once everything works right
 
     // For FuncDeclarations:
-    Array * thunks; // of struct Thunk
+    Thunks * thunks;
     FuncDeclarations * otherNestedFuncs;
     OutputStage outputStage;
     FuncFrameInfo *frameInfo;
@@ -168,7 +136,7 @@ struct Symbol : Object
 
 struct Thunk
 {
-    target_ptrdiff_t offset;
+    int offset;
     Symbol * symbol;
     Thunk();
 };
@@ -178,13 +146,15 @@ extern Symbol * symbol_name(const char * id, int sclass, TYPE * t);
 extern Symbol * struct_calloc();
 extern Symbol * symbol_generate(SymbolStorageClass sc, TYPE * type);
 extern void     symbol_func(Symbol * sym);
+extern tree     check_static_sym(Symbol * sym);
 extern void     outdata(Symbol * sym);
 inline void     obj_export(Symbol *, int) { }
 extern void     obj_moduleinfo(Symbol *sym);
+extern void     obj_tlssections();
 
 extern Symbol * symbol_tree(tree);
 extern Symbol * static_sym();
 
-extern void	slist_add(Symbol *);
-extern void	slist_reset();
+extern void     slist_add(Symbol *);
+extern void     slist_reset();
 #endif
